// init fast click
$(function () {
  FastClick.attach(document.body);
});

// 整理一些常用函数
(function (window) {
  var cbID = 0;

  // 升权之后 要更新之前请求参数的 write_access_token
  function refreshRequestParamsToken (params) {
    var data = params.data;
    if (data && data.aid && data.write_access_token) {
      xqBridge.getWriteToken({
        aid: data.aid,
        success: function (token) {
          params.data.write_access_token = token;
          callBrick(params);
        }
      });
    }
  }

  // 把回调函数绑定到window上
  function changeCallback(params) {
    var successCb = params.success;
    // token 过期 拦截success
    if (params.name === "request") {
      function processExpireToken(ret) {
        // 70005 过期 重新输入密码
        // 70006 券商服务器重启 数据丢失 重绑
        if (ret && (ret.result_code == 70005 || ret.result_code == 70006)) {
          if (Util.appVersion() >= 6.5 && ret.result_data && ret.result_data.page_type && ret.result_data.page_type == 2) {
            ret.result_data.tid = ret.result_data.oauth_params.tid;
            if (!window.hasRedirect) {
              window.hasRedirect = true;

              xqBridge.refreshBrokerToken({
                data: ret,
                success: function (obj) {
                  // 拿到token之后用新token 去请求
                  if (params.name === "request") {
                    window.hasRedirect = false;
                    refreshRequestParamsToken(params);
                  }
                },
                cancel: function () {
                  window.hasRedirect = false;
                  if (params.error && window[params.error]) {
                    window[params.error](ret);
                  }
                }
              });
            }
          } else {
            var url = ret.refresh_url || ret.rebind_url;

            if (url) {
              if (!window.hasRedirect) {
                window.hasRedirect = true;

                xqBridge.showLoading();
                xqBridge.redirect({
                  url: "/broker/proxy?url=" + encodeURIComponent(url),
                  type: "MODAL",
                  success: function () {
                    if (params.name === "request") {
                      window.hasRedirect = false;
                      refreshRequestParamsToken(params);
                    }
                  },
                  cancel: function () {
                    window.hasRedirect = false;
                    if (params.error && window[params.error]) {
                      window[params.error](ret);
                    }
                  }
                });
              }
            }
          }
        } else if (ret && ret.result_code == 70008) {
          // 手机号码没有验证
          xqBridge.verifyTelephone({
            type: "bind",
            success: function () {
              if (params.name === "request") {
                callBrick(params);
              }
            },
            cancel: function () {
              if (params.error && window[params.error]) {
                window[params.error](ret);
              }
            }
          });
        } else {
          if (successCb) {
            if (_.isFunction(successCb)) {
              successCb(ret)
            } else {
              window[successCb] && window[successCb](ret);
            }
          }
        }
      }

      // 如果success 是函数 不是cb1  或者未定义
      if (_.isFunction(successCb) || _.isUndefined(successCb)) {
        if (params.success) {
          params.success = function (ret) {
            //successCb(ret);
            processExpireToken(ret);
          }
        } else {
          params.success = function (ret) {
            processExpireToken(ret);
          }
        }
      }
    }

    var cbs = ["success", "cancel", "error", "complete", "action"]; // 可能有的回调函数
    _.each(cbs, function (cb) {
      if (params[cb] && _.isFunction(params[cb])) {
        var cbName = "cb" + (++cbID),
          realCb = params[cb];

        params[cb] = cbName;
        window[cbName] = function (data) {
          if (typeof data === "string") {
            data = decodeURIComponent(data);

            if (/^\{.*\}$/.test(data) || (/^\[.*\]$/).test(data)) {
              data = $.parseJSON(data);
            }
          }
          realCb(data);

          // setToolbar 的button params 没有name 属性  并且事件是可以多次点击  不用删除
          if (cb !== "action") {
            console.log("delete " + cbName);
            //delete window[cbName];
          }
        }
      }
    });
  }

  function callBrick(params) {

    if (_.isObject(params)) {

      // setToolbar 需要特殊处理一下 他的action 在buttons里面
      if (params.name === "setToolbar" && params.buttons && params.buttons.length) {
        if (!window.isSetToolbar) {
          // set toolbar  只绑定一次
          window.isSetToolbar = true;
          _.each(params.buttons, function (button) {
            changeCallback(button);
          });
        } else {
          return false;
        }
      } else if (params.name === "setRightNavigationButton") {
        // 也只绑一次
        if (!window.isSetRightNavigationButton) {
          window.isSetRightNavigationButton = true;
          changeCallback(params)
        } else {
          return false;
        }
      } else {
        changeCallback(params)
      }
    }

    var iframe = document.createElement('iframe'),
      parameters = _.isObject(params) ? JSON.stringify(params) : params;

    console.log(parameters);
    iframe.setAttribute('src', 'js://' + encodeURIComponent(parameters));
    document.documentElement.appendChild(iframe);
    iframe.parentNode.removeChild(iframe);
    iframe = null;
  }

  /*  // 使用说明
   xqBridge.confirm({
   title: "确认取消关注？",
   // 成功回调
   success: function () {
   alert("success");
   },
   // 取消回调
   cancel: function () {
   alert("cancel");
   },
   // 错误回调
   error: function () {
   alert("error");
   }
   });*/

  // 事件
  window.eventObj = {};

  window.xqBridge = {
    /**
     * 显示loading view
     */
    showLoading: function () {
      callBrick("showloading");
    },

    /**
     * 隐藏loading view
     */
    hideLoading: function () {
      callBrick("hideloading");
    },

    /**
     * alert view
     * @param options
     */
    alert: function (options) {
      if (options && _.isObject(options)) {
        options.name = "alert";
      }
      callBrick(options);
    },

    datePicker: function(options) {
      if (options && _.isObject(options)) {
        options.name = "datePicker";
      }
      callBrick(options);
    },

    /**
     * error view
     * @param options
     */
    error: function (options) {
      if (options && _.isObject(options)) {
        options.name = "error";
      }
      callBrick(options);
    },

    /**
     * confirm view
     * @param options
     * @returns {*}
     */
    confirm: function (options) {
      if (options && _.isObject(options)) {
        options.name = "confirm";
      }
      callBrick(options);
    },

    /**
     * 验证电话号码
     * @param options
     */
    verifyTelephone: function (options) {
      if (options && _.isObject(options)) {
        options.name = "verifyTelephone";
      }
      callBrick(options);
    },

    /**
     * 验证组合调仓购买
     *
     */
    pay: function (options) {
      if (options && _.isObject(options)) {
        options.name = "pay";
      }
      callBrick(options);
    },

    /**
     *
     * @param options
     * options 同ajax options
     * 先不包成promise了
     */
    request: function (options) {
      if (options && _.isObject(options)) {
        if (Util.appVersion() >= 7.2 && options.tid === "PINGAN"){
          options.tid = "PAMID"
        }
        options.name = "request";

        if (_.isUndefined(options.type)) {
          options.type = "GET";
        }
      }

      callBrick(options);
    },

    mergeRequests: function (options) {
      if (options && _.isObject(options)) {
        options.name = "mergeRequests";
      }

      callBrick(options);
    },
    /**
     * @param options
     * 接口说明：页面跳转
     * 输入: js://{name: "redirect", url: "/newurl", type: "PUSH/POP/MODAL/CLOSE",callback: "callbackfunction", callbackData: {}}
     * 参数说明：
     * url：可以是相对 url（雪球内部 url） 也可以是绝对 url，可选（POP 和 CLOSE 的时候不需要 url）
     * callback: PUSH 和 MODAL 的时候可以带 callback 方法，当 push 和 modal 的 view 关闭的时候执行，方法带的参数是 POP 和 CLOSE 命令里带的 callbackData
     * type=PUSH 推进 nav 栈
     * type=POP 关闭页面，回到 nav 上一个页面
     * type=MODAL 开新模态页面
     * type=CLOSE 关闭当前页面所在的模态页面
     */
    redirect: function (options) {
      if (options && _.isObject(options)) {
        options.name = "redirect";
      }

      // 临时hack一下 由于6.5 iOS客户端传递过来的url没有encode
      // 需要把所有非iOS URL为proxy.html 加个参数 hasEncode=1
      // 等6.6正式上线  这个hack就可以去掉了

      if (options.url && options.url.indexOf("broker/proxy") > -1) {
        options.url += "&hasEncode=1";
      }

      callBrick(options);
    },

    /**
     * 更新券商的tid, aid, writetoken
     * 在绑定成功的 回调页面 或者 验证密码成功的回调页面调用
     * @param options
     */
    updateBrokerInfo: function (options) {
      if (options && _.isObject(options)) {
        options.name = "updateBroker";
      }

      callBrick(options);
    },

    /**
     *
     * {name:"setToolbar", buttons: [{title: "button1", action: "button1_action", icon: "icon1_url", hightlight: true}]}
     * @param options
     * buttons 数组
     * 每个 button 内部的参数:
     * tittle: button 的文字
     * action: 点击 button 的事件
     * icon: button 文字前面的小图标 url（可选）
     * highlight: 是否是蓝底白字，比如「交易」，（可选，默认是否）
     */
    setToolbar: function (options) {
      if (options && _.isObject(options)) {
        options.name = "setToolbar";
      }

      callBrick(options);
    },

    /**
     * 获取当前用户的access_token
     * @param options
     */
    getAccessToken: function (options) {
      if (options && _.isObject(options)) {
        options.name = "getAccessToken";
      }

      callBrick(options);
    },

    /**
     * 获取指定绑定券商的写token
     * {aid:aid}
     * @param options
     */
    getWriteToken: function (options) {
      if (options && _.isObject(options)) {
        options.name = "getWriteToken";
      }

      callBrick(options);
    },

    /**
     * 获取当前用户的userInfo
     * @param options
     */
    getUserInfo: function (options) {
      if (options && _.isObject(options)) {
        options.name = "getUserInfo";
      }

      callBrick(options);
    },

    /**
     * 获取绑定券商的AID
     * @param options
     */
    getAID: function (options) {
      if (options && _.isObject(options)) {
        options.name = "getAID";
      }

      callBrick(options);
    },

    /**
     * 交易股票搜索
     * @param options
     */
    searchTCStock: function (options) {
      if (options && _.isObject(options)) {
        options.name = "searchTCStock";
      }

      callBrick(options);
    },

    /**
     * 接口说明：设置页面 toolbar
     * 输入：js://{name:"setToolbar", buttons: [{title: "button1", action: "button1_action", icon: "icon1_url", hightlight: true}, {title: "button2", action: "button2_action", icon: "icon2_url", hightlight: false}]}
     * 参数说明：
     * buttons 是一个 array
     * 每个 button 内部的参数:
     * tittle: button 的文字
     * action: 点击 button 的事件
     * icon: button 文字前面的小图标 url（可选）
     * highlight: 是否是蓝底白字，比如「交易」，（可选，默认是否）
     * @param options
     */
    setToolbar: function (options) {
      if (options && _.isObject(options)) {
        options.name = "setToolbar";
      }

      callBrick(options);
    },

    /**
     * 接口说明：设置页面右上角按钮
     * 输入： js://{name: "setRightNavigationButton", title: "下一个页面", action: 'callback'}
     * @param options
     */
    setRightNavigationButton: function (options) {
      if (options && _.isObject(options)) {
        options.name = "setRightNavigationButton";
      }

      callBrick(options);
    },
    on: function (event, callback) {
      var array = eventObj[event];

      if (array) {
        array.push(callback);
      } else {
        array = [callback];
        eventObj[event] = array;
      }
    },
    off: function (event, callback) {
      if (callback) {
        var array = eventObj[event];

        if (array && array.length) {
          array = _.reject(array, function (fn) {
            return fn == callback;
          });
        }
      } else {
        delete eventObj[event];
      }
    },
    trigger: function (event) {
      var array = eventObj[event];

      if (array && array.length) {
        _.each(array, function (fn) {
          fn();
        });
      }
    },

    /**
     * 分享
     *
     * @param options
     */
    share: function (options) {
      if (options && _.isObject(options)) {
        options.name = "share";
      }

      callBrick(options);
    },

    /**
     * 事件统计
     * @param options
     */
    trackEvent: function (options) {
      if (typeof options === "string") {
        options = {
          event: options
        }
      }
      if (options && _.isObject(options)) {
        options.name = "trackEvent";
      }

      callBrick(options);
    },
    /**
     * SNB事件统计
     * 对照文档：http://docs.snowballfinance.com/pages/viewpage.action?pageId=3081077
     * @param options (pageId:页面ID,widgetId:控件类型,widgetPosition:控件位置,foreignKey:外键)
     */
    trackSNBEvent: function (options) {
      if (Util.appVersion()< 7){
        return false;
      }
      if (options && _.isObject(options)) {
        options.name = "trackSNBEvent";
      }
      callBrick(options);
    },
    /**
     * 唤醒其他app 或者跳转下载地址
     * @param options
     */
    wakeApp: function (options) {
      if (options && _.isObject(options)) {
        options.name = "wakeApp";
      }

      callBrick(options);
    },

    /**
     * 提供原生转发输入框
     * 输入：js//{name: "newRetweet",
     * data: {content: "status content", statusId: 124442113},
     * success: "cb1",
     * cancel: "cb2",
     * error: "cb3",
     * complete: "cb4"}
     * 参数说明：content：转发的内容，statusId：转发的 status id
     */
    newRetweet: function (options) {
      if (options && _.isObject(options)) {
        options.name = "newRetweet";
      }
      callBrick(options);
    },

    /**
     * 提供原生发布评论输入框
     * 输入：js//{name: "replyComment",
     * data: {content: "status content", statusId: 124442113, username: 'testuser', commentId: 123213213, isRetweet: true},
     * success: "cb1",
     * cancel: "cb2",
     * error: "cb3",
     * complete: "cb4"}
     * 参数说明：content：评论的内容，statusId：根 status 的 id，username:被评论人的 username， commentId：评论的 comment 的 id，isRetweet：是否评论并转发
     */
    replyComment: function (options) {
      if (options && _.isObject(options)) {
        options.name = "replyComment";
      }
      callBrick(options);
    },
    bindBroker: function (options) {
      if (options && _.isObject(options)) {
        options.name = "bindBroker";
      }
      callBrick(options);
    },
    refreshBrokerToken: function (options) {
      if (options && _.isObject(options)) {
        options.name = "refreshBrokerToken";
      }
      callBrick(options);
    },
    /**
     * 跳转到交易首页
     */
    goToTradeHome: function () {
      callBrick("goToTradeHome");
    },
    /**
     * 通知客户端更新券商列表
     */
    updateBrokerList: function () {
      callBrick("updateBrokerList");
    },
    /**
     * 加密
     * js://{name: encryptWithRSA, origin: '123456' success: cb1, error: "cb2" }
     */
    encryptWithRSA: function (options) {
      if (options && _.isObject(options)) {
        options.name = "encryptWithRSA";
        options.origin = options.origin + "";
      }
      callBrick(options);
    }
  };

  window.Util = {

    /**
     * 绑定券商
     * @param options
     * success 绑定成功 回调
     * cancel 取消绑定 回调
     */
    bindBroker: function (options) {
      var url = "/broker/bind-list";
      if (options.isIB) {
        url += "?market=USHK";
      }

      xqBridge.redirect({
        url: url,
        type: "MODAL",
        success: options.success,
        cancel: options.cancel
      });
    },

    /**
     * 验证密码
     * @param options
     * url 密码过期  接口会返回验证密码的url
     * success 验证成功 回调
     * cancel 取消验证 回调
     */
    verifyPassword: function (options) {

      xqBridge.redirect({
        url: options.url,
        type: "MODAL",
        success: options.success,
        cancel: options.cancel
      });
    },

    /**
     * 检查是否绑定券商
     * @param option
     */
    checkBindBroker: function (options) {
      var that = this;

      if (options.success && options.error) {
        xqBridge.request({
          url: '/tc/snowx/securities/bound.json',
          success: function (data) {
            var brokers = data.result_data;

            if (brokers && brokers.length) {
              // 全局设置券商列表
              window.userInfo.brokerList = brokers;
              options.success(data);
            } else {
              options.success(false);
            }
          },
          error: function (e) {
            options.error(e)
          }
        })
      }
    },
    /**
     * 券商页面初始化
     * 自动判断是否需要绑定，需要绑定就跳转
     * 自动判断是否token过期
     * @param options
     */
    checkAndInit: function (options) {
      var that = this;


      if (options.success && options.cancel) {
        that.checkBindBroker({
          isIB: options.isIB,
          success: function (lists) {
            if (lists) {
              options.success(lists)
            } else {
              //that.bindBroker({
              //  isIB: options.isIB,
              //  success: function () {
              //    that.checkAndInit(options);
              //  },
              //  cancel: function (e) {
              //    options.cancel(e)
              //  }
              //});
              var url = "/broker/bind-list";
              if (options.isIB) {
                url += '?ib=true';
              }
              xqBridge.redirect({
                type: "PUSH",
                url: url,
                success: function () {
                  that.checkAndInit(options);
                },
                cancel: function (e) {
                  options.cancel(e)
                }
              })
            }
          },
          error: function (e) {
            options.cancel(e);
          }
        })
      }
    },
    /**
     * 新版本 升权  需要调用交易密码有效期设置接口
     * 如果是 每次都弹出 就每次都调用升权界面
     * 其余的用后置升权
     * @param options
     */
    newGetAndRefreshToken: function (options) {
      var that = this;

      function successCb () {
        xqBridge.getWriteToken({
          aid: options.aid,
          success: function (token) {
            options.success && options.success(token);
          }
        });
      }

      function getToken() {
        if (options.from === "trade") {
          xqBridge.trackEvent("trade_passwordAlert");
        }
        if (options.broker && options.broker.page_type == 2) {
          xqBridge.refreshBrokerToken({
            data: {
              result_data: options.broker
            },
            success: function () {
              successCb();
            },
            cancel: function (e) {
              options.error && options.error(e);
            }
          });
        } else {
          xqBridge.redirect({
            url: "/broker/proxy?url=" + encodeURIComponent(options.renew_url),
            type: 'MODAL',
            success: function () {
              successCb();
            },
            cancel: function (e) {
              options.error && options.error(e);
            }
          });
        }
      }

      // 请求密码有效期接口
      xqBridge.request({
        url: "/tc/snowx/accesstoken/expires/query.json",
        success: function (ret) {
          if (ret.result_code == 60000) {
            var data = ret.result_data;
            if (data && data.accessTokenExpires == 0) {
              getToken();
            } else {
              xqBridge.getWriteToken({
                aid: options.aid,
                success: function (token) {
                  if (token && typeof token == "string") {
                    options.success && options.success(token);
                  } else {
                    getToken();
                  }
                }
              });
            }
          } else {
            getToken();
          }
        },
        error: function () {
          getToken();
        }
      })
    },
    /**
     * 检查券商token是否过期
     */
    getAndRefreshToken: function (options) {
      var that = this;

      if (that.appVersion() >= 6.5) {
        that.newGetAndRefreshToken(options);

      } else {
        // 老的升权逻辑
        xqBridge.getWriteToken({
          aid: options.aid,
          success: function (token) {
            if (token) {
              options.success && options.success(token);
            } else {
              function successCb () {
                xqBridge.getWriteToken({
                  aid: options.aid,
                  success: function (token) {
                    options.success && options.success(token);
                  }
                });
              }
              xqBridge.redirect({
                url: "/broker/proxy?url=" + encodeURIComponent(options.renew_url),
                type: 'MODAL',
                success: function () {
                  successCb();
                },
                cancel: function (e) {
                  options.cancel && options.cancel(e);
                }
              });
            }
          }
        })

      }

    },


    /**
     * 解析query字符串为对象
     * @param query
     * @returns {{}}
     */
    parseParams: function (url) {
      var href = url || window.location.href,
        array = href.split("?"),
        query = "";

      if (array.length === 1) {
        return {};
      } else {
        query = _.last(array);
      }

      // parseParams
      var re = /([^&=]+)=?([^&]*)/g;
      var decodeRE = /\+/g;  // Regex for replacing addition symbol with a space
      var decode = function (str) {
        return decodeURIComponent(str.replace(decodeRE, " "));
      };

      query = decodeURIComponent(query);
      var params = {},
        e;

      while (e = re.exec(query)) {
        var k = decode(e[1]),
          v = decode(e[2]);

        if (k.substring(k.length - 2) === '[]') {
          k = k.substring(0, k.length - 2);
          (params[k] || (params[k] = [])).push(v);
        } else {
          params[k] = v;
        }
      }
      return params;
    },

    showError: function (str,cb) {
      var $error = $(".error-tip"),
        length = str.length,
        textWidth = length * 14,
        divWidth = textWidth + 20,
        scrollTop = $(window).scrollTop(),
        windowHeight = $(window).height();

      if (!$error.length) {
        $("body").append('<div class="error-tip"></div>');
        $error = $(".error-tip");
      }

      $error.css({
        "width": divWidth + "px",
        "margin-left": "-" + divWidth / 2 + "px",
        "top": scrollTop + windowHeight / 2 + "px"
      });

      $error
        .text(str)
        .show();

      setTimeout(function () {
        $error.hide();
        cb && cb();
      }, 1500);
    },
    setPress: function (selector, pressClass) {
      var $body = $("body");
      var startEvent = "touchstart";
      var endEvent = "touchend";
      var moveEvent = "touchmove";

      if (!'ontouchstart' in document.documentElement) {
        startEvent = "mousedown";
        endEvent = "mouseup";
        moveEvent = "mousemove";
      }

      var isMove = false;

      $body.on(startEvent, selector, function () {
        var $el = $(this);

        setTimeout(function () {
          console.log("start", isMove);
          if (!isMove && !$el.hasClass(pressClass)) {
            $el.addClass(pressClass);
            $el.data("nopress", false);
          }
        }, 100);
      });

      $body.on(moveEvent, selector, function () {
        var $el = $(this);
        isMove = true;
        console.log("move", isMove);
        $el.removeClass(pressClass);
      });

      $body.on(endEvent, selector, function () {
        var $el = $(this);

        setTimeout(function () {
          isMove = false;
          console.log("end", isMove);
          $el.removeClass(pressClass);
        }, 200);
      });

      if (navigator.userAgent.indexOf("iPhone") == -1) {
        $body.on(endEvent, function () {
          isMove = false;
        });
      }
    },
    appVersion: function () {
      var version = 5;
      var uaArr = navigator.userAgent.split(" ");// Xueqiu iPhone 6.0-dev-1111

      if (uaArr.length >= 3) {
        var longVersion = uaArr[2];
        var longVersionArr = longVersion.split("-");
        version = parseFloat(longVersionArr[0]);
      }

      return version;
    },
    getBroker: function (brokers, tid) {
      var broker;
      if (tid) {
        broker = _.find(brokers, function (b) {
          return b.tid == tid;
        });
        if (broker) {
          broker.is_default = true;
        }
      } else {
        broker = _.find(brokers, function (b) {
          return b.is_default;
        });
      }

      if (brokers.length && !broker) {
        broker = _.first(brokers);
        broker.is_default = true;
      }

      return broker;
    },
    //计算交易手续费
    getCommission: function (price, amount, action, symbol, tid) {
      if (!symbol){
        return;
      }
      var totalAmount = parseFloat(price * amount).toFixed(2),
        calculateAmount = 0,
        stampTax = 0, //印花税
        transferFee = 0, //过户费
        commission = 0; //佣金
      var stockInfo = this.getStockInfo(symbol);
      //判断回购
      var repurchaseObj = this.getRepurchase(symbol);
      var repurchase = repurchaseObj.repurchase;
      var repurchaseRate = repurchaseObj.repurchaseRate;
      var isRepurchase = repurchase !== 0;
      //交易金额
      if (repurchase !== 0) {
        totalAmount = amount || 0.00;
      }
      totalAmount = parseFloat(totalAmount).toFixed(2);

      //印花税
      if (isRepurchase) {
        stampTax = "0.00";
      } else if ((stockInfo.name === "A股" || stockInfo.name === "深B" || stockInfo.name === "沪B") && action === "SELL") {
        stampTax = totalAmount * 0.001;
      } else {
        stampTax = "0.00";
      }
      stampTax = parseFloat(stampTax).toFixed(2);

      //过户费
      if (isRepurchase) {
        transferFee = "0.00";
      } else if (symbol.search("SH") !== -1 && (stockInfo.name === "A股" || stockInfo.name === "沪B") && stockInfo.type === "股票" && parseInt(totalAmount) !== 0) {
        transferFee = totalAmount * 0.00002;
      } else {
        transferFee = "0.00";
      }
      transferFee = parseFloat(transferFee).toFixed(2);

      //佣金
      if (this.specialStockCommission(symbol, tid)) {
        commission = totalAmount * 0.00025;
        if (isRepurchase) {
          commission = totalAmount * repurchaseRate;
        } else if (0 < commission && commission < 5 && !!price && !!amount) {
          commission = 5.00;
        }
      }
      commission = parseFloat(commission).toFixed(2);
      calculateAmount = parseFloat(stampTax) + parseFloat(transferFee) + parseFloat(commission);
      return {
        calculateAmount: parseFloat(calculateAmount).toFixed(2),
        totalAmount: totalAmount,
        commission: commission,
        transferFee: transferFee,
        stampTax: stampTax,
        repurchase: repurchase,
        repurchaseRate: repurchaseRate
      };
    },
    /**
     * 显示遮罩
     */
    overlay: {
      $overlay: $("#overlay"),
      showOverlay: function (dialog, event) {
        var that = this;
        $('#container').css({overflow: 'hidden'});
        if (dialog) {
          dialog.show();
        }
        if (that.$overlay.length) {
          that.$overlay.show();
        } else {
          $('body').append("<div id='overlay'></div>");
          that.$overlay = $("#overlay");
          that.$overlay
            .css({
              'height': '100%',
              'opacity': 0.2,
              'position': 'fixed',
              'top': 0,
              'left': 0,
              'background-color': 'black',
              'width': '100%',
              'z-index': 4400,
              'display': 'none'
            })
            .show();
          if (event) {
            that.$overlay.on('touchstart', function () {
              $('#container').css({overflow: 'scroll'});
              $(this).hide();
              if (dialog) {
                dialog.hide();
              }
            })
          }
        }
        return that.$overlay;
      },
      hideOverlay: function (dialog) {
        $('body').css({overflow: 'scroll'});
        $("#overlay").hide();
        if (dialog) {
          dialog.hide();
        }
      }
    },
    /**
     * 弹出提示
     */
    toast: {
      $toast: $(".toast"),
      show: function (message) {
        var that = this;
        if (that.$toast.length > 0) {
          that.$toast.html(message).show();
          setTimeout(function () {
            that.$toast.hide();
          }, 3000);
        } else {
          that.$toast = $("<div class='toast'>" + message + "</div>");
          $("body").append(that.$toast);
          that.$toast.css({
            backgroundColor: "rgba(0,0,0,0.6)",
            fontSize: "12px",
            borderRadius: '5px',
            position: "fixed",
            bottom: "15%",
            left: "15%",
            color: "#fff",
            padding: "10px",
            display: "none",
            zIndex: "9999",
            textAlign: "center",
            width: "70%",
            marginLeft: '-10px'
          });
          that.$toast.show();
          setTimeout(function () {
            that.$toast.hide();
          }, 3000);
        }
      },
      hide: function () {
        if (this.$toast.length > 0) {
          this.$toast.hide();
        }
      }
    },
    specialStockCommission: function (symbol, tid) {
      var hasCommission = true;
      var gjCheck = [
        "SH511880",
        "SH511980",
        "SH511990",
        "SH511810",
        "SH511890",
        "SH511860",
        "SZ159003",
        "SZ159005",
        "SZ159001",
        "SH511800",
        "SH511830"
      ];
      var glCheck = [
        "SH511880",
        "SH511980",
        "SH511990",
        "SH511810",
        "SH511890",
        "SH511860",
        "SZ159003",
        "SZ159005",
        "SZ159001"
      ];
      if (tid === "GJZQ" || tid === Util.appVersion() >= 7.2 ? "PAMID" : "PINGAN") {
        if (_.contains(gjCheck, symbol)) {
          hasCommission = false;
        }
      }
      if (tid === "GLZQ"){
        if (_.contains(glCheck,symbol)){
          hasCommission = false;
        }
      }
      return hasCommission;
    },
    getRepurchase: function (code) {
      var repurchase, repurchaseRate;
      switch (code) {
        case "SH204001":
        case "SZ131810":
          repurchase = 1;
          repurchaseRate = 0.00001;
          break;
        case "SH204002":
        case "SZ131811":
          repurchase = 2;
          repurchaseRate = 0.00002;
          break;
        case "SH204003":
        case "SZ131800":
          repurchase = 3;
          repurchaseRate = 0.00003;
          break;
        case "SH204004":
        case "SZ131809":
          repurchase = 4;
          repurchaseRate = 0.00004;
          break;
        case "SH204007":
        case "SZ131801":
          repurchase = 7;
          repurchaseRate = 0.00005;
          break;
        case "SH204014":
        case "SZ131802":
          repurchase = 14;
          repurchaseRate = 0.0001;
          break;
        case "SH204028":
        case "SZ131803":
          repurchase = 28;
          repurchaseRate = 0.0002;
          break;
        case "SH204091":
        case "SZ131805":
          repurchase = 91;
          repurchaseRate = 0.0003;
          break;
        case "SH204182":
        case "SZ131806":
          repurchase = 182;
          repurchaseRate = 0.0003;
          break;
        default:
          repurchase = 0;
          repurchaseRate = 0.00001;
          break;
      }
      return {
        repurchase: repurchase,
        repurchaseRate: repurchaseRate
      };
    },
    getStockInfo: function (stockid) {
      var specialHKStocks = {
          HKHSI: 1,
          HKHSF: 1,
          HKHSU: 1,
          HKHSP: 1,
          HKHSC: 1,
          HKVHSI: 1,
          HKHSCEI: 1,
          HKHSCCI: 1,
          HKGEM: 1,
          HKHKL: 1
        },
        USIndexes = {
          DJI30: 1,
          NASDAQ: 1,
          SP500: 1,
          ICS30: 1,
          SLR10: 1,
          TMT20: 1,
          HCP10: 1,
          EDU10: 1
        },
        bitcoins = {
          BTCNCNY: {
            money: "￥",
            market: "比特币（CNY）"
          },
          MTGOXAUD: {
            money: "AU$",
            market: "比特币（AUD）"
          },
          MTGOXCNY: {
            money: "￥",
            market: "比特币（CNY）"
          },
          MTGOXEUR: {
            money: "€",
            market: "比特币（EUR）"
          },
          MTGOXGBP: {
            money: "£",
            market: "比特币（GBP）"
          },
          MTGOXJPY: {
            money: "J￥",
            market: "比特币（JPY）"
          },
          MTGOXUSD: {
            money: "$",
            market: "比特币（USD）"
          },

          // 新增一些Bitcoins
          LOCALBTCAUD: {
            money: "AU$",
            market: "比特币（AUD）"
          },
          MTGOXCNY: {
            money: "￥",
            market: "比特币（CNY）"
          },
          BTCDEEUR: {
            money: "€",
            market: "比特币（EUR）"
          },
          BTCEEUR: {
            money: "€",
            market: "比特币（EUR）"
          },
          LOCALBTCGBP: {
            money: "£",
            market: "比特币（GBP）"
          },
          BTCEUSD: {
            money: "$",
            market: "比特币（USD）"
          },
          BITSTAMPUSD: {
            money: "$",
            market: "比特币（USD）"
          }
        },
        fundSymbolArray = ["SH500", "SH510", "SH511", "SH512", "SH513", "SH518", "SH519", "SZ15", "SZ18", "SZ16"],
        stockInfo;

      if (stockid && ( ~_.indexOf(fundSymbolArray, stockid.substr(0, 5)) || ~_.indexOf(fundSymbolArray, stockid.substr(0, 4)) )) {
        stockInfo = {
          type: "基金",
          name: "基金",
          exchange: "沪深",
          money: "￥"
        }
      } else if (/^MF\d+$/.test(stockid)) {
        stockInfo = {
          type: "货币基金",
          name: "货币基金",
          exchange: "沪深",
          money: "￥"
        }
      } else if (/^P\d+$/.test(stockid)) {
        stockInfo = {
          type: "私募基金",
          name: "私募基金",
          exchange: "沪深",
          money: "￥"
        }

        if (stockid === "P000057") {
          stockInfo.money = "$";
        }
      } else if (/^FP\d+$/.test(stockid)) {
        stockInfo = {
          type: "理财产品",
          name: "理财产品",
          exchange: "沪深",
          money: "￥"
        }
      } else if (/^[\d\w]+\.FM$/.test(stockid)) {
        stockInfo = {
          type: "国债期货",
          name: "国债期货",
          exchange: "沪深",
          money: "￥"
        }
      } else if (/^TP\d+$/.test(stockid)) {
        stockInfo = {
          type: "信托产品",
          name: "信托产品",
          exchange: "沪深",
          money: "￥"
        }
      } else if (/^ZH\d{6}$/.test(stockid)) {
        stockInfo = {
          type: "投资组合",
          name: "投资组合",
          exchange: "组合",
          money: "￥"
        }
      } else if (/^S[HZ]\d+$/.test(stockid) || (/^CSI/.test(stockid) && stockid.length === 9)) {
        if (/^SZ200/.test(stockid)) {//SH900||SZ200为HK$B股
          stockInfo = {
            type: "股票",
            name: "深B",
            exchange: "沪深",
            money: "HK$"
          }
        } else if (/^(SH(201|202|203|204|131)|SZ(13|395032))/.test(stockid)) {
          // 回购
          stockInfo = {
            type: "回购",
            name: "回购",
            exchange: "沪深",
            money: "￥"
          }
        } else if (/^(SH(01|02|13)|SZ(10|09))/.test(stockid)) {
          // 国债
          stockInfo = {
            type: "国债",
            name: "国债",
            exchange: "沪深",
            money: "￥"
          }
        } else if (/^(SH12|SZ11)/.test(stockid)) {
          // 企债
          stockInfo = {
            type: "企债",
            name: "企债",
            exchange: "沪深",
            money: "￥"
          }
        } else if (/^(SH11|SZ12)/.test(stockid)) {
          // 可转债
          stockInfo = {
            type: "可转债",
            name: "可转债",
            exchange: "沪深",
            money: "￥"
          }
        } else if (/^SH900/.test(stockid)) {
          stockInfo = {
            type: "股票",
            name: "沪B",
            exchange: "沪深",
            money: "$"
          }
        } else if (/^(SH00|SZ399|CSI)/.test(stockid)) {//指数
          stockInfo = {
            type: "指数",
            name: "指数",
            exchange: "沪深",
            money: ""
          }
        } else {
          stockInfo = {
            type: "股票",
            name: "A股",
            exchange: "沪深",
            money: "￥"
          }
        }
      } else if (/^\d+$/.test(stockid)) {
        stockInfo = {
          type: "股票",
          name: "港股",
          exchange: "港股",
          money: /^8/.test(stockid) ? "￥" : "HK$"
        }
      } else if (specialHKStocks[stockid] || USIndexes[stockid]) {//指数
        stockInfo = {
          type: "指数",
          name: "指数",
          exchange: "",
          money: ""
        }
      } else {
        var bitcoin = bitcoins[stockid];

        if (bitcoin) {
          stockInfo = {
            type: "比特币",
            name: "比特币",
            exchange: bitcoin.market,
            money: bitcoin.money
          }
        } else {
          stockInfo = {
            type: "股票",
            name: "美股",
            exchange: "美股",
            money: "$"
          }
        }
      }
      return stockInfo;
    },
    getEtypeBySymbol: function (symbol) {
      if (!symbol) {
        return "";
      }
      var etype = "";
      var stockInfo = this.getStockInfo(symbol);
      var type = stockInfo.exchange;

      if (type === "美股") {
        etype = "USEX";
      }

      if (type === "港股") {
        etype = "HKEX";
      }

      if (type === "沪深" && symbol.indexOf("SH") > -1) {
        etype = "SHEX";
      }

      if (type === "沪深" && symbol.indexOf("SZ") > -1) {
        etype = "SZEX";
      }

      if (symbol === "799999") {
        etype = "SHEX";
      }
      return etype;
    },
    isFund: function (symbol) {
      var fundSymbolArray = ["SH500", "SH502", "SH510", "SH511", "SH513", "SZ15", "SZ18", "SZ16"];
      if (_.indexOf(fundSymbolArray, symbol.substr(0, 5)) > -1 || _.indexOf(fundSymbolArray, symbol.substr(0, 4)) > -1) {
        return true;
      }
      return false;
    }
  };

  // 需要前置初始化的项
  baseInit = function (cb) {
    window.userInfo = {};

    window.viewDidAppear = function(){

    };
    window.viewDidDisappear = function(){

    };


  };

  baseInit()
})(window);
