/**
 * Created by lyj on 5/5/15.
 */

define(function (require, exports, module) {

  var brokerListHtml = require('template/order/trade-broker-list.js');
  var stockListHtml = require('template/order/order-stock-list.js');

  var params = Util.parseParams();

  var isNavigate = false;

  function errorHandler(e) {
    xqBridge.hideLoading();
    if (e && e.result_code == 60000) {
      return true;
    }
    else if (e && (e.result_code == 70005 || e.result_code == 70006)) {
      xqBridge.redirect({
        type: 'POP_CANCEL'
      });
    } else if (e && (e.error_description || e.msg)) {
      xqBridge.alert({
        message: e.msg || e.error_description,
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    } else {
      xqBridge.alert({
        message: '网络错误',
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    }
  }

  module.exports = {
    init: function () {
      var that = this;

      that.isIB = params.tid === "IB";
      Util.checkBindBroker({
        isIB: that.isIB,
        success: function (brokerList) {
          if (brokerList) {
            var brokers = brokerList.result_data;
            var broker = Util.getBroker(brokers, Util.parseParams().tid);

            that.broker = broker;
            that.aid = broker.aid;
            that.tid = broker.tid;
            that.renew_url = broker.renew_url;

            var html = brokerListHtml({
              data: {
                brokers: [broker]
              }
            });

            $('.broker-list').append(html);
            that.getStockList($(".order-position"));
            that.bindEvent();
            $(".trade").show();
          } else {
            xqBridge.alert({
              title: "请先添加证券账户",
              success: function () {
                xqBridge.redirect({
                  type: 'POP_CANCEL'
                });
              }
            });
          }
        },
        error: function (e) {
          xqBridge.redirect({
            type: "POP_CANCEL"
          });
        }
      })
    },

    //检查券商帐号绑定
    checkBind: function (cb) {
      var that = this;
      // 页面初始化
      Util.checkAndInit({
        success: function (data) {
          if (!errorHandler(data)) {
            return;
          }
          var brokers = data.result_data;
          var broker = Util.getBroker(brokers, Util.parseParams().tid);

          if (broker) {
            that.broker = broker;
            that.aid = broker.aid;
            that.renew_url = broker.renew_url;
            that.trader_name = broker.trader_name;
            //进入是刷新writetoken
            Util.getAndRefreshToken({
              aid: broker.aid,
              renew_url: broker.renew_url,
              success: function () {
                var html = brokerListHtml({
                  data: {
                    brokers: brokerList
                  }
                });
                $('.broker-list').html(html);
                if (cb) {
                  cb();
                }
              },
              error: function () {
                xqBridge.redirect({
                  type: "POP_CANCEL"
                });
              }
            })
          }
        },

        // 初始化失败退出交易页面
        cancel: function () {
          //xqBridge.redirect({
          //  type: "POP_CANCEL"
          //});
        }
      });
    },
    getStockList: function ($container) {
      var that = this;
      xqBridge.request({
        url: "/tc/snowx/position/list.json",
        type: "GET",
        data: {
          aid: that.aid,
          tid: that.tid
        },
        success: function (ret) {
          var stocks = ret.result_data;
          var cost = 0;

          stocks = _.filter(stocks, function (stock) {
            return stock.enable_amount > 0;
          });

          var priceStr;
          _.each(stocks, function (stock) {
            var price = stock.last_price + "";
            var array = price.split(".");
            if (array.length > 1) {
              var len = array[1].length;
              if (len < 2) {
                len = 2;
              }
              price -= price * 0.003;
              priceStr = price.toFixed(len);
            } else {
              price -= 1;
              priceStr = price.toFixed(2);
            }


            stock.last_price = priceStr;
            cost += price * stock.enable_amount;
          });

          $(".number").html(stocks.length);

          var html = stockListHtml({
            data: {
              cost: cost,
              list: ret.result_data
            }
          });

          $container.html(html).show();
        },
        error: function (xhr) {
          console.log(xhr);
        },
        complete: function () {
          xqBridge.hideLoading();
        }
      });

    },
    bindEvent: function () {
      var that = this;

      // 切换plan
      $("body").on("click", ".order-plan button", function () {
        var $button = $(this);
        if ($button.hasClass("select")) {
          return false;
        }

        var $latestContainer = $(".order-plan-latest");
        var $allContainer = $(".order-plan-all");

        var plan = $button.data("plan");
        that.plan = plan;

        $button.addClass("select");
        $button.siblings().removeClass("select");

        if ($button.hasClass("all")) {
          $allContainer.show();
          $latestContainer.hide();
        } else {
          var html = $latestContainer.html();

          if (html.length) {
            $latestContainer.show();
            $allContainer.hide();
          } else {
            that.getStockList($latestContainer);
          }
        }
      });

      $("body").on('click', ".select-container", function () {
        var $el = $(this).find(".stock-select");

        if ($el.hasClass("checked")) {
          $el.removeClass("checked");
        } else {
          $el.addClass("checked");
        }
        updateCost();
      });

      // 飞弹份数
      $('body').on('click', ".stock-multi-volume .button", function () {
        var container = that.plan == "1" ? $(".order-plan-all") : $(".order-plan-latest");
        var v = container.find('.stock-multi-volume input');

        if (!$(this).hasClass('disabled')) {
          v[0].previous = v.val();
          if ($(this).hasClass('plus')) {


            v.val(parseInt(v.val()) + 1);
          } else {
            v.val(parseInt(v.val()) - 1);

          }
          v[0].now = parseInt(v.val());
          v[0].times = v[0].now / v[0].previous;
          if (v.val() == 1) {
            container.find('.stock-multi-volume .minus').addClass('disabled');
          } else {
            container.find('.stock-multi-volume .minus').removeClass('disabled');
          }
          updateVolume();
          updateCost();
        }

      });

      $('body').on('blur', '.stock-multi-volume input', function () {
        var container = that.plan == "1" ? $(".order-plan-all") : $(".order-plan-latest");
        var numberReg = /^[0-9]*[1-9][0-9]*$/;
        var v = $(this);
        if (!numberReg.test($(this).val())) {
          Util.showError("必须是正整数");
          $(this).val(v[0].now || 1);
        } else if ($(this).val() != v[0].now) {
          v[0].previous = v[0].now || 1;
          v[0].now = v.val();
          v[0].times = v[0].now / v[0].previous;
          updateVolume();
          updateCost();
          if ($(this).val() == 1) {
            container.find('.stock-multi-volume .minus').addClass('disabled');
          } else {
            container.find('.stock-multi-volume .minus').removeClass('disabled');
          }

        }


      });
      //
      function updateVolume() {
        var container = that.plan == "1" ? $(".order-plan-all") : $(".order-plan-latest");
        var volumes = container.find('.stock-volume .volume');
        var v = container.find('.stock-multi-volume input');

        for (var i = 0; i < volumes.length; i++) {
          $(volumes[i]).val(parseInt($(volumes[i]).val() * v[0].times));
        }

      }

      function getOrders() {
        var orders = [];
        var isVerified = true;
        var $container = $(".order-position");

        $(".stock-list li", $container).each(function () {
          var $li = $(this);
          var $select = $li.find(".stock-select");

          if ($select.hasClass("checked")) {
            var symbol = $li.data("symbol");
            var name = $li.find(".stock-name").text();
            var current = $.trim($li.find(".current").val());
            var volume = $.trim($li.find(".volume").val());
            var numberReg = /^-?\d+\.?\d*$/;

            if (!numberReg.test(current)) {
              Util.showError("价格必须是数字");
              isVerified = false;
              return false;
            }

            if (!numberReg.test(volume)) {
              Util.showError("数量必须是数字");
              isVerified = false;
              return false;
            }

            if (volume > 0) {
              orders.push({
                aid: that.aid,
                action: "SELL",
                scode: symbol,
                sname: name,
                price: current,
                amount: volume
              });
            }
          }
        });

        return isVerified && orders;
      }

      function getConfirmMessage(sellOrders) {
        var message = "";

        if (sellOrders.length) {
          message += "市价卖出：\n";

          _.each(sellOrders, function (sellOrder) {
            message += sellOrder.sname + " " + sellOrder.amount + "股 \n";
          });
        }

        return message;
      }

      function updateCost() {
        var orders = getOrders();
        var cost = 0;

        _.each(orders, function (order) {
          cost += parseFloat(order.price) * parseFloat(order.amount);
        });

        var $container = $(".order-position");
        $container.find(".amount").html(cost.toFixed(2));
        $(".number").text(orders.length);
      }

      $("body").on("blur", "input", function () {
        updateCost();
      });

      // 统计关闭页面
      $(window).unload(function () {
        if (!isNavigate) {
          xqBridge.trackEvent("cube_order_plan_cancel");
        }
      });


      // 打开协议页面
      $("body").on("click", ".order-term a", function () {
        var url = $(this).attr("href");
        xqBridge.redirect({
          url: url,
          type: "MODAL"
        });
        return false;
      });

      // 绑定券商
      $('body').on('click','.broker-none', function () {
          that.checkBind();
      });

      //提交订单
      $('body').on('click', '.submit', function () {
        xqBridge.trackEvent("sellAll");
        var $button = $(this);
        if ($button.data("disabled")) {
          return false;
        }

        var orders = getOrders();
        var orderCount = orders.length;

        if (orders && orderCount) {
          $button.data("disabled", false);

          Util.getAndRefreshToken({
            aid: that.aid,
            renew_url: that.renew_url,
            broker: that.broker,
            success: function (token) {

              xqBridge.confirm({
                title: "清仓确认",
                message: getConfirmMessage(orders),
                alignment: "left",
                success: function () {
                  var results = [];

                  function cb() {
                    if (results.length === orderCount) {
                      xqBridge.hideLoading();
                      var failOrders = [];
                      _.each(results, function (order) {
                        if (order.result_code !== "60000") {
                          failOrders.push(order);
                        }
                      });
                      if (failOrders.length) {
                        window.location.href = "./order-fail.html"
                        + "?orders=" + encodeURIComponent(JSON.stringify(results))
                        + "&tid=" + that.broker.tid;
                      } else {
                        window.location.href = "./order-success.html?count=" + results.length + "&tid=" + that.broker.tid;
                      }
                    }
                  }

                  xqBridge.showLoading();

                  //分开请求  先请求第一单  看token 是否过期
                  var first = orders.shift();
                  first.write_access_token = token;
                  first.tid = that.tid;

                  //先派第一个兵去侦察一下
                  //如果token 过期 会弹出升权对话框
                  //升权完成之后会重新 发送第一个请求
                  //第一个订单送出之后  会并发剩余的单子
                  xqBridge.request({
                    url: '/tc/order/place.json',
                    type: 'POST',
                    data: first,
                    success: function (ret) {
                      ret = _.extend(ret, first);
                      results.push(ret);
                      cb();
                      xqBridge.getWriteToken({
                        aid: that.aid,
                        success: function (newToken) {
                          //批量下剩余单
                          var data =  {
                            writeToken: newToken,
                            data: [],
                            aid: that.aid,
                            tid: that.tid
                          };
                          _.each(orders,function(item,index){
                            data.data[index] = item;
                            data.data[index].otype = "MKT";
                            data.data[index].scode = item.scode + "";
                          });
                          data.data = JSON.stringify(data.data);
                          xqBridge.request({
                            url:'/tc/snowx/order/placebatch.json',
                            type: 'POST',
                            data: data,
                            success : function(res){
                              results = results.concat(res.result_data);
                              cb();
                            },
                            error:function(err){
                              Util.toast.show(err.msg);
                              xqBridge.hideLoading();
                            }
                          })
                        }
                      });
                    },
                    error: function () {
                      xqBridge.hideLoading();
                    }
                  });
                },
                cancel: function () {
                }
              });
            },
            error: function () {
              //Util.showError('取消飞单');
            }
          });
        } else {
          if (_.isArray(orders)) {
            Util.showError("请选择股票");
          }
        }
      });
    }
  };
});
