define(function (require, exports, module) {
  var bindListTemplate = require('template/trade/bind-list.js');
  var params = Util.parseParams();

  module.exports = {
    init: function () {
      this.getCurrentMarket();
      this.getData();
      this.bindEvent();
    },
    /**
     * 获取当前市场
     */
    getCurrentMarket: function () {
      var market = $(".market-list").find(".select").text();
      $(".market").text("");
    },
    getBrokersByMarket: function (market) {
      var tid = "IB";
      var brokers = [];
      if (market === "CN") {
        brokers = _.filter(this.brokers, function (broker) {
          return broker.tid != tid;
        });
      } else {
        brokers = _.filter(this.brokers, function (broker) {
          return broker.tid == tid;
        });
      }

      return brokers;
    },
    getData: function () {
      var that = this;

      this.market = params.market || "CN";

      if (Util.appVersion() <= 6.5) {
        this.market = "CN";
      } else {
        $(".markets").show();
      }

      if (this.market != "CN") {
        $(".market-list li")
          .removeClass("select");

        $(".market-list li:last")
          .addClass("select");
      }

      xqBridge.request({
        url: "/tc/snowx/securities.json",
        type: "GET",
        data: {},
        success: function (ret) {
          // 成功
          if (ret.result_code == 60000) {
            var brokers = ret.result_data;
            that.brokers = brokers;
            that.renderBrokers = that.getBrokersByMarket(that.market);
            //不区分市场了
            //that.renderHtml(that.renderBrokers);
            that.brokers = that.brokers.map(function (item) {
              if (item.tid === "IB") {
                item.trader_name = "盈透证券(美/港股)"
              }
              return item
            });
            that.renderHtml(that.brokers);

            // 广告加载
            setTimeout(function () {
              seajs.use("ad-banner.js", function (ad) {
                ad.init("SecuritiesList_ad", "SecuritiesList_closeAd");
              });
            }, 200);
          }
        },
        error: function () {
          xqBridge.error("获取数据出错");
        },
        complete: function () {
          xqBridge.hideLoading();
        }
      });
    },
    renderHtml: function (brokers) {
      $(".open-broker, .open-ib").hide();
      $(".no-brokers").hide();

      if (brokers.length) {
        $('.brokers').html(bindListTemplate({
          data: {
            brokers: brokers
          }
        }));

        if (this.market == "CN") {
          $(".open-broker").show();
        } else {
          $(".open-ib").show();
        }
      } else {
        $(".no-brokers").show();
        $(".open-broker").hide();
      }
    },
    bindEvent: function () {
      var that = this;
      Util.setPress(".broker-list li .top", "onpress");
      Util.setPress(".broker-list li .bottom", "onpress");

      // 切换市场
      $(".market-list").on("click", "li", function () {
        var $li = $(this),
          market = $li.data("market");

        if ($li.hasClass("select")) {
          return false;
        } else {
          $(".broker-list").html("");
          $li.siblings().removeClass("select");
          $li.addClass("select");
          that.market = market;
          that.renderBrokers = that.getBrokersByMarket(that.market);
          that.renderHtml(that.renderBrokers);
        }
      });

      if (params.ib){
        $(".market-list .last").trigger( "click" );
      }

      // 选定券商
      $(".brokers").on("click", ".broker-list li .top", function () {
        var tid = $(this).data("tid"),
          bindUrl = $(this).data("bindurl"),
          broker = _.find(that.brokers, function (b) {
            return b.tid == tid;
          });

        xqBridge.trackEvent("SecuritiesList_bind" + tid);
        xqBridge.trackSNBEvent({
          pageId: 33,
          widgetId: 2,
          widgetPosition: $(this).parent().index()+1,
          foreignKey:tid || ''
        });
        // 看是否绑定手机号
        xqBridge.request({
          url: "/account/bind/show.json",
          data: {},
          type: "GET",
          success: function (ret) {
            var cb = function () {
              // 调用原生的
              if (Util.appVersion() >= 6.5 && broker && broker.page_type == 2 && broker.tid != "IB") {
                xqBridge.bindBroker({
                  broker: broker,
                  success: function () {
                    //Util.showError("绑定成功");
                  },
                  error: function () {
                    //Util.showError("绑定出错");
                  },
                  cancel: function () {
                    //Util.showError("取消绑定");
                  }
                });
              } else {
                if (broker.tid == "IB") {
                  bindUrl = "http://xueqiu.com/broker/ib/bind?step=1";
                }
                window.location.href = encodeURI(bindUrl);
              }
            };

            // 手机号
            if (ret.telephone) {
              cb();
            } else {
              xqBridge.trackEvent("bindPhone_bindSecurities");
              xqBridge.verifyTelephone({
                type: "bind",
                success: function () {
                  cb();
                }
              });
            }
          }
        });
      });

      // 雪球交易协议
      $(".agreement").on("click", function () {
        xqBridge.redirect({
          url: "http://xueqiu.com/law/broker-trade",
          type: "MODAL"
        });
        return false;
      });

      // 点击开户
      $(".open").on("click", function () {
        xqBridge.redirect({
          url: "http://xueqiu.com/broker/open/pingan",
          type: "PUSH"
        });
        return false;
      });
    }
  };
});
