/**
 * Created by flynngao on 15/2/8.
 */

define(function (require, exports, module) {
  xqBridge.trackEvent("myaccount");
  // 检查是否绑定
  var brokerListTemplate = require('template/trade/trade-broker-list.js');
  var accountInfoTemplate = require('template/trade/trade-account-info.js');
  var positionListTemplate = require('template/trade/trade-position-list.js');

  function errorHandler(e) {
    xqBridge.hideLoading();
    if (e && e.result_code == 60000) {
      return true;
    }
    else if (e && (e.result_code == 70005 || e.result_code == 70006)) {
      xqBridge.redirect({
        type: 'POP_CANCEL'
      });
    } else if (e && (e.error_description || e.msg)) {
      xqBridge.alert({
        message: e.msg || e.error_description,
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    } else {
      xqBridge.alert({
        message: '网络错误',
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    }
  }

  module.exports = {
    init: function () {
      var that = this;
      if(Util.parseParams().from){
        xqBridge.trackEvent('myaccount_' + Util.parseParams().from)
      }
      xqBridge.getUserInfo({
        success: function (userInfo) {
          if (userInfo) {
            window.userInfo = userInfo;
          } else {
            window.userInfo = {};
          }

          if (userInfo && !userInfo.uid) {
            xqBridge.hideLoading();

            xqBridge.alert({
              message: '请先登录雪球',
              success: function () {
                xqBridge.redirect({
                  type: 'POP_CANCEL'
                });
              }
            });
          }

          if (userInfo && userInfo.stColor) {
            $('body').addClass('oppositeColor');
          }
          if (!that.isBindEvent) {
            that.bindEvent();
            that.isBindEvent = true;
          }
          that.checkBind();
        },
        error: function (e) {
          errorHandler(e);
        }
      });
    },
    checkBind: function () {
      var that = this;
      Util.checkBindBroker({
        success: function (isBind) {
          if (isBind) {
            that.getAccountData(Util.parseParams()['scode'])
          } else {
            $('.unbind').removeClass('fn-hide');
            $('.my-broker-account').addClass('fn-hide');
            that.setToolbar(true);
            xqBridge.hideLoading();
          }
        },
        error: function (e) {
          errorHandler(e);
        }
      })
    },
    getAccountInfo: function () {
      var deffered = jQuery.Deferred();
      var that = this;
      xqBridge.request({
        url: "/tc/fund/query.json",
        type: "GET",
        data: {
          aid: that.aid,
          tid: that.tid
        },
        success: function (ret) {
          deffered.resolve(ret);
        },
        error: function (error) {
          deffered.reject(error);
        }
      });
      return deffered.promise();
    },
    getPositionList: function () {
      var deffered = jQuery.Deferred();
      var that = this;
      xqBridge.request({
        url: "/tc/position/list.json",
        type: "GET",
        data: {
          aid: that.aid
        },
        success: function (ret) {
          deffered.resolve(ret);
        },
        error: function (error) {
          deffered.reject(error);
        }
      });
      return deffered.promise();
    },

    /**
     * 数据为空时 渲染空数据
     */
    renderNullData: function () {
      $('.account-info').html(accountInfoTemplate({
        data: {
          accountInfo: {
            asset_balance: 0,
            begin_balance: null,
            buy_frozen_balance: null,
            cur: "CNY",
            current_balance: 0,
            enable_balance: 0,
            fetch_balance: 0,
            fetch_cash: null,
            foregift_balance: null,
            frozen_balance: null,
            fund_balance: null,
            market_value: 0,
            mortgage_balance: null,
            unfrozen_balance: null
          },
          total_income_balance: 0
        }
      }));

      $('.stocks-list tbody').html(positionListTemplate({
        data: {
          positions: []
        }
      }));
    },
    /**
     * 渲染广告 下拉刷新 toolbar 之类
     */
    renderWidgets: function () {
      var that  = this;

      that.setToolbar();
      $('.unbind').addClass('fn-hide');
      $('.my-broker-account').removeClass('fn-hide');

      // 下拉刷新
      if (that.myScroll) {
        $(".pullDownTime span").html(moment(new Date()).format("YYYY-MM-DD HH:mm:ss"));
        that.myScroll.refresh();
      }

      // 广告
      setTimeout(function () {
        seajs.use("ad-banner.js", function (ad) {
          ad.init("mySecurities_ad", "mySecurities_closeAd");
        });
      }, 200);

    },
    fixAccountInfoData: function (fundInfo, rebalancingInfo) {
      function fixNumber(number, postfix) {
        var numberStr = "";
        var style = "";

        postfix = postfix || "";

        if (_.isUndefined(number) || _.isNull(number)) {
          return {number: "-" + postfix, style: style};
        }

        if (postfix == "%") {
          number *= 100;
        }
        if (number > 0) {
          numberStr = "+" + number.toFixed(2)
          style = "stock_up";
        } else {
          numberStr = number.toFixed(2)
          if (number < 0) {
            style = "stock_down";
          }
        }

        if (postfix) {
          numberStr += postfix;
        }

        return {number: numberStr, style: style};
      }

      var data = {
        accountInfo: fundInfo.result_data[0]
      };

      var totalObj = fixNumber(rebalancingInfo.total_income_balance);
      data.total_income_balance = totalObj.number;
      data.totalIncomeStyle = totalObj.style;

      var dayRateObj = fixNumber(rebalancingInfo.day_income_rate, "%");
      var dayBalanceObj = fixNumber(rebalancingInfo.day_income_balance);

      data.day_income_balance = dayBalanceObj.number;
      data.day_income_rate = dayRateObj.number;
      data.dayIncomeStyle = dayBalanceObj.style;

      return data;
    },
    getAccountData: function () {
      var that = this;
      // 请求券商列表，顺带带了aid
      var brokerList = window.userInfo.brokerList;
      var broker = _.find(brokerList, function (b) {
        return b.is_default;
      });

      if (brokerList.length && !broker) {
        broker = _.first(brokerList);
        broker.is_default = true;
      }

      that.broker = broker;

      var aid = broker.aid;
      that.aid = aid;
      that.tid = broker.tid;

      //请求帐号内仓位数据以及仓位列表
      $('.broker-list').html(brokerListTemplate({
        data: {
          brokers: brokerList
        }
      }));

      $.when(that.getAccountInfo(), that.getPositionList())
        .then(function (fundInfo, rebalancingInfo) {
          if (fundInfo.result_code == 60000 && rebalancingInfo.result_code == 60000) {
            $('.account-info').html(accountInfoTemplate({
              data: that.fixAccountInfoData(fundInfo, rebalancingInfo)
            }));

            $('.stocks-list tbody').html(positionListTemplate({
              data: {
                positions: rebalancingInfo.result_data
              }
            }));
          } else {
            that.renderNullData();

            var ret = _.find([fundInfo, rebalancingInfo], function (r) {
              return r.result_code != 60000;
            });
            var msg = ret.msg || "获取数据出错，请下拉刷新重新获取。";
            Util.showError(msg);
          }

          xqBridge.hideLoading();
          that.renderWidgets();
        }).fail(function (ret) {
          xqBridge.hideLoading();
          if (ret) {
            if (ret.result_code == 70006) {
              that.renderNullData();
              that.renderWidgets();
              Util.showError("为确保交易安全，请下拉刷新重新登录证券账户");
            } else {
              if (ret.msg) {
                Util.showError(ret.msg);
              }
            }
          }
        });
    },
    setToolbar: function (isEmpty) {
      var that = this;
      var buttons = [
        {
          title: "下单",
          action: function () {
            xqBridge.trackEvent("mySecurities_trade");
            xqBridge.redirect({
              url: '/broker/trade',
              type: 'PUSH',
              success: function () {

              },
              cancel: function () {
                that.init();
              }
            })
          },
          highlight: true
        },
        {
          title: "委托单",
          action: function () {
            xqBridge.redirect({
              url: '/broker/orders?aid=' + that.aid,
              type: 'PUSH',
              success: function () {

              },
              cancel: function () {
                that.init();
              }
            })
          }
        },
        {
          title: "交易记录",
          action: function () {
            xqBridge.redirect({
              //url: '/broker/trade-history?aid=' + userInfo.brokerList[0].aid,
              url: '/broker/trade-history',
              type: 'PUSH',
              success: function () {

              },
              cancel: function () {
                that.init();
              }
            })
          }
        }
      ];

      // 银证转账字段在broker里面
      if (this.broker && this.broker.bank_trans) {
        buttons.push({
          title: "银证转账",
          action: function () {
            xqBridge.trackEvent("mySecurities_bank");
            xqBridge.redirect({
              url: '/broker/transfer',
              type: 'PUSH',
              success: function () {

              },
              cancel: function () {
                that.init();
              }
            });
          }
        });
      }

      if (Util.appVersion() > 6.2) {
        window.isSetToolbar = false;
      }

      if (isEmpty) {
        buttons = [];
      }
      xqBridge.setToolbar({
        name: "setToolbar",
        buttons: buttons
      });
    },

    bindEvent: function () {
      var that = this;
      var isIPhone = navigator.userAgent.indexOf("iPhone") > -1;

      xqBridge.setRightNavigationButton({
        title: "管理",
        action: function () {
          xqBridge.trackEvent("mySecurities_secManage");
          xqBridge.redirect({
            url: '/broker/config',
            type: "PUSH",
            success: function () {
              that.init();
            },
            cancel: function () {
              that.init();
            }
          })
        }
      });

      $('body').on('click', 'tbody tr', function (e) {
        xqBridge.trackEvent("mySecurities_stockClick");
        var symbol = $(this).data("symbol");
        if (symbol.indexOf("SH") > -1 || symbol.indexOf("SZ") > -1) {
          xqBridge.redirect({
            url: 'http://xueqiu.com/S/' + symbol,
            type: "PUSH",
            success: function () {

            },
            cancel: function () {
              that.init();
            }
          });
        } else {
          if (!$('tbody').hasClass('disabled')) {
            $('tbody').addClass('disabled');
            var tr = this;
            xqBridge.redirect({
              url: '/broker/trade?scode=' + $(tr).data('scode'),
              type: "PUSH",
              success: function () {

              },
              cancel: function () {
                $('tbody').removeClass('disabled');
                that.init();
              }
            })
          }
        }
      });

      // 拉到页面底部 弹上去
      if (isIPhone) {
        $('body').on('touchmove', function (e) {
          if (e.originalEvent.pageY > $(window).height() && that.myScroll && that.isRefreshing) {
            that.myScroll._resetPos(200);
          }
        });
      }

      if (window.iScroll && !this.myScroll) {
        var $pullDown = $("#pullDown");
        var $pullDownText = $(".pullDownText");
        var topOffset = $pullDown.outerHeight();
        function beforScrollStartHandler(e) {
          if (!isIPhone) {
            e.preventDefault();
          }
        }

        $(".pullDownTime span").html(moment(new Date()).format("YYYY-MM-DD HH:mm:ss"));

        this.myScroll = new window.iScroll('wrapper', {
          useTransition: !isIPhone,
          onBeforeScrollStart: beforScrollStartHandler,
          topOffset: topOffset,
          preventDefault: !isIPhone,
          onRefresh: function () {
            if ($pullDown.hasClass("loading")) {
              $pullDown.removeClass("loading");
              $pullDownText.html("下拉刷新");
            }
          },
          onScrollMove: function () {
            if (this.y > 5 && !$pullDown.hasClass("flip")) {
              $pullDown.addClass("flip");
              $pullDownText.html("释放刷新");
              that.isRefreshing = true;
              this.minScrollY = 0;
            } else if (this.y < 5 && $pullDown.hasClass("flip")) {
              $pullDown.removeClass("flip");
              $pullDownText.html("下拉刷新");
              this.minScrollY = -topOffset;
            }
          },
          onScrollEnd: function () {
            that.isRefreshing = false;
            // 由于未知原因  调用refresh之后会再次触发 onscrollend 通过this.y >= 0 来阻止重复刷新
            if ($pullDown.hasClass("flip") && this.y >= 0) {
              $pullDown.addClass("loading");
              $pullDownText.html("数据载入中...");
              that.init();
            }
          }
        });
      }

      // 切换券商
      $("body").on("click", ".broker-list .broker", function () {
        var $parent = $(this).parent();
        var $broker = $(this);
        var $container = $(".broker-list");

        if ($parent.hasClass("select-list")) {
          // 切换
          if ($parent.hasClass("selected")) {
            return false;
          }

          $parent.addClass("selected");
          $parent.siblings().removeClass("selected");

          xqBridge.trackEvent("mySecurities_switch");
          var aid = $broker.data("aid");
          xqBridge.showLoading();
          xqBridge.request({
            url: "/tc/securities/default.json",
            type: "POST",
            data: {
              changetoaid: aid
            },
            success: function (ret) {
              if (ret.result_code == 60000) {
                $(".select-list").hide();
                $(".overlay").hide();
                $container.removeClass("expand");
                that.init();
              }
            },
            error: function () {
              xqBridge.hideLoading();
              Util.showError("切换账号出错");
            }
          });

        } else {

          var $arrow = $(this).find(".arrow");
          if (!$arrow.length) {
            return false;
          }
          // 显示
          var $brokerList = $(".select-list");
          if ($brokerList.is(":hidden")) {
            $brokerList.show();
            $(".overlay").show();
            $container.addClass("expand");
          } else {
            $brokerList.hide();
            $(".overlay").hide();
            $container.removeClass("expand");
          }
        }
      });

      $("body").on("click", ".overlay", function () {
        $(this).hide();
        $(".broker-list").removeClass("expand");
        $(".select-list").hide();
      });

      $("body").on("click", ".bind-button", function () {
        xqBridge.trackEvent("mySecurities_bind");
        xqBridge.redirect({
          url: "/broker/bind-list",
          type: "MODAL",
          success: function () {
            that.init();
          }
        });
      });

      $("body").on("click", ".open-button", function () {
        xqBridge.trackEvent("mySecurities_open");
        xqBridge.redirect({
          url: "/broker/open-list",
          type: "MODAL",
          success: function () {
            that.init();
          }
        });
      });

      $("body").on("click", ".new", function () {
        xqBridge.trackEvent("mySecurities_guide");
        var href = $(this).find("a").attr("href");
        xqBridge.redirect({
          url: href,
          type: "PUSH"
        });
        return false;
      });
    }
  }
});