/**
 * Created by lyj on 6/24/15.
 */
define(function (require, exports, module) {
  var bindListTemplate = require('template/trade/open-list.js');
  var params = Util.parseParams();
  if (params.from) {
    xqBridge.trackEvent("openList_" + params.from);
  }

  var isIPhone = navigator.userAgent.indexOf("iPhone") > -1;
  module.exports = {
    init: function () {
      this.renderSlide();
      this.getCurrentMarket();
      this.getData();
      this.bindEvent();
      this.addNavBtn();
    },
    renderSlide: function () {
      var that = this;

      xqBridge.request({
        url: "/tc/snowx/ad/list.json",
        data: {
          ad_type: 1
        },
        success: function (ret) {
          var list = ret.result_data;
          var imageList = [];

          that.imgToUrl = {};

          _.each(list, function (item) {
            imageList.push({
              content: item.image_src +'?sortid=' + item.sortid,
              sortid: item.sortid
            });

            that.imgToUrl[item.image_src +'?sortid=' + item.sortid] = item.ad_url;
          });
          imageList.sort(function (a, b) {
            return a.sortid - b.sortid
          });
          if (imageList.length) {
            var width = $("body").width();
            var height = 300 / 960 * width;
            var top = height + 40 - 10;

            $("#iSlider-wrapper").css({
              height: height,
              backgroundImage: "url('./images/common_defaultimage_logo.png')",
              backgroundRepeat: "no-repeat",
              backgroundPosition: "center",
              backgroundSize: height / 2 + "px " + height / 2 + "px"
            });

            if (imageList.length > 1) {
              var opts = {
                type: 'pic',
                data: imageList,
                dom: document.getElementById("iSlider-wrapper"),
                animateType: "rotate",
                isLooping: true,
                duration: 3000,
                isAutoplay: true,
                initIndex: 0
              };
              var	islider = new iSlider(opts);

              $(".islider-dot-wrap").css({
                top: top
              });
            } else {
              $("#iSlider-wrapper").html('<img width="100%" src="' + imageList[0].content + '"/>');
            }
          } else {
            $("#iSlider-wrapper").hide();
          }
        }
      });
    },
    /**
     * 获取当前市场
     */
    getCurrentMarket: function () {
      var market = $(".market-list").find(".select").text();
      $(".market").text("");
    },
    getBrokersByMarket: function (market) {
      var tid = "IB";
      var brokers = [];
      if (market === "CN") {
        brokers = _.filter(this.brokers, function (broker) {
          return broker.tid != tid;
        });
      } else {
        brokers = _.filter(this.brokers, function (broker) {
          return broker.tid == tid;
        });
      }

      return brokers;
    },
    getData: function () {
      var that = this;
      this.market = params.market || "CN";

      if (Util.appVersion() < 6.6) {
        this.market = "CN";
        $("#iSlider-wrapper").css({
          "margin-top": "0px"
        });
      } else {
        $(".markets").show();
      }

      xqBridge.request({
        url: "/tc/snowx/opening/list.json",
        type: "GET",
        data: {},
        success: function (ret) {
          // 成功
          if (ret.result_code == 60000) {
            that.fixData(ret.result_data);

            var brokers = ret.result_data;
            that.brokers = brokers;
            that.renderBrokers = that.getBrokersByMarket(that.market);
            //不区分市场了
            //that.renderHtml(that.renderBrokers);
            that.brokers = that.brokers.map(function (item) {
              if (item.tid === "IB") {
                item.trader_name = "盈透证券(美/港股)"
              }
              return item
            });
            that.renderHtml(that.brokers);
          }
        },
        error: function (xhr) {
          Util.showError("获取数据失败，请检查网络");
        },
        complete: function () {
          xqBridge.hideLoading();
        }
      });
    },
    fixData: function (brokers) {
      _.each(brokers, function (broker) {
        if (broker.tags) {
          broker.tags = broker.tags.split(",");
        } else {
          broker.tags = [];
        }
      });
    },
    renderHtml: function (brokers) {
      $(".no-brokers").hide();
      if (brokers.length) {
        $('.brokers').html(bindListTemplate({
          data: {
            brokers: brokers
          }
        }));
      } else {
        $(".no-brokers").show();
      }
      //ios和android位置偏差
      if (!isIPhone) {
        $(".tags span").css({padding: "1px 6px"});
      }
    },
    bindEvent: function () {
      var that = this;

      Util.setPress(".broker-list li .top", "onpress");
      Util.setPress(".broker-list li .bottom", "onpress");

      $("#iSlider-wrapper").on("click", "img", function () {
        var src = decodeURIComponent(this.src);
        xqBridge.trackSNBEvent({
          pageId: 32,
          widgetId: 4,
          widgetPosition: src.substring(src.search('=')+1)
        });
        var url = that.imgToUrl[src];

        xqBridge.redirect({
          url: url,
          type: "PUSH"
        });
      });

      // 切换市场
      $(".market-list").on("click", "li", function () {
        var $li = $(this),
          market = $li.data("market");

        if ($li.hasClass("select")) {
          return false;
        } else {
          $(".broker-list").html("");
          $li.siblings().removeClass("select");
          $li.addClass("select");
          that.market = market;
          that.renderBrokers = that.getBrokersByMarket(that.market);
          that.renderHtml(that.renderBrokers);
        }
      });
      if (params.ib){
        $(".market-list .last").trigger( "click" );
      }
      // 券商活动
      $(".brokers").on("click", ".broker-list li", function (e) {
        var $target = $(e.target);
        var tid = $(this).data("tid");
        // 事件统计
        if ($target.hasClass("button")) {
          xqBridge.trackEvent("SecuritiesList_openButton_" + tid);
          xqBridge.trackEvent("openButton_" + tid);
          xqBridge.trackSNBEvent({
            pageId: 32,
            widgetId: 2,
            widgetPosition: $(this).index()+1,
            foreignKey: tid || ''
          });
        } else {
          xqBridge.trackEvent("SecuritiesList_open" + tid);
          xqBridge.trackSNBEvent({
            pageId: 32,
            widgetId: 5,
            widgetPosition: $(this).index()+1,
            foreignKey: tid || ''
          });
        }

        var cb = function () {
          // 开户 不是活动
          xqBridge.showLoading();

          var notifyType = 1;
          if (Util.appVersion() > 6.7) {
            notifyType = 2;
          }

          // 请求跳转数据
          xqBridge.request({
            url: "/tc/snowx/opening.json",
            data: {
              tid: tid,
              notify_type: notifyType
            },
            type: "POST",
            success: function (ret) {
              if (ret.result_code == 60000) {
                var obj = ret.result_data;
                // 点按钮  直接开户
                if ((obj.preSignUrl && !$target.hasClass("button")) || tid == "IB") {
                  var url = obj.preSignUrl + "?fromList=true";

                  xqBridge.showLoading();
                  xqBridge.redirect({
                    url: "/broker/proxy?url=" + encodeURIComponent(url),
                    type: "MODAL"
                  });
                } else {
                  var isIPhone = navigator.userAgent.indexOf("iPhone") > -1,
                    notifyUrl = isIPhone ? obj.ios_notify_url : obj.apk_notify_url,
                    downloadUrl = isIPhone ? obj.ios_download_url : obj.apk_download_url,
                    openType = "",
                    array = notifyUrl.split("-");

                  if (obj.notify_type) {
                    notifyType = obj.notify_type;
                  } else {
                    notifyType = 1;
                  }

                  if (array.length === 2 && !isIPhone) {
                    notifyUrl = array[1];
                    openType = array[0];
                  }

                  if (isIPhone && tid !== "GJZQ" && tid !== "PINGAN") {
                    that.toast.show("开始安装后，请按Home键在桌面查看");
                  }

                  xqBridge.wakeApp({
                    notifyUrl: notifyUrl,
                    downloadUrl: downloadUrl,
                    openType: openType,
                    notifyType: notifyType
                  });
                }
              } else {
                Util.showError(ret.msg);
              }
            },
            error: function (xhr) {

            },
            complete: function () {
              xqBridge.hideLoading();
            }
          });
        };

        // 看是否绑定手机号
        xqBridge.request({
          url: "/account/bind/show.json",
          data: {},
          type: "GET",
          success: function (ret) {
            // 手机号
            if (ret.telephone) {
              cb();
            } else {
              xqBridge.trackEvent("bindPhone_bindSecurities");
              xqBridge.verifyTelephone({
                type: "bind",
                success: function () {
                  cb();
                }
              });
            }
          }
        });
      });

      // 雪球交易协议
      $(".agreement").on("click", function () {
        xqBridge.redirect({
          url: "/broker/terms",
          type: "MODAL"
        });
        return false;
      });
    },
    addNavBtn: function () {
      xqBridge.setRightNavigationButton({
        title: "开户帮助",
        action: function () {
          xqBridge.trackSNBEvent({
            pageId: 32,
            widgetId: 3,
            widgetPosition: 1
          });
          xqBridge.redirect({
            url: 'http://xueqiu.com/broker/trade/qa',
            type: "PUSH",
            success: function () {
            },
            cancel: function () {
            }
          })
        }
      })
    },
    //toast a message
    toast: {
      show: function (message) {
        "use strict";
        var $toast = $(".toast");
        if ($toast.length > 0) {
          setTimeout(function () {
            $toast.show();
            setTimeout(function () {
              $toast.hide();
            }, 3000);
          }, 2000);
        } else {
          $toast = $("<div class='toast'>" + message + "</div>");
          $("body").append($toast);
          $toast.css({
            backgroundColor: "rgba(0,0,0,.7)",
            fontSize: "12px",
            position: "fixed",
            bottom: "15%",
            left: "15%",
            color: "#fff",
            padding: "10px",
            display: "none",
            textAlign: "center",
            width: "70%",
            marginLeft: '-10px'
          });
          setTimeout(function () {
            $toast.show();
            setTimeout(function () {
              $toast.hide();
            }, 3000);
          }, 2000);
        }
      },
      hide: function () {
        "use strict";
        if ($(".toast").length > 0) {
          $(".toast").hide();
        }
      }
    }
  }
});
