/**
 * Created by lyj on 2/9/15.
 */

define(function (require, exports, module) {
  // 统计
  xqBridge.trackEvent("orders");

  function errorHandler(e) {
    xqBridge.hideLoading();
    if (e && e.result_code == 60000) {
      return true;
    }
    else if (e && (e.result_code == 70005 || e.result_code == 70006)) {
      xqBridge.redirect({
        type: 'POP_CANCEL'
      });
    } else if (e && (e.error_description || e.msg)) {
      xqBridge.alert({
        message: e.msg || e.error_description,
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    } else {
      xqBridge.alert({
        message: '网络错误',
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    }
  }

  var orderAction = {
    "BUY": "买入",
    "SELL": "卖出"
  };

  var orderProp = {
    "BS": "买卖",
    "AS": "配股",
    "TRANS": "转托",
    "ORDER": "申购",
    "BORDER": "回购",
    "RATION": "配售",
    "SPECIFY": "指定",
    "TTRANS": "转股",
    "BSELL": "回售",
    "INTEREST": "股息"
  };

  var orderStatus = {
    "NO_REPORT": "未报",
    "WAIT_REPORT": "待报",
    "REPORTED": "已报",
    "WAIT_WITHDRAW": "已报待撤",
    "PART_WAIT_WITHDRAW": "部成待撤",
    "PART WITHDRAW": "部撤",
    "WITHDRAWED": "已撤",
    "PART_CONCLUDED": "部成",
    "CONCLUDED": "已成",
    "INVALID": "废单",
    "PURCHASED": "已申购"
  };

  var orderStatusIB = {
    "PendingSubmit": "未报",
    "PendingCancel": "待撤",
    "PreSubmitted": "待报",
    "Submitted": "已报",
    "ApiCanceled": "正撤",
    "Cancelled": "已撤",
    "Filled": "已成",
    "Inactive": "废单"
  };

  // 根据订单状态获取操作状态
  function getOpByStatus (status) {
    var opName = "",
      color = "",
      op = "",
      statusColor = "";

    if (_.indexOf(["NO_REPORT", "REPORTED", "PART_CONCLUDED", "WAIT_WITHDRAW"], status) > -1) {
      opName = "撤单";
      color = "blue";
      op = "cancel";
    } else if (_.indexOf(["WAIT_REPORT", "WAIT_REPORT", "PART_WAIT_WITHDRAW"], status) > -1) {
      opName = "撤单";
      color = "gray";
      op = "cancel";
    } else if (_.indexOf(["PART WITHDRAW", "WITHDRAWED", "CONCLUDED", "INVALID", "PURCHASED"], status) > -1) {
      opName = "撤单";
      color = "gray";
      op = "cancel";
      statusColor = "gray";
    }

    return {opName: opName, color: color, op: op, statusColor: statusColor};
  }

  function getOpByStatusForIB(status) {
    var canEdit = false;
    // 这几个状态  可以撤单 可以改单
    if (_.indexOf(["PendingSubmit", "PreSubmitted", "Submitted"], status) > -1) {
      canEdit = true;
    }

    return {canEdit: canEdit};
  }

  // 修复数据
  function fixData(order, isIB) {
    order.actionName = orderAction[order.action];
    order.opropName = orderProp[order.oprop];
    order.statusName = orderStatus[order.ostatus];
    order.price = parseFloat(order.price).toFixed(3);
    if (order.cprice) {
      order.cprice = parseFloat(order.cprice).toFixed(3);
    } else {
      order.cprice = "0.000";
    }

    if (!order.camount) {
      order.camount = 0;
    }

    if (isIB) {
      order.statusName = orderStatusIB[order.ostatus];
    }

    if (order.oprop && order.oprop.indexOf("MKT") > -1) {
      order.price = "市价";
    }

    if (order.otime && !_.isNull(order.otime)) {
      var timeStr = order.otime + "";
      if (timeStr.length === 5) {
        timeStr = "0" + timeStr;
      }
      order.time = timeStr.slice(0, 2) + ":" + timeStr.slice(2, 4) + ":" + timeStr.slice(4, 6);
    } else {
      order.time = "";
    }

    var opInfo;
    if (isIB) {
      opInfo = getOpByStatusForIB(order.ostatus);
    } else {
      opInfo = getOpByStatus(order.ostatus);
    }

    _.extend(order, opInfo);
  }

  var tradeList = require('template/trade/orders.js');


  module.exports = {
    init: function () {
      var that = this;
      that.tid = Util.parseParams().tid;
      that.isIB = that.tid === "IB";

      if (that.isIB) {
        $(".trade-list")
          .find("th")
          .eq(1)
          .text("股票/代码");
      }

      Util.checkAndInit({
        isIB: that.isIB,
        success: function (data) {
          if (!errorHandler(data)) {
            return;
          }
          var brokerList = data.result_data;
          var broker = Util.getBroker(brokerList, that.tid);


          if (broker) {
            that.broker = broker;
            that.aid = broker.aid;
            that.renew_url = broker.renew_url;
            that.getOrders();
            if (!that.isBindEvent) {
              that.bindEvent();
            }
          }
        },

        // 初始化失败退出交易页面
        cancel: function () {
          xqBridge.redirect({
            type: "POP_CANCEL"
          });
        }
      });
    },
    getOrders: function () {
      var that = this;

      xqBridge.request({
        url: "/tc/snowx/order/list.json",
        data: {
          aid: that.aid,
          tid: that.tid,
          sort: 1
        },
        type: "GET",
        success: function (ret) {
          errorHandler(ret);
          var tradeListData = ret.result_data;

          if (tradeListData.length) {
            _.each(tradeListData, function (order) {
              fixData(order, that.isIB);
            });

            var html = tradeList({
              data: {
                tradeList: tradeListData,
                isIB: that.isIB
              }
            });

            $('.trade-list tbody').html(html);
            $(".trade-list").show();
          } else {
            var noDataHtml = '<tr><td class="noData" colspan="5"><img src="./images/no-history.png"/><p class="title">暂无委托单</p></td></tr>'
            $('.trade-list tbody').html(noDataHtml);
            $(".trade-list").show();
          }

          $(".clone")
            .empty()
            .append($(".trade-list").clone())
            .show();
        },
        complete: function () {
          xqBridge.hideLoading();
          if (that.myScroll) {
            $(".pullDownTime span").html(moment(new Date()).format("YYYY-MM-DD HH:mm:ss"));
            that.myScroll.refresh();
          }
        }
      });
    },
    bindEvent: function () {
      this.isBindEvent = true;
      var that = this;
      var lastClickTime = 0;

      $("body").on("click", function (e) {
        var $target = $(e.target);

        if (!$target.hasClass("more") && !$target.parents(".ib-operate").length) {
          $(".ib-operate").hide();
          return false;
        }
      });

      // ib 点击更多  弹出操作列表
      $("body").on("click", ".more", function (e) {
        var current = new Date().getTime();
        var delta = current - lastClickTime;
        lastClickTime = current;

        if (delta > 400) {
          var $operate = $(this).parent().find(".ib-operate");

          if ($operate.is(":visible")) {
            $operate.hide();
          } else {
            $(".ib-operate").hide();
            $operate.show();
          }
        }
        return false;
      });

      // 修改订单
      $("body").on("click", ".edit-order", function () {
        var $tr = $(this).parents("tr");
        var action = $tr.data("action");
        var scode = $tr.data("scode");
        var price = $tr.data("price");
        var amount = $tr.data("amount");
        var oid = $tr.data("oid");
        var etype = $tr.data("etype") || "";
        var symbol = scode;
        var tid = that.tid;

        if (etype && (/^SH|Z/.test(etype))) {
          symbol = etype.slice(0, 2) + scode;
        }

        var params = {
          scode: symbol,
          price: price,
          amount: amount,
          oid: oid,
          tid: tid,
          action: action
        };

        var current = new Date().getTime();
        var delta = current - lastClickTime;
        lastClickTime = current;

        if (delta > 400) {
          xqBridge.redirect({
            url: "http://xueqiu.com/broker/trade?" + $.param(params),
            type: "PUSH"
          });
        }
      });

      // 重新下单
      $("body").on("click", ".reorder", function () {
        var $tr = $(this).parents("tr");
        var scode = $tr.data("scode");
        var etype = $tr.data("etype") || "";
        var symbol = scode;
        var tid = that.tid;

        if (etype && (/^SH|Z/.test(etype))) {
          symbol = etype.slice(0, 2) + scode;
        }

        var params = {
          scode: symbol,
          tid: tid
        };

        var current = new Date().getTime();
        var delta = current - lastClickTime;
        lastClickTime = current;

        if (delta > 400) {
          xqBridge.redirect({
            url: "http://xueqiu.com/broker/trade?" + $.param(params),
            type: "PUSH"
          });
        }
      });

      if (window.iScroll && !this.myScroll) {
        var $pullDown = $("#pullDown");
        var $pullDownText = $(".pullDownText");
        var topOffset = $pullDown.outerHeight();
        var isIPhone = navigator.userAgent.indexOf("iPhone") > -1;
        function beforScrollStartHandler(e) {
          if (!isIPhone) {
            e.preventDefault();
          }
        }

        $(".pullDownTime span").html(moment(new Date()).format("YYYY-MM-DD HH:mm:ss"));

        this.myScroll = new window.iScroll('wrapper', {
          useTransition: !isIPhone,
          onBeforeScrollStart: beforScrollStartHandler,
          topOffset: topOffset,
          preventDefault: !isIPhone,
          onRefresh: function () {
            if ($pullDown.hasClass("loading")) {
              $pullDown.removeClass("loading");
              $pullDownText.html("下拉刷新");
            }
          },
          onScrollMove: function () {
            if (this.y > 5 && !$pullDown.hasClass("flip")) {
              $pullDown.addClass("flip");
              $pullDownText.html("释放刷新");
              this.minScrollY = 0;
            } else if (this.y < 5 && $pullDown.hasClass("flip")) {
              $pullDown.removeClass("flip");
              $pullDownText.html("下拉刷新");
              this.minScrollY = -topOffset;
            }
          },
          onScrollEnd: function () {
            // 由于未知原因  调用refresh之后会再次触发 onscrollend 通过this.y >= 0 来阻止重复刷新
            if ($pullDown.hasClass("flip") && this.y >= 0) {
              $pullDown.addClass("loading");
              $pullDownText.html("数据载入中...");
              that.init();
            }
          }
        });
      }


      // bind op event
      $(".trade-list").on("click", ".btn, .cancel-order", function () {
        var current = new Date().getTime();
        var delta = current - lastClickTime;
        lastClickTime = current;

        if (delta < 400) {
          return false;
        }

        var $parent = $(this).parent(),
          $this = $(this),
          $tr = $this.parents("tr");

        // 灰色撤单不可点击
        if ($parent.hasClass("gray")) {
          return false;
        }
        xqBridge.trackSNBEvent({
          pageId: 37,
          widgetId: 2,
          widgetPosition: $parent.parent().index()+1,
          foreignKey: that.tid || ''
        });
        if ($this.data("disabled")) {
          return false;
        }

        // 交易 跳转交易页面
        if ($this.hasClass("trade")) {
          xqBridge.redirect({
            url: "/broker/trade?scode=" + $tr.data("scode"),
            type: "PUSH"
          });
        } else {
          $this.data("disabled", true);
          // 撤单
          var title = "撤销委托单",
            sname = $.trim($tr.data("sname")),
            scode = $.trim($tr.data("scode")),
            price = $.trim($tr.data("price")),
            amount = $.trim($tr.data("amount")),
            etype = $.trim($tr.data("etype")),
            oid = $tr.data("oid");

          var message = ""
            + "股票：" + sname + " " + scode + "\n"
            + "价格：" + price + "\n"
            + "数量：" + amount;

          xqBridge.confirm({
            title: title,
            message: message,
            alignment: "left",
            success: function () {

              function request(token) {
                // 调用撤单接口
                xqBridge.request({
                  url: "/tc/snowx/order/cancel.json",
                  type: "POST",
                  data: {
                    oid: oid + "",
                    aid: that.aid,
                    tid: that.tid,
                    etype: etype,
                    write_access_token: token
                  },
                  success: function (ret) {
                    if (ret.result_code == 60000) {
                      xqBridge.showLoading();
                    } else {
                      Util.showError(ret.msg);
                    }
                    that.getOrders();
                  },
                  error: function (error) {
                    if (error && error.error_description) {
                      Util.showError(error.error_description);
                    } else {
                      Util.showError("撤单出错");
                    }
                    that.getOrders();
                  },
                  complete: function () {
                    $this.data("disabled", false);
                  }
                });
              }

              // 检查write_token 是否过期
              Util.getAndRefreshToken({
                aid: that.aid,
                renew_url: that.renew_url,
                broker: that.broker,
                success: function (token) {
                  request(token);
                },
                cancel: function () {
                  //$this.data("disabled", false);
                }
              });
            },
            cancel: function () {
              $this.data("disabled", false);
            }
          });
        }
      });
    }
  };
});