define(function (require, exports, module) {

  var brokerList = require('template/trade/trade-broker-list.js');
  var priceInfoList = require('template/trade/trade-price-info.js');
  var wudangList = require('template/trade/trade-wudang.js');

  var specialBrokers = ["IB"];
  var params = Util.parseParams();
  var interval;

  function errorHandler(e) {
    xqBridge.hideLoading();
    if (e && e.result_code == 60000) {
      return true;
    }
    else if (e && (e.result_code == 70005 || e.result_code == 70006)) {
      xqBridge.redirect({
        type: 'POP_CANCEL'
      });
    } else if (e && (e.error_description || e.msg)) {
      xqBridge.alert({
        message: e.msg || e.error_description,
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    } else {
      xqBridge.alert({
        message: '网络错误',
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    }
  }

  /**
   * 简单校验器
   */
  var Validator = function (rules) {
    this.rules = rules;
  };

  /**
   * 全规则校验，返回提示文案
   */
  Validator.prototype.validateAll = function () {
    for (var name in rules) {
      //if (!!rules[name].submitskip || !$('.' + name).length) {
      //  continue;
      //}
      if (rules[name].rule() !== true) {
        Util.toast.show(rules[name].rule());
        return false;
      }
    }
    return true;
  };

  /**
   * 根据ID进行校验
   */
  Validator.prototype.validate = function (rulename) {
    if (this.rules[rulename]) {
      return this.rules[rulename].rule();
    }
    return true;
  };

  var rules = {
    "scode": {
      "rule": function () {
        var val = $('#scode').data('symbol');

        if (!val) {
          return "未选择股票";
        }
        return true;
      }
    },
    "price": {
      "rule": function () {
        var val = $('.price').val() || params.price;
        if (!val) {
          return "价格不能为空";
        } else if (!/^-?\d+\.?\d*$/.test(val)) {
          return "价格必须是数字";
        }
        return true;
      }
    },
    "amount": {
      "rule": function () {
        var val = $('.amount').val();
        if (!val) {
          return "数量不能为空";
        } else if (!/^-?\d+\.?\d*$/.test(val)) {
          return "数量必须是数字";
        }
        return true;
      }
    }
  };

  module.exports = {
    init: function () {
      xqBridge.trackEvent("trade");
      this.action = params.action || "BUY";
      this.symbol = params.scode;
      this.etype = Util.getEtypeBySymbol(this.symbol);
      this.isUSHK = this.etype === "USEX" || this.etype === "HKEX";
      this.tid = params.tid;
      this.type = "LMT"; // 交易类型  市价单 MKT 限价单 LMT
      this.isEdit = false;
      this.isVisible = true;
      this.updateBodyClass();
      this.coupon = {};
      this.isPurchase = false;
      this.quote = {}; //股票信息
      this.couponList = [];
      this.hasCoupon = 'init';
      this.validate = new Validator(rules);
      if (params.subscribeCode) {
        this.subscribeInit();
      } else {
        this.normalInit();
      }
    },
    subscribeInit: function () {
      var that = this;
      that.subscribe = true;
      $("#submit-order").find('span').html("申购");
      $(".trade-price").hide();
      $(".trade-info").not(".new-stock").remove();
      $(".new-stock").show();
      $(".commission").hide();
      $('.trade-stock').addClass("hold");
      $('.trade-action').hide();
      $(".subscribeCode").text(params.subscribeCode);
      $(".price").text(params.price + "元");
      $("#scode").attr('readonly', true);
      $('.amount-select').addClass('sub');
      $('.subtip').show();
      //$('.trade-stock').css({marginTop: "53px"});
      $('.btn').addClass('new-stock-button');
      xqBridge.showLoading();
      xqBridge.getUserInfo({
        success: function (userInfo) {
          if (userInfo && userInfo.stColor) {
            $('body').addClass('oppositeColor');
          }
          Util.checkBindBroker({
            success: function (ret) {
              that.brokers = ret.result_data || [];
              that.changeBroker(that.tid);
              xqBridge.hideLoading();
              $('.trade').removeClass('fn-hide');
              // 初始化symbol
              if (params.scode) {
                $('#scode').val(params.name + ' ' + params.scode).data('symbol', params.scode).data('name', params.name);
                that.getStockAmount();
              }
              if (interval) {
                clearInterval(interval);
              }
              $("#container").find("*").off();
              that.bindEvent();
            },
            error: function (e) {
              xqBridge.redirect({
                type: "POP_CANCEL"
              });
            }
          });
        },
        error: function (e) {
          errorHandler(e);
        }
      });
    },
    normalInit: function () {
      var that = this;
      this.subscribe = false;
      if (this.action === "SELL") {
        $("#submit-order span").html("卖  出");
        $('.trade-action .select').removeClass('select');
        $(".trade-action button:last").addClass('select');
      }
      if (this.tid === "IB") {
        $(".commission").hide();
      }
      xqBridge.showLoading();
      xqBridge.getUserInfo({
        success: function (userInfo) {
          if (userInfo) {
            window.userInfo = userInfo;
          } else {
            window.userInfo = {};
          }
          if (userInfo && userInfo.stColor) {
            $('body').addClass('oppositeColor');
          }
          Util.checkBindBroker({
            success: function (ret) {
              that.brokers = ret.result_data || [];
              that.changeBroker(that.tid);
              xqBridge.hideLoading();
              $('.trade').removeClass('fn-hide');
              that.caculateStyle();
              that.updateBodyClass();
              // 初始化symbol
              if (params.scode) {
                that.getPanKouInfo(params.scode, true);
                if (params.oid) {
                  that.isEdit = true;
                  $("#scode")
                    .attr("disabled", "disabled")
                    .css({
                      "background-color": "#fff"
                    });
                }
              } else {
                $('.wudang-prices').html(wudangList({
                  data: that.fixData({})
                })).addClass("no-data");
              }
              that.bindEvent();
            },
            error: function (e) {
              xqBridge.redirect({
                type: "POP_CANCEL"
              });
            }
          });
        },
        error: function (e) {
          errorHandler(e);
        }
      });
    },
    updateBodyClass: function () {
      var $body = $("body");

      $body.removeClass("BUY");
      $body.removeClass("SELL");
      $body.addClass(this.action);

      if (this.isUSHK) {
        $("body").addClass("USHK");
        $(".mkt-price").text("市价");
      } else {
        $("body").removeClass("USHK");
        $(".mkt-price").text("最优五档即时成交剩余撤销");
      }

      if (this.broker && this.broker.tid && _.indexOf(specialBrokers, this.broker.tid) > -1) {
        $body.addClass("special-broker");
      } else {
        $body.removeClass("special-broker");
      }
    },
    caculateStyle: function () {
      var width = $(document).width();

      var textWidth = width - 110 - width * 0.06 - 80 - 15;
      if (this.isUSHK) {
        textWidth += 110;
      }
      $(".price, .amount").css({
        width: textWidth
      });

      var $typeGroup = $(".type-group");
      var offset = $typeGroup.offset();
      var outerHeight = $typeGroup.outerHeight();
      var outerWidth = $typeGroup.outerWidth();
      if (offset) {
        $(".trade-type").css({
          top: offset.top + outerHeight,
          left: 0,
          width: outerWidth,
          "padding-left": width * 0.03
        });
      }
    },
    getBrokers: function (brokers, tid, etype) {
      var that = this;
      if (brokers.length) {
        if (etype) {
          brokers = _.filter(brokers, function (b) {
            return b.etype && b.etype.indexOf(etype) > -1;
          });
        }
      }
      if (brokers.length) {
        // 先找默认的券商
        var defaultBroker = _.find(brokers, function (b) {
          return b.is_default;
        });

        if (!defaultBroker) {
          defaultBroker = _.first(brokers);
          defaultBroker.is_default = true;
        }

        // 再找指定tid的券商
        if (tid) {
          var broker = _.find(brokers, function (b) {
            return b.tid == tid;
          });

          // 找到之后设置指定券商is_default
          if (broker) {
            broker.is_default = true;

            if (defaultBroker && defaultBroker.tid != tid) {
              defaultBroker.is_default = false;
            }
          } else {
            noneSub();
          }
        }
      } else {
        noneSub();
      }
      function noneSub() {
        if (that.subscribe) {
          brokers = [];
          if (Util.appVersion() < 7.2) {
            xqBridge.confirm({
              title: "提示",
              message: "请更新最新雪球客户端",
              alignment: "left",
              success_button: "更新",
              cancel_button: "返回",
              success: function () {
                window.location.href = "http://xueqiu.com/xz";
              },
              cancel: function () {
                xqBridge.redirect({
                  type: "POP"
                })
              }
            })
          } else {
            xqBridge.confirm({
              title: "提示",
              message: "新股申购只支持平安证券，请添加平安证券后再试。",
              alignment: "left",
              success_button: "添加",
              cancel_button: "返回",
              success: function () {
                xqBridge.redirect({
                  type: "MODAL",
                  url: "/broker/bind-list"
                })
              },
              cancel: function () {
                xqBridge.redirect({
                  type: "POP"
                })
              }
            })
          }
        }
      }

      return brokers;
    },
    checkCoupon: function () {
      var that = this;
      var commission = parseFloat($(".commission-amount").text());
      if (that.symbol === "799999") {
        return false;
      }
      if (that.subscribe) {
        $(".coupon").hide();
        return false;
      }
      if (that.hasCoupon === 'init' && that.broker) {
        xqBridge.request({
          url: '/tc/snowx/reward/best/query.json',
          data: {
            tid: that.broker.tid,
            commission: parseFloat(commission)
          },
          success: function (data) {
            if (data.result_code === '60000') {
              xqBridge.request({
                url: '/tc/snowx/reward/owner/query.json',
                success: function (data) {
                  if (data.result_code === '60000') {
                    if (data.result_data === null || data.result_data.length === 0) {
                      $(".coupon").css({display: 'block'});
                      $("#coupon-text").text("暂无券可用").css({color: "#ccc"});
                      that.coupon = {};
                      that.hasCoupon = "keep";
                    } else {
                      that.couponList = data.result_data;
                      that.hasCoupon = "yep";
                      that.checkCoupon();
                    }
                  } else {
                    that.hasCoupon = false;
                    $(".coupon").hide();
                  }
                },
                error: function (err) {
                  that.hasCoupon = false;
                  $(".coupon").hide();
                }
              })
            } else if (data.result_code === '74007') {
              $(".coupon").css({display: 'block'});
              $("#coupon-text").text("今日交易券使用已满3次").css({color: "#ccc"});
              $(".coupon i").hide();
              that.coupon = {};
              $(".coupon").off('click');
              that.hasCoupon = 'keep';
            } else {
              that.hasCoupon = false;
              $(".coupon").hide();
            }
          },
          error: function () {
            that.hasCoupon = false;
            $(".coupon").hide();
          }
        });
      } else if (that.hasCoupon === "yep") {
        var couponArray = that.couponList;
        var bestCoupon = {};
        if (that.isPurchase) {
          couponArray = couponArray.filter(function (item) {
            return item.status === "ACTIVE" && item.name !== "亏损抵用券"
          });
        }
        couponArray.filter(function (item) {
          return item.status === "ACTIVE"
        }).reverse().map(function (coupon) {
          if (_.isEmpty(bestCoupon)) {
            bestCoupon = coupon;
          } else {
            var thisTime = moment(coupon.expiryDate);
            var thatTime = moment(bestCoupon.expiryDate);
            if (thisTime.diff(thatTime, 'days') < 0) {
              bestCoupon = coupon;
            } else if (thisTime.diff(thatTime, 'days') === 0) {
              if (coupon.name === bestCoupon.name) {
                if (coupon.faceValue > bestCoupon.faceValue) {
                  bestCoupon = coupon;
                }
              } else {
                if (coupon.name === "交易体验券") {
                  bestCoupon = coupon;
                }
              }
            }
          }
        });
        if (couponArray.length === 0) {
          $(".coupon").hide();
          return false;
        } else {
          $(".coupon").css({display: 'block'});
          $("#coupon-text")
            .css({color: "#FF7700"})
            .text(bestCoupon.name + "(￥" + bestCoupon.faceValue + ")");
          $(".coupon i").show();
        }
        that.coupon = bestCoupon;
      } else if (that.hasCoupon === "keep") {
        return false;
      } else {
        that.hasCoupon = false;
        $(".coupon").hide();
      }
    },
    //检查券商帐号绑定
    checkBind: function (order) {
      var that = this;
      if (that.broker) {
        if (that.broker.status == "SUCCESS") {
          xqBridge.trackEvent("trade_confirmAlert");
          xqBridge.trackSNBEvent({
            pageId: 36,
            widgetId: 16,
            widgetPosition: 1,
            foreignKey: that.tid || ''
          });
          xqBridge.confirm({
            title: order.title,
            message: order.message,
            alignment: "left",
            success: function () {
              xqBridge.trackSNBEvent({
                pageId: 36,
                widgetId: 17,
                widgetPosition: 1,
                foreignKey: that.tid || ''
              });
              // 取交易token
              Util.getAndRefreshToken({
                aid: that.broker.aid,
                renew_url: that.broker.renew_url,
                broker: that.broker,
                from: "trade",
                success: function (token) {
                  xqBridge.showLoading();
                  var orderParams = {
                    aid: that.broker.aid,
                    tid: that.broker.tid,
                    action: order.action,
                    scode: order.scode,
                    sname: order.sname,
                    price: order.price,
                    amount: order.amount,
                    etype: order.etype,
                    otype: that.type,
                    write_access_token: token
                  };
                  if (that.type === "MKT") {
                    orderParams.oprop = that.oprop;
                  }
                  if (params.oid) {
                    orderParams.oid = params.oid;
                  }
                  if (that.subscribe && params.subscribeCode) {
                    orderParams.scode = params.subscribeCode;
                  }
                  xqBridge.request({
                    url: '/tc/snowx/order/place.json',
                    type: 'POST',
                    data: orderParams,
                    timeout: 10000,
                    success: function (data) {
                      if (data.result_code == 60000) {
                        var cb = function (status) {
                          var obj = {
                            sname: order.sname,
                            symbol: that.symbol,
                            action: order.action,
                            price: order.price,
                            oid: data.result_data[0].oid,
                            tid: that.broker.tid,
                            otype: that.type
                          };
                          var commission = parseFloat($(".commission-amount").text());
                          if (status === "success") {
                            obj.coupon = order.coupon.faceValue;
                            obj.couponType = order.coupon.name;
                          }
                          if (that.subscribe) {
                            obj.newStock = true;
                          }
                          window.location.href = "./order-success.html?" + $.param(obj);
                        };
                        // 清除定时器
                        if (interval) {
                          clearInterval(interval);
                        }
                        //锁定佣金券
                        if ($(".coupon").is(':visible') && !_.isEmpty(order.coupon)) {
                          var transParams = {
                            reward_owner_id: order.coupon.rewardOwnerId,
                            tid: that.broker.tid,
                            aid: that.broker.aid,
                            oid: data.result_data[0].xoid
                          };
                          xqBridge.request({
                            url: '/tc/snowx/reward/lock.json',
                            type: 'POST',
                            data: transParams,
                            success: function (data) {
                              // 买股票自动加入自选股
                              if (that.action === "BUY") {
                                that.followStock(that.symbol, cb, "success");
                              } else {
                                cb("success");
                              }
                            },
                            error: function (err) {
                              // 买股票自动加入自选股
                              if (order.action === "BUY") {
                                that.followStock(that.symbol, cb, "success");
                              } else {
                                cb();
                              }
                            }
                          })
                        } else {
                          // 买股票自动加入自选股
                          if (order.action === "BUY") {
                            that.followStock(that.symbol, cb);
                          } else {
                            cb();
                          }
                        }
                      } else if (data.result_code === "70109") {
                        xqBridge.confirm({
                          title: "提示",
                          message: "您的可用资金不足，请先进行银证转账后再试。",
                          alignment: "left",
                          success_button: "去转账",
                          success: function () {
                            xqBridge.redirect({
                              url: '/broker/transfer',
                              type: "PUSH"
                            })
                          }
                        })
                      } else if (data.result_code === "70105") {
                        xqBridge.confirm({
                          title: "提示",
                          message: "您未做指定交易,是否进行指定交易.",
                          alignment: "left",
                          success_button: "下单指定",
                          success: function () {
                            var orderParams = {
                              aid: that.broker.aid,
                              tid: that.broker.tid,
                              action: "BUY",
                              scode: "799999",
                              sname: "799999",
                              price: 1,
                              amount: 1,
                              etype: "SZEX",
                              otype: "LMT",
                              write_access_token: token
                            };
                            if (params.oid) {
                              orderParams.oid = params.oid;
                            }
                            xqBridge.request({
                              url: '/tc/snowx/order/place.json',
                              type: 'POST',
                              data: orderParams,
                              timeout: 10000,
                              success: function (data) {
                                if (data.result_code == 60000) {
                                  var obj = {
                                    sname: order.sname,
                                    symbol: that.symbol,
                                    action: order.action,
                                    price: order.price,
                                    oid: data.result_data[0].oid,
                                    tid: that.broker.tid,
                                    otype: that.type
                                  };
                                  window.location.href = "./order-success.html?" + $.param(obj);
                                }
                              }
                            })
                          }
                        })
                      } else if (["70010", "70000", "79014", "70007"].indexOf(data.result_code) !== -1) {
                        errcb();
                      } else {
                        Util.toast.show(data.msg);
                      }
                    },
                    error: function (e) {
                      if (e.result_code === "70005") {
                        Util.toast.show(e.msg);
                      } else {
                        errcb();
                      }
                    },
                    complete: function () {
                      xqBridge.hideLoading();
                      $('#submit-order').removeClass("disabled");
                    }
                  });
                  function errcb() {
                    xqBridge.confirm({
                      title: "提示",
                      message: "委托提交中\n\n是否成功请在委托页面查看。",
                      alignment: "center",
                      success_button: "查看委托单",
                      success: function () {
                        xqBridge.redirect({
                          url: '/broker/orders',
                          type: "PUSH"
                        })
                      }
                    });
                  }
                },
                error: function () {
                  $('#submit-order').removeClass("disabled");
                  //Util.toast.show('取消下单');
                }
              });
            },
            cancel: function () {
              $('#submit-order').removeClass("disabled");
              xqBridge.trackSNBEvent({
                pageId: 36,
                widgetId: 18,
                widgetPosition: 1,
                foreignKey: that.tid || ''
              });
            }
          });
        } else {
          Util.toast.show("请先完成券商绑定流程");
        }
      } else {
        // 页面初始化
        if (that.subscribe) {
          return false;
        }
        Util.checkAndInit({
          success: function (data) {
            if (!errorHandler(data)) {
              return;
            }
            var brokers = data.result_data;
            if (!that.broker) {
              $(".broker").remove();
              that.brokers = brokers;
              that.changeBroker();
            }
          },
          // 初始化失败退出交易页面
          cancel: function () {
            $("#submit-order").removeClass("disabled");
          },
          isIB: that.etype === "USEX"
        });
      }
    },
    // 关注股票
    followStock: function (symbol, cb, status) {
      function _follow() {
        var url = "/v4/stock/portfolio/addstock.json";
        xqBridge.request({
          url: url,
          type: "POST",
          data: {
            symbol: symbol,
            category: 2,
            isnotice: 1
          },
          complete: function () {
            cb(status);
          }
        });
      }

      xqBridge.getUserInfo({
        success: function (userInfo) {
          var uid = userInfo.uid;
          xqBridge.request({
            url: "/stock/portfolio/hasexist.json",
            data: {
              code: symbol,
              uid: uid
            },
            success: function (ret) {
              if (ret && ret.hasExist) {
                cb(status);
                return;
              }
              _follow();
            },
            error: function (error) {
              cb(status);
            }
          });
        }
      });
    },
    /**
     * 保留小数位
     * @param data
     * @returns {*}
     */
    fixData: function (data) {
      var that = this;
      var needParseKeys = ["current", "bp1", "bp2", "bp3", "bp4", "bp5", "sp1", "sp2", "sp3", "sp4", "sp5", "fall_stop", "rise_stop"];
      var keys = needParseKeys.concat(["bc1", "bc2", "bc3", "bc4", "bc5", "sc1", "sc2", "sc3", "sc4", "sc5"]);
      _.each(keys, function (key) {
        var val = data[key];
        if (!_.isUndefined(val)) {
          if (_.indexOf(needParseKeys, key) > -1) {
            var fixedNum = Util.isFund ? 3 : 2;
            data[key] = parseFloat(val).toFixed(fixedNum);
          }
        } else {
          data[key] = "-";
        }
      });
      if (_.isUndefined(data.percentage)) {
        data.percentStr = "-";
      } else {
        data.percentStr = parseFloat(data.percentage).toFixed(2);
      }
      return data;
    },
    getPanKouInfo: function (symbol, isInit) {
      var oldSymbol = this.symbol;
      this.symbol = symbol;
      this.isFund = Util.isFund(symbol);
      var that = this;

      function errorCb() {
        if (symbol !== oldSymbol || isInit) {
          if (symbol === "799999") {
            $('#scode').val('登记指定 ' + symbol).data('symbol', symbol).data('name', symbol);
            $(".price").val("1");
            $(".amount").attr("placeholder", "股票数量")
              .val("1");
          } else {
            $('#scode').val(symbol + ' ' + symbol).data('symbol', symbol).data('name', symbol);
            $(".price").val("");
            if (that.subscribe) {
              $(".amount").attr("placeholder", "申购数量").val("");
            } else {
              $(".amount").attr("placeholder", "股票数量").val("");
            }
          }

          $('.price-info').html(priceInfoList({
            data: {}
          }));

          $('.wudang-prices').html(wudangList({
            data: that.fixData({})
          })).addClass("no-data");
        }
        if (that.subscribe) {
          that.getStockAmount();
        }
      }

      xqBridge.request({
        url: '/tc/snowx/pankou.json',
        data: {
          symbol: symbol
        },
        success: function (data) {
          // 股票代码存在的时候
          if (data && data.code) {
            data = that.fixData(data);
            that.data = data;
            xqBridge.request({
              url: '/v4/stock/quote.json',
              data: {
                code: symbol
              },
              success: function (quote) {
                var quoteData = quote[symbol];
                that.quote = quoteData;
                if (quoteData.flag === '0' && isInit && !that.subscribe) {
                  xqBridge.request({
                    url: '/preipo/subdate.json',
                    type: "GET",
                    data: {
                      begin: new Date().toISOString().substring(0, 10),
                      end: new Date().toISOString().substring(0, 10),
                      count: 50
                    },
                    success: function (result) {
                      result = result.filter(function (item) {
                        return item.symbol === symbol
                      });
                      if (result.length > 0) {
                        params.scode = result[0].symbol;
                        params.name = result[0].name;
                        params.action = "BUY";
                        params.subscribeCode = result[0].onl_subcode;
                        params.subscribeQty = result[0].onl_submaxqty * 10000;
                        params.price = result[0].iss_price || 0;
                        params.tid = "PAMID";
                        that.init();
                      } else {
                        normal();
                      }
                    },
                    error: function (err) {
                    }
                  })
                } else {
                  normal();
                  pankou();
                }
                function normal() {
                  $('.wudang-prices').html(wudangList({
                    data: data
                  })).removeClass("no-data");
                  $('.price-info').html(priceInfoList({
                    data: data
                  }));
                  //逆回购
                  if (quoteData.type === '17') {
                    that.isPurchase = true;
                    //切换新股票
                    if (symbol !== oldSymbol || isInit) {
                      var day = '(' + parseInt(quoteData.name.substring(quoteData.name.length - 3, quoteData.name.length)) + '天期)';
                      $("#scode").val(data.name + ' ' + data.code + day).data('symbol', data.code).data('name', data.name);
                      $("button.sell").trigger('click');
                      $('.trade-action').hide();
                    }
                    if (quoteData.exchange === "SH") {
                      $(".plus, .minus", $(".amount-group"))
                        .data("unit", 100000)
                        .find(".step")
                        .text('10万');
                    } else if (quoteData.exchange === "SZ") {
                      $(".plus, .minus", $(".amount-group"))
                        .data("unit", 1000)
                        .find(".step")
                        .text(1000);
                    } else {
                      $(".plus, .minus", $(".amount-group"))
                        .data("unit", 100)
                        .find(".step")
                        .text(100);
                    }
                    if (!$('.type-group').hasClass('disable')) {
                      $('.trade-type').find('li').eq(0).trigger('click');
                      $('.type-group').addClass('disable');
                    }
                    if (quoteData.trading_date && quoteData.trading_date.length > 0
                      && quoteData.actual_date && quoteData.actual_date.length > 0
                      && quoteData.preferable_date && quoteData.preferable_date.length > 0) {
                      var tradDate = moment(parseInt(quoteData.trading_date)),
                        actualDate = moment(parseInt(quoteData.actual_date)),
                        preferableDate = moment(parseInt(quoteData.preferable_date)),
                        $out = $('.purchase-out'),
                        $fund = $('.fund-back'),
                        $pick = $('.can-pick');
                      $('.purchase-out .purchase-date').text(tradDate.format('YYYY-MM-DD'));
                      $('.fund-back .purchase-date').text(actualDate.format('YYYY-MM-DD'));
                      $('.can-pick .purchase-date').text(preferableDate.format('YYYY-MM-DD'));
                      $('.purchase-time').show();
                    } else {
                      $('.purchase-time').hide();
                    }
                    $('.rise-stop').hide();
                    $('.fall-stop').hide();
                    $('.all-in').hide();
                    $('.per').hide();
                    $('.purchase').show();
                    $('.purchase-tip').show();
                    $('.trade-stock').css({marginTop: "53px"});
                    $('.stock-current .current-text').text('最新利率(%)');
                    $('span.sell>:first-child').text('借出');
                    $('span.buy>:first-child').text('借入');
                    $('p.price-text').text('借出年利率(%)');
                    $('p.amount-text').text('借出金额');
                    $("#submit-order span").text("借  出");
                  } else {
                    that.isPurchase = false;
                    $('.rise-stop').show();
                    $('.fall-stop').show();
                    $('.per').show();
                    $('.purchase').hide();
                    $('.purchase-tip').hide();
                    $('.trade-stock').css({marginTop: "0"});
                    $('.stock-current .current-text').text('最新');
                    $('span.sell>:first-child').text('卖');
                    $('span.buy>:first-child').text('买');
                    if (!that.subscribe) {
                      $('p.price-text').text('价格');
                      $('p.amount-text').text('数量');
                    }
                    $('.purchase-time').hide();
                    if ($("#submit-order span").text() === "借  出") {
                      $("button.buy").trigger('click');
                    }
                    if (!that.subscribe) {
                      $('.trade-action').show();
                    } else {
                      $('.trade-stock').css({marginTop: "53px"});
                    }
                    if ($('.type-group').hasClass('disable')) {
                      $('.type-group').removeClass('disable');
                    }
                    $('#scode').val(data.name + ' ' + data.code).data('symbol', data.code).data('name', data.name);
                  }
                  that.caculateStyle();
                }
              },
              error: function () {
                pankou();
                errorCb();
              }
            });
            function pankou() {
              // 价格不为空 或者更换了symbol
              // 价格变化的时候 才会去更新amount
              var currentValue = $.trim($(".price").val());
              if ((currentValue.length === 0 && !$(".price").is(":focus")) || symbol !== oldSymbol) {
                if (that.action === "SELL") {
                  if (data.bp1 == 0.0 || data.sp1 === '-') {
                    $('.price').val(data.current + "");
                  } else {
                    $('.price').val(data.bp1 + "");
                  }
                } else {
                  if (data.sp1 == 0.0 || data.sp1 === '-') {
                    $('.price').val(data.current + "");
                  } else {
                    $('.price').val(data.sp1 + "");
                  }
                }
                that.getStockAmount();
                if (params.price) {
                  if (params.price == "市价") {
                    $(".trade-type-text").text("市价委托");
                    $(".trade-type li")
                      .removeClass("selected")
                      .eq(1)
                      .addClass("selected");

                    that.type = "MKT";
                    $(".trade-container").removeClass("LMT MKT").addClass(that.type);
                  } else {
                    $(".price").val(params.price);
                  }
                }
              }
              // 选定五档上面和当前输入框中价格相同的档位
              if (that.type == "LMT") {
                $(".wudang-price").each(function () {
                  var text = $(this).text();
                  if (text == $(".price").val()) {
                    $(this).parents(".prices").addClass("onpress");
                  }
                });
              }
              if (symbol !== oldSymbol || isInit) {
                that.hasCoupon = 'init';
                that.calculateCommission();
              }
            }
          } else {
            errorCb();
          }
        },
        error: function () {
          // 请求的股票不存在
          errorCb();
        }
      });
    },
    getStockAmount: function () {
      var that = this;
      var action = $('.trade-action .select').data('action');
      var enable;
      if (action == 'BUY') {
        enable = '最大可买';
      } else {
        if (that.isPurchase) {
          enable = "最大可借出"
        } else {
          enable = '最大可卖';
        }
      }
      if (that.subscribe) {
        enable = "最大可申购";
      }

      $(".amount").attr("placeholder", enable + "--");
      if (!this.broker && !this.subscribe) {
        return false;
      }

      var aid = this.broker.aid;
      var tid = this.broker.tid;
      var scode = $('#scode').data('symbol');
      var price = $('.price').val() || params.price;
      if (tid === "IB" && !price) {
        price = "0";
      }
      if (aid && action && scode && price) {
        if (that.tid !== "PAMID" && that.isPurchase) {
          $(".amount")
            .attr("placeholder", "借出金额")
            .data("amount", 0);
          return false;
        }
        if (that.subscribe) {
          if (params.scode.search("SH") !== -1) {
            $(".plus, .minus", $(".amount-group"))
              .data("unit", 1000)
              .find(".step")
              .text('1000');
          }
          if (params.scode.search("SZ") !== -1) {
            $(".plus, .minus", $(".amount-group"))
              .data("unit", 500)
              .find(".step")
              .text('500');
          }
          xqBridge.request({
            url: '/tc/snowx/newshare/amount.json',
            type: "GET",
            data: {
              aid: aid,
              tid: tid,
              etype: that.etype
            },
            success: function (data) {
              var maxAmount;
              if (!params.subscribeQty && data.result_data[that.etype]) {
                maxAmount = data.result_data[that.etype].amount;
              } else if (params.subscribeQty && !data.result_data[that.etype]) {
                maxAmount = params.subscribeQty;
              } else if (params.subscribeQty && data.result_data[that.etype]) {
                maxAmount = params.subscribeQty > data.result_data[that.etype].amount ? data.result_data[that.etype].amount : params.subscribeQty;
              } else {
                maxAmount = '--';
              }
              $(".amount")
                .attr("placeholder", enable + maxAmount)
                .data("amount", maxAmount === "--" ? 0 : maxAmount);
            },
            error: function (err) {
              Util.toast.show(err);
            }
          })
        } else {
          xqBridge.request({
            url: '/tc/snowx/order/condition/query.json',
            data: {
              aid: aid,
              tid: tid,
              action: action,
              scode: scode,
              price: price
            },
            success: function (data) {
              if (data.result_code == 60000) {
                var obj = data.result_data[0];
                if (obj) {
                  $('.price').data('etype', obj.etype);
                  if ((obj.enable_amount || obj.enable_amount == 0 ) && !that.subscribe) {
                    var enableAmount = obj.enable_amount;
                    var level = 1;
                    if (that.isPurchase) {
                      level = 100;
                    }
                    enable += obj.enable_amount * level;
                    enableAmount = obj.enable_amount * level;
                    $(".amount")
                      .attr("placeholder", enable)
                      .data("amount", enableAmount);
                  }
                  if (params.amount) {
                    $(".amount").val(params.amount);
                  }

                  var priceStep = obj.price_step == 0 || _.isNull(obj.price_step) ? 10 : obj.price_step;
                  var step = priceStep * 0.001;
                  if (that.broker && that.broker.tid && _.indexOf(specialBrokers, that.broker.tid) > -1) {
                    step = _.isNull(obj.price_step) || _.isUndefined(obj.price_step) ? "0.01" : obj.price_step;
                    step = step * 1;
                  }

                  // step < 0.001 隐藏掉
                  var $priceGroup = $(".price-group");
                  var $plusPrice = $priceGroup.find(".plus");
                  var $minusPrice = $priceGroup.find(".minus");

                  _.each([$plusPrice, $minusPrice], function ($el) {
                    $el
                      .data("step", step)
                      .find(".step")
                      .text(step);
                  });
                  $priceGroup.find(".step").show();
                  if (step < 0.001) {
                    $priceGroup.find(".step").css({fontSize: "8px"}).show();
                  }
                  if (!that.isPurchase) {
                    var buyUnit = obj.buy_unit == 0 || _.isNull(obj.buy_unit) ? 100 : obj.buy_unit;
                    $(".plus, .minus", $(".amount-group"))
                      .data("unit", buyUnit)
                      .find(".step")
                      .text(buyUnit);
                  }

                  if (obj.entrust_prop && obj.entrust_prop.length) {
                    var item = _.first(obj.entrust_prop);
                    that.oprop = item.oprop;
                    $(".mkt-price").text(item.desp);
                  }

                }
              } else {
                enable += '--';
                $(".amount")
                  .attr("placeholder", enable)
                  .data("amount", 0);
              }
            },
            error: function (e) {
              enable += '--';
              $(".amount").attr("placeholder", enable).data('amount', 0);
              $('.price').data('etype', '');
            }
          });
        }
      } else {
        enable += '--';
        $(".amount").attr("placeholder", enable).data('amount', 0);
        $('.price').data('etype', '');
      }
    },
    changeBroker: function (tid) {
      var brokers = this.getBrokers(this.brokers, tid, this.etype);
      var broker = _.find(brokers, function (b) {
        return b.is_default;
      });

      this.broker = broker;

      var html = brokerList({
        data: {
          brokers: brokers
        }
      });

      $('.broker-list').html(html);
      this.tid = tid;
    },
    //计算手续费
    calculateCommission: function () {
      var that = this;
      if (that.subscribe) {
        return false;
      }
      var price = $(".price").val(),
        amount = $(".amount").val(),
        $stampTax = $("#stampTax"),
        $transferFee = $("#transferFee"),
        $totalCommission = $("#totalCommission"),
        $commission = $("#commission"),
        $totalAmount = $("#totalAmount"),
        calculateObj = Util.getCommission(price, amount, that.action, that.symbol, that.broker ? that.broker.tid : ""),
        totalAmount = calculateObj.totalAmount,
        calculateAmount = calculateObj.calculateAmount,
        stampTax = calculateObj.stampTax, //印花税
        transferFee = calculateObj.transferFee, //过户费
        commission = calculateObj.commission, //佣金
        repurchase = calculateObj.repurchase,
        repurchaseRate = calculateObj.repurchaseRate;
      if (!!totalAmount && !!price && !!amount) {
        if (that.isPurchase) {
          $totalAmount.text(amount);
        } else {
          $totalAmount.text(price + " x " + amount + " = " + totalAmount);
        }
      } else {
        $totalAmount.text(0.00);
      }
      //印花税
      $stampTax.html("<div>" + parseFloat(stampTax).toFixed(2) + "</div><div class='amount-tiny'>卖出股票收取0.1%，买入不收</div>");
      //过户费
      $transferFee.html("<div>" + parseFloat(transferFee).toFixed(2) + "</div><div class='amount-tiny'>买卖沪市股票收取，十万分之二</div>");
      //佣金
      if (repurchase !== 0) {
        $commission.html("<div>" + parseFloat(commission).toFixed(2) + "</div><div class='amount-tiny'>按" + repurchaseRate * 100 + "%估算</div>");
      } else {
        $commission.html("<div>" + parseFloat(commission).toFixed(2) + "</div><div class='amount-tiny'>按万2.5估算，每笔最低5元</div>");
      }
      $(".commission-amount").text(calculateAmount);
      $totalCommission.text(calculateAmount);
      that.checkCoupon();
      if (that.isPurchase) {
        that.calculatePurchase(calculateAmount);
      }
      return calculateAmount;
    },
    calculatePurchase: function (commission) {
      var that = this;
      commission = commission || 0;
      var expectEarnings = 0.00,
        price = $(".price").val(),
        amount = $(".amount").val(),
        name = that.quote.name,
        days = 0;
      var purchaseDay = parseInt(name.substring(name.length - 3, name.length));
      if (that.quote.exchange === "SH") {
        days = 360;
      }
      if (that.quote.exchange === "SZ") {
        days = 365;
      }
      expectEarnings = parseFloat(amount * price / 100 * purchaseDay / days - commission).toFixed(2);
      $('.purchase span').text(expectEarnings + "元");
    },
    bindEvent: function () {
      var that = this;
      $(".dialog").css({marginTop: "-" + $(".dialog").height() / 2 + "px"});

      //Util.setPress(".trade-info .buttons button", "onpress");
      //Util.setPress(".amount-select div", "onpress");

      // 定时刷新盘口信息
      if (!that.subscribe) {
        var ts = 3000;
        interval = setInterval(function () {
          if (that.symbol && !$(".price").is(":focus") && !$(".amount").is(":focus") && !that.isSelecting && that.isVisible) {
            that.getPanKouInfo(that.symbol, false, true);
          }
        }, ts);
      }
      $(".pingan-open").on('click', function () {
        xqBridge.redirect({
          url: 'http://xueqiu.com/broker/open/pingan',
          type: "PUSH"
        })
      });
      $(".pingan-bind").on('click', function () {
        xqBridge.redirect({
          url: '/broker/bind-list',
          type: "PUSH"
        })
      });
      window.viewDidAppear = function () {
        that.isVisible = true;

      };

      // app切入后台之后 不再刷新盘口
      window.viewDidDisappear = function () {
        that.isVisible = false;
      };

      $('.amount,.price').on('input', function (e) {
        var $target = $(this),
          val = $target.val();
        $target.val(val.replace('-', ''));
      });

      // 点击overlay的时候 关掉所有下拉
      $(".overlay").on("click", function () {
        $(this).hide();

        // 关掉委托类型
        $(".trade-type").hide();
        $(".type-group").removeClass("active");

        // 关掉券商选择
        $(".select-list").hide();
        $(".broker-list").removeClass("expand");
      });

      // 保险
      $("body").on("click", ".insurance", function () {
        xqBridge.redirect({
          url: "http://xueqiu.com/broker/insurance/pingan",
          type: "MODAL"
        });
      });

      // 弹出委托选项
      $("body").on("click", ".type-group", function () {
        if ($(this).hasClass('disable')) {
          return false;
        }
        if (that.isEdit) {
          Util.toast.show("改单状态禁止切换委托类型");
          return false;
        }

        var $tradeType = $(".trade-type");
        var $overlay = $(".overlay");

        if ($tradeType.is(":visible")) {
          $(this).removeClass("active");
        } else {
          $(this).addClass("active");
        }

        $overlay.toggle();
        $tradeType.toggle();
      });

      // 选择委托类型
      $(".trade-type").on("click", "li", function () {
        var $li = $(this);
        var $overlay = $(".overlay");
        xqBridge.trackSNBEvent({
          pageId: 36,
          widgetId: 6,
          widgetPosition: $(this).index() + 1,
          foreignKey: that.tid || ''
        });
        if ($li.hasClass("selected")) {
          $li.parent().hide();
          $(".type-group").removeClass("active");
          $overlay.hide();
          return false;
        }

        $li.addClass("selected");
        $li.siblings().removeClass("selected");
        var text = $li.find("span").text();
        $(".trade-type-text").text(text);

        that.type = $li.data("type");
        $(".trade-container").removeClass("LMT MKT").addClass(that.type);

        setTimeout(function () {
          $li.parent().hide();
          $(".type-group").removeClass("active");
          $overlay.hide();
        }, 200);

        return false;
      });

      //选择佣金券
      $(".coupon").on('click', function () {
        var type = '';
        if (that.isPurchase) {
          type = "purchase";
        }
        xqBridge.redirect({
          url: "/broker/coupon-list?coupon=" + that.coupon.rewardOwnerId + "&type=" + type,
          type: "MODAL",
          success: function (data) {
            if (data) {
              that.couponList.map(function (item) {
                if (data === 'none') {
                  $(".coupon").css({display: 'block'});
                  $("#coupon-text").text("未使用").css({color: "#ccc"});
                  that.coupon = {};
                  $(".coupon i").hide();
                  that.hasCoupon = 'keep';
                }
                if (item.rewardOwnerId === data) {
                  that.coupon = item;
                  $(".coupon").css({display: 'block'});
                  $("#coupon-text")
                    .css({color: "#FF7700"});
                  $("#coupon-text").text(item.name + "(￥" + item.faceValue + ")");
                  that.hasCoupon = 'keep';
                }
              });
            }
          }
        })
      });

      $(".no-broker .open").on('click', function (e) {
        e.preventDefault();
        var url = 'http://xueqiu.com/broker/open/pingan?from=xiadan';
        if (that.etype === "USEX") {
          url = 'http://xueqiu.com/broker/open/ib';
        }
        xqBridge.redirect({
          type: "PUSH",
          url: url
        })
      });
      $(".no-broker .add").on('click', function (e) {
        e.preventDefault();
        var url = "/broker/bind-list";
        if (that.etype === "USEX") {
          url += '?ib=true';
        }
        xqBridge.redirect({
          type: "MODAL",
          url: url
        })
      });
      var isIPhone = navigator.userAgent.indexOf("iPhone") > -1;
      if (!isIPhone) {
        $(".no-broker a").css({padding: '5px'});
      }

      // 切换券商
      if (!that.subscribe) {
        $("body").on("click", ".broker-list .broker", function () {
          if (that.isEdit) {
            Util.toast.show("改单状态禁止切换券商");
            return false;
          }

          var $parent = $(this).parent();
          var $broker = $(this);
          var $container = $(".broker-list");
          var $overlay = $(".overlay");

          if ($parent.hasClass("select-list")) {
            // 切换
            if ($parent.hasClass("selected")) {
              return false;
            }

            $parent.addClass("selected");
            $parent.siblings().removeClass("selected");

            var aid = $broker.data("aid");

            xqBridge.showLoading();
            xqBridge.request({
              url: "/tc/snowx/securities/default.json",
              type: "POST",
              data: {
                changetoaid: aid
              },
              success: function (ret) {
                if (ret.result_code == 60000) {
                  var tid = $broker.data("tid");
                  that.changeBroker(tid);
                  console.log($parent);
                  xqBridge.trackSNBEvent({
                    pageId: 36,
                    widgetId: 2,
                    widgetPosition: $('.selected').index(),
                    foreignKey: tid || ''
                  });
                  $(".select-list").hide();
                  $overlay.hide();
                  $container.removeClass("expand");
                  that.updateBodyClass();
                  that.getStockAmount();
                  that.calculateCommission();
                }
              },
              error: function () {
                Util.toast.show("切换账号出错");
              },
              complete: function () {
                xqBridge.hideLoading();
              }
            });
          } else {
            var $arrow = $(this).find(".arrow");
            if (!$arrow.length) {
              return false;
            }
            // 显示
            var $brokerList = $(".select-list");
            if ($brokerList.is(":hidden")) {
              $brokerList.show();
              $overlay.show();
              $container.addClass("expand");
            } else {
              $brokerList.hide();
              $overlay.hide();
              $container.removeClass("expand");
            }
            that.calculateCommission();
          }
        });
      } else {
        $(".broker-list .arrow").hide();
      }

      // 股票搜索
      if (!that.subscribe) {
        $('.control').on('click', 'p', function (e) {
          $(this).find('input').focus();
        });
        $('#scode').on('focus', function (e) {
          e.preventDefault();
          var scode = this;
          $(scode).blur();
          if (!$(scode).hasClass('search-opened')) {
            xqBridge.trackEvent("trade_selectStock");
            xqBridge.trackSNBEvent({
              pageId: 36,
              widgetId: 5,
              widgetPosition: 1,
              foreignKey: that.tid || ''
            });
            $(scode).addClass('search-opened');
            var etype = that.broker ? that.broker.etype : "";
            xqBridge.searchTCStock({
              etype: etype,
              success: function (data) {
                that.isUSHK = etype.search("USEX") !== -1 || etype.search("HKEX") !== -1;
                //if (that.broker && that.broker.etype && that.broker.etype.indexOf(that.etype) == -1) {
                //  Util.toast.show("当前券商不支持购买此股票");
                //}
                $(scode).removeClass('search-opened');
                $(scode).val(data).data('symbol', data).data('name', data);
                that.getPanKouInfo(data, true);
                that.caculateStyle();
                that.updateBodyClass();
                $(scode).removeClass('search-opened');
              },
              cancel: function () {
                $(scode).removeClass('search-opened');
              }
            });
          }
        });
      }

      // 加减价格
      $('.price-group button').on('click', function () {
        var $price = $(".price");
        var price = parseFloat($price.val());

        if (_.isNaN(price)) {
          return false;
        }

        var step = parseFloat($(this).data("step"));
        var fixedNumber = (step + "").replace("0.", "").length;

        // 买卖切换
        $('.trade-action button').on('click', function () {
          if (that.isEdit) {
            Util.toast.show("改单状态禁止切换买卖类型");
            return false;
          }
        });

        if ($(this).hasClass("minus")) {
          price -= step;
          xqBridge.trackEvent("trade_priceDown");
          xqBridge.trackSNBEvent({
            pageId: 36,
            widgetId: 9,
            widgetPosition: 1,
            foreignKey: that.tid || ''
          });
        } else {
          xqBridge.trackEvent("trade_priceUp");
          xqBridge.trackSNBEvent({
            pageId: 36,
            widgetId: 8,
            widgetPosition: 1,
            foreignKey: that.tid || ''
          });
          price += step;
        }

        if (price > 0) {
          var priceStr = price.toFixed(fixedNumber);
          $price.val(priceStr);
          that.getStockAmount();
        }
        that.calculateCommission();
        return false;
      });

      // 加减数量
      $(".amount-group button").on('click', function () {
        var $amount = $(".amount");
        var amount = parseInt($amount.val() || "0");
        var $button = $(this);

        if (_.isNaN(amount)) {
          return false;
        }

        var resultAmount = amount;
        var unit = parseInt($button.data("unit"), 10);
        if ($button.hasClass("minus")) {
          xqBridge.trackEvent("trade_numberDown");
          xqBridge.trackSNBEvent({
            pageId: 36,
            widgetId: 10,
            widgetPosition: 1,
            foreignKey: that.tid || ''
          });
          resultAmount -= unit;
        } else {
          xqBridge.trackEvent("trade_numberUp");
          xqBridge.trackSNBEvent({
            pageId: 36,
            widgetId: 11,
            widgetPosition: 1,
            foreignKey: that.tid || ''
          });
          resultAmount += unit;
        }

        if (resultAmount >= 0) {
          $amount.val(resultAmount);
        }
        that.calculateCommission();
        return false;
      });

      $('body').on('click', '.control', function (e) {
        $(this).find('input').focus();
      });

      // 买卖切换
      $('.trade-action button').on('click', function () {
        if (that.isEdit) {
          Util.toast.show("改单状态禁止切换买卖类型");
          return false;
        }

        var symbol = $('#scode').data('symbol');
        $('.trade-action .select').removeClass('select');
        $(this).addClass('select');
        that.getStockAmount();
        that.action = $(this).data("action");
        that.updateBodyClass();
        var actionText = that.action === "BUY" ? "买  入" : "卖  出";
        xqBridge.trackSNBEvent({
          pageId: 36,
          widgetId: that.action === "BUY" ? 4 : 3,
          widgetPosition: 1,
          foreignKey: that.tid || ''
        });
        $("#submit-order span").html(actionText);
        if (that.action === "SELL") {
          $("body").addClass("SELL");
          $(".all-in").show();
        } else {
          $("body").removeClass("SELL");
          $(".all-in").hide();
        }
        that.calculateCommission();
        return false;
      });

      //价格变动
      $('.price').on('blur', function (e) {
        that.getStockAmount();
        that.calculateCommission();
      });

      // 5档切换价格
      $('body').on('click', '.prices', function () {
        $(this).addClass("onpress");
        $(this).siblings().removeClass("onpress");
        $('.price').val($(this).find('.wudang-price').text());
        that.getStockAmount();
        that.calculateCommission();
      });

      // 计算手续费
      $("input[type='number']").on('blur', function (e) {
        that.calculateCommission();
      });
      $(".amount").on('click', function (e) {
        xqBridge.trackSNBEvent({
          pageId: 36,
          widgetId: 10,
          widgetPosition: 1,
          foreignKey: that.tid || ''
        });
      });
      $(".price").on('click', function (e) {
        xqBridge.trackSNBEvent({
          pageId: 36,
          widgetId: 7,
          widgetPosition: 1,
          foreignKey: that.tid || ''
        });
      });
      //手续费参考
      $(".question-mark").on('click', function () {
        xqBridge.trackEvent("trade_fee");
        xqBridge.trackSNBEvent({
          pageId: 36,
          widgetId: 14,
          widgetPosition: 1,
          foreignKey: that.tid || ''
        });
        Util.overlay.showOverlay($(".dialog"), true);
      });
      $("#close-dialog").on('click', function () {
        Util.overlay.hideOverlay($(".dialog"));
      });

      // 未绑定券商点击绑定券商
      //$("body").on("click", ".no-broker", function () {
      //  that.checkBind();
      //});

      // 一键清仓
      $(".all-in").on("click", function () {
        xqBridge.trackEvent("trade_sellAll");

        if (that.broker) {
          if (that.broker.status === "SUCCESS") {
            xqBridge.redirect({
              url: "/broker/order?tid=" + that.broker.tid,
              type: "PUSH"
            });
          } else {
            Util.toast.show("请先完成绑定流程！");
          }
        } else {
          Util.toast.show("请先绑定券商！");
        }

        return false;
      });

      // 点击最新 涨停 跌停  自动填充价格到价格输入框
      $("body").on("click", ".stock-current", function () {
        if (that.type === "LMT" && that.data && that.data.current) {
          $(".price").val(that.data.current);
          $(".prices").removeClass("onpress");
          that.calculateCommission();
        }
      });
      $("body").on("click", ".rise-stop", function () {
        if (that.type === "LMT" && that.data && that.data.rise_stop) {
          $(".price").val(that.data.rise_stop);
          $(".prices").removeClass("onpress");
          that.calculateCommission();
        }
      });

      $("body").on("click", ".fall-stop", function () {
        if (that.type === "LMT" && that.data && that.data.fall_stop) {
          $(".price").val(that.data.fall_stop);
          $(".prices").removeClass("onpress");
          that.calculateCommission();
        }
      });

      // 仓位选择
      $(".amount-select div").on("click", function () {
        console.log("change amount");
        var amount = $(".amount").data("amount");
        var $el = $(this);
        var resultAmount = 0;
        if (amount) {
          if ($el.hasClass("three")) {
            xqBridge.trackEvent("trade_third");
            xqBridge.trackSNBEvent({
              pageId: 36,
              widgetId: 13,
              widgetPosition: 1,
              foreignKey: that.tid || ''
            });
            resultAmount = Math.floor(amount / 300) * 100;
          }
          if ($el.hasClass("two")) {
            xqBridge.trackEvent("trade_half");
            xqBridge.trackSNBEvent({
              pageId: 36,
              widgetId: 13,
              widgetPosition: 2,
              foreignKey: that.tid || ''
            });
            resultAmount = Math.floor(amount / 200) * 100;

          }
          if ($el.hasClass("all")) {
            xqBridge.trackEvent("trade_whole");
            xqBridge.trackSNBEvent({
              pageId: 36,
              widgetId: 13,
              widgetPosition: 3,
              foreignKey: that.tid || ''
            });
            resultAmount = amount;
          }

          if (resultAmount < 100) {
            resultAmount = 100;
          }
        }
        if (resultAmount > 0) {
          $(".amount").val(resultAmount);
        }
        that.calculateCommission();
      });

      //提交订单
      $('#submit-order').on('click', function (e) {
        e.preventDefault();
        xqBridge.trackEvent("trade_buyAndSell");
        var $button = $(this);
        if ($button.hasClass("disabled")) {
          return false;
        }
        $button.addClass('disabled');
        // 防止出错之后  没有删除掉disabled
        setTimeout(function () {
          $button.removeClass('disabled');
        }, 2000);
        var orderParams = {};
        orderParams.action = $('.trade-action .select').data('action');
        orderParams.title = orderParams.action == 'BUY' ? '确认买入' : '确认卖出';
        orderParams.scode = $('#scode').data('symbol');
        orderParams.sname = $('#scode').data('name');
        orderParams.price = $('.price').val() || params.price;
        orderParams.etype = that.etype;
        orderParams.amount = $('.amount').val();
        orderParams.total = orderParams.price * orderParams.amount;
        orderParams.totalText = orderParams.total.toFixed(2);
        orderParams.coupon = that.coupon;
        if (that.isPurchase) {
          if (!orderParams.price) {
            Util.toast.show("请输入借出年利率");
            return
          }
          if (!orderParams.amount) {
            Util.toast.show("请输入借出金额");
            return
          }
        }
        xqBridge.trackSNBEvent({
          pageId: 36,
          widgetId: 15,
          widgetPosition: 1,
          foreignKey: that.tid || ''
        });
        if (that.isUSHK) {
          var priceStep = $(".plus", $(".price-group")).data("step");
          if (priceStep && priceStep != "0") {
            var times = 1 / priceStep;
            if (parseInt(orderParams.price * times, 10) % 1) {
              Util.toast.show("买卖价格需为报价最小单位整数倍");
              return false;
            }
          }
          if (orderParams.amount % 1 && orderParams.action === "BUY") {
            Util.toast.show("买入数量需为整数");
            return false;
          }
          var buyStep = $(".plus", $(".amount-group")).data("unit");
          if (buyStep && orderParams.etype === "HKEX" && orderParams.action === "BUY" && orderParams.amount % buyStep) {
            Util.toast.show("数量需为每手股数的整数倍");
            return false;
          }
        }
        if (orderParams.action === "BUY") {
          xqBridge.trackEvent("trade_buy");
        } else {
          xqBridge.trackEvent("trade_sell");
        }
        if (that.isPurchase) {
          var day = parseInt(that.quote.name.substring(that.quote.name.length - 3, that.quote.name.length)) + '天期';
          orderParams.message = '借出周期：' + (day ? day + ' ' : '') + '\n' +
            '借出年利率：' + (that.type === "LMT" ? orderParams.price : "市价") + '%\n' +
            '借出金额：' + orderParams.amount + '元\n预期收益：' + $('.purchase span').text().replace('元', '') + '（参考）\n';
          orderParams.amount = orderParams.amount / 100;
        } else {
          orderParams.message = '股票：' + (orderParams.sname ? orderParams.sname + ' ' : '') + orderParams.scode + '\n' +
            '价格：' + (that.type === "LMT" ? orderParams.price : "市价") + '\n' +
            '数量：' + orderParams.amount + '\n';
        }
        if (that.type === "LMT" && !that.isPurchase) {
          orderParams.message += '金额：' + orderParams.totalText + '\n';
        }
        if (that.broker && !that.subscribe && that.symbol !== "799999") {
          if (_.indexOf(specialBrokers, that.broker.tid) == -1) {
            orderParams.message += '手续费：' + that.calculateCommission() + '（参考）\n';
          }
        }
        if ($(".coupon").is(':visible') && !_.isEmpty(that.coupon)) {
          orderParams.message += that.coupon.name + "：-" + that.coupon.faceValue + '元\n';
        }
        if (that.type === "MKT") {
          var isAbove = parseFloat(orderParams.price) >= parseFloat($('.rise-stop .stock_up').text());
          var isBelow = parseFloat(orderParams.price) <= parseFloat($('.fall-stop .stock_down').text());
          if ((isAbove || isBelow) && that.broker.tid !== "IB") {
            xqBridge.confirm({
              title: "提示",
              message: "涨跌停时市价委托会出现废单，是否继续？",
              alignment: "center",
              success_button: "继续",
              success: function () {
                that.checkBind(orderParams);
              },
              cancel: function () {
                return false;
              }
            })
          }
          if (!orderParams.price || orderParams.price === "") {
            $('.price').val(0);
            orderParams.price = "0.00";
          }
        }
        if (!that.validate.validateAll()) {
          $button.removeClass("disabled");
          return;
        }
        //下单
        that.checkBind(orderParams);
      });
    }
  };
});
