/**
 * Created by lyj on 6/23/15.
 */
define(function (require, exports, module) {
  // 统计
  xqBridge.trackEvent("transfer-history");
  function errorHandler(e) {
    xqBridge.hideLoading();
    if (e && e.result_code == 60000) {
      return true;
    }
    else if (e && (e.result_code == 70005 || e.result_code == 70006)) {
      xqBridge.redirect({
        type: 'POP_CANCEL'
      });
    } else if (e && (e.error_description || e.msg)) {
      xqBridge.alert({
        message: e.msg || e.error_description,
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    } else {
      xqBridge.alert({
        message: '网络错误',
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    }
  }

  module.exports = {
    init: function () {
      var that = this;

      Util.checkAndInit({
        success: function (data) {
          if (!errorHandler(data)) {
            return;
          }
          var brokerList = data.result_data;
          var broker = Util.getBroker(brokerList, Util.parseParams().tid);

          if (broker) {
            that.broker = broker;
            that.aid = broker.aid;
            that.tid = broker.tid;

            xqBridge.getUserInfo({
              success: function (userInfo) {
                that.uid = userInfo.uid;
                that.getBank(function () {
                  that.getTransferList();
                });
              }
            });
          }
        },

        // 初始化失败退出交易页面
        cancel: function () {
          xqBridge.redirect({
            type: "POP_CANCEL"
          });
        }
      });
    },
    getTransferList: function (page) {
      var that = this;
      page = page || 1;

      var now = Date.now();
      var start = now - 30 * 24 * 60 * 60 * 1000;
      var startStr = moment(start).format("YYYYMMDD");
      var endStr = moment(now).format("YYYYMMDD");

      var params = {
        aid: that.aid,
        tid: that.tid,
        uid: that.uid,
        cur: "CNY",
        bank_no: that.bank.bank_no,
        start_date: startStr,
        end_date: endStr,
        pos: page,
        row: 20
      };
      xqBridge.request({
        url: "/tc/snowx/bank/transfer/query.json",
        data: params,
        type: "GET",
        success: function (ret) {
          if (ret.result_code == 60000) {
            //ret.resultdata =  [
            //  {
            //    "branch_no":"20",
            //    "fund_account":12345,
            //    "bank_no":"6",
            //    "bank_name":"招行存管",
            //    "oid":"55117",
            //    "trans_name":"存管签约开户",
            //    "money_type":"人民币",
            //    "occur_balance":0,
            //    "entrust_time":135715,
            //    "entrust_name":"成功",
            //    "entrust_status":"2",
            //    "pos":"201503090000055117",
            //    "ctime": "2016030800000000"
            //  }
            //];

            if (ret.result_data && ret.result_data.length) {
              var list = that.fixData(ret.result_data);
              that.renderHtml(list);
            } else {
              var noDataHtml = '<tr><td class="noData" colspan="4"><div class="img"></div><p>暂无转账记录</p></td></tr>'
              $('.transfer-history tbody').html(noDataHtml);
            }

          } else {
            Util.showError(ret.msg);
          }
        },
        error: function (error) {
          console.log(error);
        },
        complete: function () {
          xqBridge.hideLoading();
        }
      });
    },
    getBank: function (cb) {
      var that = this;

      xqBridge.request({
        url: "/tc/snowx/bank/query.json",
        data: {
          aid: that.aid,
          uid: that.uid,
          tid: that.tid
        },
        type: "GET",
        success: function (ret) {
          var bank = _.first(ret.result_data);

          if (bank) {
            that.bank = bank;
            if (cb) {
              cb();
            }
          } else {
            xqBridge.hideLoading();
            xqBridge.alert({
              message: ret.msg || "此券商暂不支持银证转账",
              success: function () {
                xqBridge.redirect({
                  type: 'POP_CANCEL'
                });
              }
            });
          }
        },
        error: function () {
          xqBridge.hideLoading();
        }
      });
    },

    fixData: function (list) {
      var array = [];
      var moneySymbols = {
        "CNY": "￥"
      };
      _.each(list, function (item) {
        var obj = {};
        obj.type = item.trans_name;
        //obj.amount = moneySymbols[item.money_type] + item.occur_balance;
        obj.amount = parseFloat(item.occur_balance).toFixed(2);
        obj.state = item.entrust_name;

        item.ctime += "";
        if (item.ctime && item.ctime.length >= 8) {
          var timeStr = item.ctime.slice(0, 8);
          obj.time = timeStr.slice(0, 4) + "-" + timeStr.slice(4, 6) + "-" + timeStr.slice(6, 8);
        } else {
          obj.time = "-";
        }
        array.push(obj);
      });

      return array;
    },

    renderHtml: function (list) {
      var html = '';
      _.each(list, function (item) {
        var tr = '<tr>';
        tr += '<td class="op">' + item.type + '</td>';
        tr += '<td class="amount">' + item.amount + '</td>';
        tr += '<td class="time">' + item.time + '</td>';
        tr += '</tr>';
        tr += '<tr>';
        tr += '<td colspan="3">';
        tr += '<div>'+item.state+'</div>';
        tr += '</td>';
        tr += '</tr>';
        html += tr;
      });

      $(".transfer-history tbody").html(html);
    }
  };
});