/**
 * Created by lyj on 6/23/15.
 */
define(function (require, exports, module) {
  // 统计
  xqBridge.trackEvent("transfer");

  function errorHandler(e) {
    xqBridge.hideLoading();
    if (e && e.result_code == 60000) {
      return true;
    }
    else if (e && (e.result_code == 70005 || e.result_code == 70006)) {
      xqBridge.redirect({
        type: 'POP_CANCEL'
      });
    } else if (e && (e.error_description || e.msg)) {
      xqBridge.alert({
        message: e.msg || e.error_description,
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    } else {
      xqBridge.alert({
        message: '网络错误',
        success: function () {
          xqBridge.redirect({
            type: 'POP_CANCEL'
          });
        }
      });
    }
  }

  module.exports = {
    init: function () {
      var that = this;
      this.bankDefer = $.Deferred();
      this.assetsDefer = $.Deferred();
      $.when(this.bankDefer, this.assetsDefer).then(function () {
        xqBridge.hideLoading();
        $(".container").show();
        that.bindEvent();
      });
      xqBridge.setRightNavigationButton({
        title: "转账记录",
        action: function () {
          xqBridge.trackSNBEvent({
            pageId: 39,
            widgetId: 6,
            widgetPosition: 1,
            foreignKey: that.broker.tid || ''
          });
          xqBridge.redirect({
            url: '/broker/transfer-history?tid=' + that.broker.tid,
            type: "PUSH",
            success: function () {
            },
            cancel: function () {
            }
          })
        }
      });

      Util.checkAndInit({
        success: function (data) {
          if (!errorHandler(data)) {
            return;
          }
          var brokerList = data.result_data;
          var broker = Util.getBroker(brokerList, Util.parseParams().tid);

          if (broker) {
            that.broker = broker;
            that.aid = broker.aid;
            that.tid = broker.tid;
            xqBridge.trackSNBEvent({
              pageId: 39,
              widgetId: 1,
              widgetPosition: 1,
              foreignKey: that.broker.tid || ''
            });
            xqBridge.getUserInfo({
              success: function (userInfo) {
                that.uid = userInfo.uid;
                that.getBank();
                that.getAccountInfo();
                // that.bindEvent();
              }
            });
          }
        },

        // 初始化失败退出交易页面
        cancel: function () {
          xqBridge.redirect({
            type: "POP_CANCEL"
          });
        }
      });
    },
    getBank: function () {
      var that = this;

      xqBridge.request({
        url: "/tc/snowx/bank/query.json",
        data: {
          aid: that.aid,
          tid: that.tid,
          uid: that.uid
        },
        type: "GET",
        success: function (ret) {
          var bank = _.first(ret.result_data);

          if (bank) {
            that.bank = bank;
            $(".bank-name")
              .text(bank.bank_name)
              .data("bno", bank.bank_no);
            var direction = $(".transfer-type .select").data("type");
            if (direction == 1) {
              if (bank.intoNeed === 'bank') {
                $(".bank-password").show();
              } else if (bank.intoNeed === "fund") {
                $(".fund-password").show();
              } else if (bank.intoNeed === "both") {
                $(".bank-password").show();
                $(".fund-password").show();
              } else if (!!bank.needBankPW) {
                $(".bank-password").show();
              }
            }
            if (direction == 2) {
              if (bank.outNeed === 'bank') {
                $(".bank-password").show();
              } else if (bank.outNeed === "fund") {
                $(".fund-password").show();
              } else if (bank.outNeed === "both") {
                $(".bank-password").show();
                $(".fund-password").show();
              } else if (!!bank.needFundPW) {
                $(".fund-password").show();
              }
            }
          } else {
            xqBridge.alert({
              message: ret.msg || "尚未绑定银行卡，请联系券商",
              success: function () {
                xqBridge.redirect({
                  type: 'POP_CANCEL'
                });
              }
            });
          }
        },
        complete: function () {
          that.bankDefer.resolve();
        }
      });
    },
    getAccountInfo: function () {
      var that = this;
      xqBridge.request({
        url: "/tc/snowx/fund/query.json",
        type: "GET",
        data: {
          aid: that.aid,
          tid: that.tid
        },
        success: function (ret) {
          var accountInfo = _.first(ret.result_data);
          var amount = "--";
          if (accountInfo) {
            amount = accountInfo.fetch_balance;
          }
          $(".enable-amount").text(amount);
          $(".enable").click(function () {
            $(".amount").val(amount);
          });
        },
        complete: function () {
          that.assetsDefer.resolve();
        }
      });
    },
    bindEvent: function () {
      var that = this;
      var transferOut = 0;
      var transferIn = 0;
      $("li").on('click', function () {
        $(this).find("input").focus();
      });
      $(".transfer-type").on("click", "div", function () {
        if ($(this).hasClass("select")) {
          return false;
        } else {
          $(this).siblings().removeClass("select");
          $(this).addClass("select");

          var direction = $(".transfer-type .select").data("type");
          var bank = that.bank;

          $(".bank-password, .fund-password").hide();

          if (direction == 1) {
            if (bank.intoNeed === 'bank') {
              $(".bank-password").show();
            } else if (bank.intoNeed === "fund") {
              $(".fund-password").show();
            } else if (bank.intoNeed === "both") {
              $(".bank-password").show();
              $(".fund-password").show();
            } else if (!!bank.needBankPW) {
              $(".bank-password").show();
            }
            xqBridge.trackSNBEvent({
              pageId: 39,
              widgetId: 2,
              widgetPosition: 1,
              foreignKey: that.broker.tid || ''
            });
            xqBridge.trackEvent("bank_bankToSec");
            $(".submit button").text("转入到证券");
            $("body").removeClass("out");
          }
          if (direction == 2) {
            if (bank.outNeed === 'bank') {
              $(".bank-password").show();
            } else if (bank.outNeed === "fund") {
              $(".fund-password").show();
            } else if (bank.outNeed === "both") {
              $(".bank-password").show();
              $(".fund-password").show();
            } else if (!!bank.needFundPW) {
              $(".fund-password").show();
            }
            xqBridge.trackSNBEvent({
              pageId: 39,
              widgetId: 3,
              widgetPosition: 1,
              foreignKey: that.broker.tid || ''
            });
            xqBridge.trackEvent("bank_secToBank");
            $(".submit button").text("转出到银行");
            $("body").addClass("out");
          }
          if ($(this).data("type") === 1) {
            transferOut = $(".amount").val();
            $(".amount").val(transferIn);
          } else {
            transferIn = $(".amount").val();
            $(".amount").val(transferOut);
          }
        }
      });

      $(".submit").on("click", function () {
        var $button = $(this);
        var uid = that.uid;
        var aid = that.aid;
        var tid = that.tid;
        var transfer_direction = $(".transfer-type").find(".select").data("type");
        var bankNo = $(".bank-name").data("bno");
        var balance = $.trim($(".amount").val());
        var fundPassword = $.trim($(".fpassword").val());
        var bankPassword = $.trim($(".bpassword").val());

        var param = {
          uid: uid,
          aid: aid,
          tid: tid,
          transfer_direction: transfer_direction,
          cur: "CNY"
        };

        if (!bankNo) {
          Util.showError("获取不到银行信息");
          return false;
        }

        if (balance === "") {
          Util.showError("请输入正确的转账金额");
          return false;
        }
        var hasbank = $(".bank-password").is(":visible");
        var hasfund = $(".fund-password").is(":visible");
        if (hasbank) {
          var bankDefer = $.Deferred();
          if (!bankPassword) {
            Util.showError("请输入银行密码");
            return false;
          } else {
            if (Util.appVersion() >= 7.3) {
              xqBridge.encryptWithRSA({
                origin: bankPassword,
                success: function (data) {
                  param.bank_password_encrypted = data;
                  bankDefer.resolve();
                },
                error: function () {
                  param.bank_password = bankPassword;
                  bankDefer.resolve();
                }
              });
            } else {
              param.bank_password = bankPassword;
              bankDefer.resolve();
            }
          }
        }

        if (hasfund) {
          var fundDefer = $.Deferred();
          if (!fundPassword) {
            Util.showError("请输入资金密码");
            return false;
          } else {
            if (Util.appVersion() >= 7.3) {
              xqBridge.encryptWithRSA({
                origin: fundPassword,
                success: function (data) {
                  param.fund_password_encrypted = data;
                  fundDefer.resolve();
                },
                error: function () {
                  param.fund_password = fundPassword;
                  fundDefer.resolve();
                }
              });
            } else {
              param.fund_password = fundPassword;
              fundDefer.resolve();
            }
          }
        }

        _.extend(param, {
          bank_no: bankNo,
          balance: balance
        });
        $.when(bankDefer, fundDefer).then(function () {
          Util.getAndRefreshToken({
            aid: aid,
            renew_url: that.broker.renew_url,
            broker: that.broker,
            success: function (token) {
              param.write_access_token = token;
              if ($button.hasClass("disabled")) {
                return false;
              }
              $button.addClass("disabled");
              xqBridge.showLoading();
              xqBridge.request({
                url: "/tc/snowx/bank/transfer.json",
                type: "POST",
                data: param,
                success: function (ret) {
                  if (ret.result_code == 60000) {
                    var first = _.first(ret.result_data);
                    if (first && first.oid) {
                      Util.showError("委托成功，编号为：" + first.oid, function () {
                        xqBridge.redirect({
                          url: "/broker/transfer-history?tid=" + that.broker.tid,
                          type: "PUSH"
                        });
                      });
                    } else {
                      Util.showError(ret.msg);
                      xqBridge.redirect({
                        url: "/broker/transfer-history?tid=" + that.broker.tid,
                        type: "PUSH"
                      });
                    }
                    $('input').val("");
                    that.getAccountInfo();
                  } else if (["70010", "70000", "79014", "70007"].indexOf(ret.result_code) !== -1) {
                    errcb();
                  } else {
                    Util.showError(ret.msg);
                  }
                },
                error: function (e) {
                  if (e.result_code === "70005") {
                    Util.toast.show(e.msg);
                  } else {
                    errcb();
                  }
                },
                complete: function () {
                  $button.removeClass("disabled");
                  xqBridge.hideLoading();
                }
              });
              function errcb() {
                xqBridge.confirm({
                  title: "提示",
                  message: "银转委托提交中\n\n是否成功请在银转记录页面查看。",
                  alignment: "center",
                  success_button: "查看银转记录",
                  success: function () {
                    xqBridge.redirect({
                      url: "/broker/transfer-history?tid=" + that.broker.tid,
                      type: "PUSH"
                    });
                  }
                });
              }
            },
            cancel: function () {
              xqBridge.hideLoading();
              $button.removeClass("disabled");
            },
            error: function () {
              xqBridge.hideLoading();
              $button.removeClass("disabled");
            }
          });
        });
      });
    }
  }
});
