(function(window){

  var dayPeriods = ["1d", "5d", "1m", "6m", "3y", "all"];
  var klinePeriods = ["1day", "1week", "1month"];
  var fundPeriods = ["unit", "unit5", "seven", "million"]; // 单位净值 七日年化  万份收益

  // 不显示复权
  var noFuQuanSymbols = ["SH000001", "SZ399001", "SZ399006"];

  // 不显示五档
  var noPankouSymbols = [
    "SH000001",
    "SH000002",
    "SH000003",
    "SH000008",
    "SH000009",
    "SH000010",
    "SH000011",
    "SH000012",
    "SH000016",
    "SH000017",
    "SH000132",
    "SH000300",
    "SZ399001",
    "SZ399002",
    "SZ399003",
    "SZ399004",
    "SZ399005",
    "SZ399006",
    "SZ399100",
    "SZ399101",
    "SZ399106",
    "SZ399107",
    "SZ399108",
    "SZ399333",
    "SZ399606"
  ];

  // 分辨率
  var PIXEL_RATIO = (function () {
    var ctx = document.createElement("canvas").getContext("2d"),
      dpr = window.devicePixelRatio || 1,
      bsr = ctx.webkitBackingStorePixelRatio ||
        ctx.mozBackingStorePixelRatio ||
        ctx.msBackingStorePixelRatio ||
        ctx.oBackingStorePixelRatio ||
        ctx.backingStorePixelRatio || 1;

    return dpr / bsr;
  })();

  function decimal_2(decimal) {
    if(!decimal){//TODO
      return "-";
    }
    var f_x = parseFloat(decimal);
    var readable = ["", "万", "亿"];
    var index = 0;
    while (decimal > 10000 && index < 2) {
      decimal /= 10000;
      index ++;
    }
    f_x = Math.round(decimal*100)/100;
    var s_x = f_x.toString();
    var pos_decimal = s_x.indexOf('.');
    if (pos_decimal < 0) {
      pos_decimal = s_x.length;
      s_x += '.';
    }
    while (s_x.length <= pos_decimal + 2) {
      s_x += '0';
    }
    return s_x + readable[index];
  }

  function formatTime(ts, format, isFull) {
    if ( !ts ) {
      return "";
    }
    var months = {
      Jan: "01",
      Feb: "02",
      Mar: "03",
      Apr: "04",
      May: "05",
      Jun: "06",
      Jul: "07",
      Aug: "08",
      Sep: "09",
      Oct: "10",
      Nov: "11",
      Dec: "12"
    };

    var dateStr = _.isNumber(ts) ? new Date(ts).toString() : ts,
      arr = dateStr.split(" "),
      y, m, d;

    if ( (arr[2] + "").length == 1 ) {
      arr[2] = "0" + arr[2];
    }
    m = months[arr[1]];
    d = arr[2];

    if ( arr[3] && /\d{4}/.test(arr[3]) ) {
      y = arr[3];
    } else {
      y = arr[5];
    }
    if ( format ) {
      if ( format == "mm-dd" ) {
        return [m,d].join("-");
      } if ( format == "mm月dd日" ){
        return m + "月" + d + "日";
      } else {
        return [y, m, d].join("-");
      }
    } else {
      if ( isFull ) {
        return [y, m, d].join("-") + " " + arr[4];
      } else {
        return [y, m, d].join("-");
      }
    }
  }

  var dayTs = 24 * 60 * 60 * 1000;

  var CP = window.CanvasRenderingContext2D && CanvasRenderingContext2D.prototype;
  if (CP && CP.lineTo){
    CP.dashedLine = function(x,y,x2,y2,dashArray){
      if (!dashArray) dashArray=[10,5];
      if (dashLength==0) dashLength = 0.001; // Hack for Safari
      var dashCount = dashArray.length;
      this.moveTo(x, y);
      var dx = (x2-x), dy = (y2-y);
      var slope = dx ? dy/dx : 1e15;
      var distRemaining = Math.sqrt( dx*dx + dy*dy );
      var dashIndex=0, draw=true;
      while (distRemaining>=0.1){
        var dashLength = dashArray[dashIndex++%dashCount];
        if (dashLength > distRemaining) dashLength = distRemaining;
        var xStep = Math.sqrt( dashLength*dashLength / (1 + slope*slope) );
        if (dx<0) xStep = -xStep;
        x += xStep
        y += slope*xStep;
        this[draw ? 'lineTo' : 'moveTo'](x,y);
        distRemaining -= dashLength;
        draw = !draw;
      }
    }
  }


  function callBrick(parameters) {
    var iframe = document.createElement('iframe');
    iframe.setAttribute('src', 'js:' + encodeURIComponent(JSON.stringify(parameters)));
    document.documentElement.appendChild(iframe);
    iframe.parentNode.removeChild(iframe);
    iframe = null;
    return false;
  }

  function getStockInfo(stockid) {
    var specialHKStocks = {
        HKHSI: 1,
        HKHSF: 1,
        HKHSU: 1,
        HKHSP: 1,
        HKHSC: 1,
        HKVHSI: 1,
        HKHSCEI: 1,
        HKHSCCI: 1,
        HKGEM: 1,
        HKHKL: 1
      },
      USIndexes = {
        DJI30: 1,
        NASDAQ: 1,
        SP500: 1,
        ICS30: 1,
        SLR10: 1,
        TMT20: 1,
        HCP10: 1,
        EDU10: 1
      },
      bitcoins = {
        BTCNCNY: {
          money: "￥",
          market: "比特币（CNY）"
        },
        MTGOXAUD: {
          money: "AU$",
          market: "比特币（AUD）"
        },
        MTGOXCNY: {
          money: "￥",
          market: "比特币（CNY）"
        },
        MTGOXEUR: {
          money: "€",
          market: "比特币（EUR）"
        },
        MTGOXGBP: {
          money: "£",
          market: "比特币（GBP）"
        },
        MTGOXJPY: {
          money: "J￥",
          market: "比特币（JPY）"
        },
        MTGOXUSD: {
          money: "$",
          market: "比特币（USD）"
        },


        // 新增一些Bitcoins
        LOCALBTCAUD: {
          money: "AU$",
          market: "比特币（AUD）"
        },
        MTGOXCNY: {
          money: "￥",
          market: "比特币（CNY）"
        },
        BTCDEEUR: {
          money: "€",
          market: "比特币（EUR）"
        },
        BTCEEUR: {
          money: "€",
          market: "比特币（EUR）"
        },
        LOCALBTCGBP: {
          money: "£",
          market: "比特币（GBP）"
        },
        BTCEUSD: {
          money: "$",
          market: "比特币（USD）"
        },
        BITSTAMPUSD: {
          money: "$",
          market: "比特币（USD）"
        }
      },
      fundSymbolArray = ["SH500", "SH510", "SH511", "SH513", "SZ15", "SZ18", "SZ16"],
      stockInfo;

    if ( stockid && ( ~_.indexOf(fundSymbolArray, stockid.substr(0,5)) || ~_.indexOf(fundSymbolArray, stockid.substr(0,4))) ) {
      stockInfo = { money: "￥", market: "", bigType: "基金", type: "基金" };
    } else if ( /^S[HZ]\d+$/.test(stockid) ){

      // SH900||SZ200为HK$B股
      if ( /^SZ200/.test(stockid) ) {
        stockInfo = { money: "HK$", market: "深B", bigType: "沪深", type: "股票" };

        // 回购
      } else if ( /^(SH(201|202|203|204|131)|SZ(13|395032))/.test(stockid) ) {
        stockInfo = { money: "￥", market: "回购", bigType: "沪深", type: "回购" };

        // 国债
      } else if ( /^(SH(01|02|13)|SZ(10|09))/.test(stockid) ) {
        stockInfo = { money: "￥", market: "国债", bigType: "沪深", type: "国债" };

        // 企债
      } else if ( /^(SH12|SZ11)/.test(stockid) ) {
        stockInfo = { money: "￥", market: "企债", bigType: "沪深", type: "企债" };

        // 可转债
      } else if ( /^(SH11|SZ12)/i.test(stockid) ) {
        stockInfo = { money: "￥", market: "可转债", bigType: "沪深", type: "可转债" };

        // 沪B
      } else if ( /^SH900/.test(stockid) ) {
        stockInfo = { money: "$", market: "沪B", bigType: "沪深", type: "股票" };

        // 指数
      } else if ( /^(SH00|SZ399)/.test(stockid) ) {//指数
        stockInfo = { money: "", market: "", bigType: "沪深", type: "指数" };

        // 普通股票
      } else {
        stockInfo = { money: "￥", market: "A股", bigType: "沪深", type: "股票" };
      }

      // 货币基金
    } else if ( /^MF\d+$/.test(stockid) ) {
      stockInfo = { money: "￥", market: "MF", bigType: "货币基金", type: "货币基金" };

      // 信托产品
    } else if ( /^TP\d+$/.test(stockid) ) {
      stockInfo = { money: "￥", market: "TP", bigType: "信托产品", type: "信托产品" };

      // 国债期货
    } else if ( /^[\d\w]+\.FM$/.test(stockid) ) {
      stockInfo = { money: "￥", market: "FM", bigType: "国债期货", type: "国债期货" };

      // 私募基金
    } else if ( /^P\d+$/.test(stockid) ) {
      stockInfo = { money: "￥", market: "P", bigType: "私募基金", type: "私募基金" };

      // 理财产品
    } else if ( /^FP\d+$/.test(stockid) ) {
      stockInfo = { money: "￥", market: "FP", bigType: "理财产品", type: "理财产品" };

      // 港股
    } else if ( /^\d+$/.test(stockid) ) {
      stockInfo = { money: /^8/.test(stockid) ? "￥": "HK$", market: "港股", bigType: "港股", type: "股票" };

      // 指数
    } else if ( specialHKStocks[stockid] || USIndexes[stockid] ) {//指数
      stockInfo = { money: "", market: "", bigType: "指数", type: "指数" };

      // bitcoin 美股
    } else {
      var bitcoin = bitcoins[stockid];

      if ( bitcoin ) {
        stockInfo = { money: bitcoin.money, market: bitcoin.market, bigType: "比特币", type: "比特币" };
      } else {
        stockInfo = { money: "$", market: "美股", bigType: "美股", type: "股票" };
      }
    }
    return stockInfo;
  }

  var _getCanvas= function(w, h, ratio) {
    var can = document.createElement("canvas");
    if (!ratio) {
      ratio = PIXEL_RATIO;
    }

    can.width = w * ratio;
    can.height = h * ratio;
    can.id = Math.round(Math.random() * 100);
    can.style.width = w + "px";
    can.style.height = h + "px";
    can.getContext("2d").setTransform(ratio, 0, 0, ratio, 0, 0);
    return can;
  }

  // util functions 
  function fixCoord(coord){
    var intCoord = parseInt(coord);
    if ( PIXEL_RATIO === 1 ) {
      if ( coord < intCoord ) {
        return intCoord - 0.5;
      } else {
        return intCoord + 0.5;
      }
    } else {
      return coord;
    }
  }

  function _tsToDate(ts, format) {
    var d = new Date(ts);
    var dd = d.getDate();
    var mm = d.getMonth() + 1;
    dd = (dd >= 10? dd : '0' + dd);
    mm = (mm >= 10? mm : '0' + mm);
    yy = d.getFullYear();

    if (format && format == "mm-dd") {
      return mm + '-' + dd;
    } else {
      return yy + '-' + mm + '-' + dd;
    }
  };


  function _minmax2d(data) {
    var max = -Infinity;
    var min = Infinity;

    for(var i in data) {
      for (j in data[i]) {
        if (data[i][j] >= max)  max = data[i][j];
        if (data[i][j] < min) min = data[i][j];
      }
    }
    return [min, max];
  };

  function _minmax1d(data) {
    var max = -Infinity;
    var min = Infinity;

    for(var i in data) {
      if (data[i] >= max)  max = data[i];
      if (data[i] < min) min = data[i];
    }
    return [min, max];
  };

  function _drawline(ctx, x1, y1, x2, y2, color, width) {
    color = color || "#111111";
    var width = width || 1.0;

    var w = ctx.lineWidth;
    ctx.lineWidth = width;
    ctx.strokeStyle = color;
    ctx.beginPath();
    ctx.moveTo(fixCoord(x1), fixCoord(y1));
    ctx.lineTo(fixCoord(x2), fixCoord(y2));
    ctx.stroke();
    ctx.closePath();
    ctx.lineWidth = w;
  };

  function _drawlines(ctx, array, color, width) {
    color = color || "#111111";
    width = width || 1.0;

    var w = ctx.lineWidth,
      length = array.length;

    ctx.lineWidth = 0.001;
    /*
     *var my_gradient=ctx.createLinearGradient(0, 0, 0, 500);
     *my_gradient.addColorStop(0, "#rgba(0, 0, 255, 0.25)");
     *my_gradient.addColorStop(1, "#fff");
     */
    ctx.fillStyle = "rgba(21, 119, 202, 0.15)";
    ctx.strokeStyle = color;
    ctx.beginPath();

    for (var i = 0; i < length; i+=3) {
      var s = array[i],
        x = array[i + 1],
        y = array[i + 2];

      if (s === "M") {
        ctx.moveTo(x, y);
      } else if (s === "L") {
        ctx.lineTo(x, y);
      }
    }
    ctx.fill();
    ctx.closePath();
    ctx.stroke();
    ctx.lineWidth = 1;
  }

  var darkTheme = {
    dif: "#d20",
    dea: "#014375",
    macd: "red",
    background : '#222',
    border: "#888",
    borderColor: "#444",
    pselect: "#222",
    label: '#888',
    stroke: '#333',
    gridlines: '#AAAAAA',
    overlays : ['#CB2BC6','#5217DB','#18E197', '#DED71F','#DE521F', '#10F5B8', '#A6ACE2', '#DF9FB0'], //['#FF6600', '#FFFF33', '#FFFF33', '#00CCFF', '#3366FF'],
    bbands: ['#aabbcc', '#aabbcc', '#aabbcc'],
    //macd: ['#0000FF', '#FF0000', '#aabbcc'],
    psar: '#CCFFFF',
    rcandle: '#3a1',
    gcandle: '#d20',
    lineplot: '#CCCCCC',
    idcss: 'position:absolute; border: 2px solid #0066CC; background: #FFFFCC;font-size:10px;font-family:arial,sans-serif;text-align:center;width:80px;height:100px; padding:2px;',
    oloffset : 0,
    ma5:"#cd4343",
    ma10:"#8C3037",
    ma20:"#FFCA53",
    ma30:"#0055A2",
    stepLine: "#555"
  };

  var lightTheme = {
    dif: "#d20",
    dea: "#014375",
    macd: "red",
    background : '#fff',
    border: "#ccc",
    borderColor: "#ddd",
    pselect: "#f1f1f1",
    label: '#888',
    stroke: '#f0f0f0',
    gridlines: '#111111',
    overlays : ['#CB2BC6','#5217DB','#18E197', '#DED71F','#DE521F', '#10F5B8', '#A6ACE2', '#DF9FB0'], //['#FF6600', '#FFFF33', '#FFFF33', '#00CCFF', '#3366FF'],
    //overlays : ['#50124E','#250966','#075237','#3F3D08', '#3F1607', '#05523E', '#3F4158','#583F45'], //['#663300', '#FFCC00', '#000066', '#00CCFF', '#3366FF'],
    bbands: ['#001122', '#001122', '#001122'],
    //macd: ['#000088', '#880000', '#aabbcc'],
    psar: '#000099',
    rcandle: '#3a1',
    gcandle: '#d20',
    lineplot: '#333333',
    idcss: 'position:absolute; border: 2px solid #0066CC; background: #FFFFCC;font-size:10px;font-family:arial,sans-serif;text-align:center;width:80px;height:100px; padding:2px;',
    oloffset : 0,
    ma5:"#cd4343",
    ma10:"#8C3037",
    ma20:"#FFCA53",
    ma30:"#0055A2",
    stepLine: "#f5f5f5"
  };

  var stockChart = function(){


  }

  stockChart.prototype = {
    // property
    current: undefined,
    type: SNB.data.type,
    ctype: SNB.data.ctype,
    qtype: SNB.data.qtype, // quote的type类型  用来区分各种基金
    symbol: SNB.data.symbol,
    period: SNB.data.period,
    ktype: "normal",
    smallHeightRate: [0.2, 0.8],
    bigHeightRate: [0.1, 0.15, 0.75],
    labelWith: 0,
    window: {},
    chart: {},
    theme: darkTheme,
    // 点参数
    cp: {
      begin: undefined,
      end: undefined,
      minwidth: 6,
      type: 1
    },

    // methods
    init: function(target, options){
      if ( options.theme === "light" ) {
        this.theme = lightTheme;
      }

      this.type = options.type || this.type;

      // ctype 现在分三种 day 日线 kline k线 fund 基金净值
      this.ctype = options.ctype || this.ctype;
      this.qtype = options.qtype || this.qtype;
      this.symbol = options.symbol || this.symbol;
      this.period = options.period || this.period;
      this.isIPAD = options.isIPAD || this.isIPAD;
      this.access_token = options.access_token;
      this.indicator = "volume";

      if (_.isUndefined(this.ctype)) {
        if (_.indexOf(dayPeriods, this.period) > -1) {
          this.ctype = "day";
        } else if (_.indexOf(klinePeriods, this.period) > -1){
          this.ctype = "kline";
        } else {
          this.ctype = "fund";
        }
      }


      $("#container").addClass(this.type);
      $("body").addClass(options.theme);

      if ( !_.isUndefined(options.st_color) && options.st_color == 0) {
        $("#container").addClass("oppositeColor");
      }

      this.st_color = options.st_color;

      SNB.data.stockInfo = getStockInfo(this.symbol);
      this.hasPankou = false;

      var bigType = SNB.data.stockInfo.bigType;

      if ((bigType === "沪深" || bigType === "基金")
        && _.indexOf(noPankouSymbols, this.symbol) === -1) {
        this.hasPankou = true;
      }

      var that = this;
      this.target = document.getElementById(target);
      this.getWH(options);
      this.renderHtml();

      var width = this.chart.width,
        height = this.chart.height;

      this.canvas = _getCanvas(this.window.width, height);
      //this.canvas.transferControlToProxy();

      this.ctx = this.canvas.getContext("2d");
      this.ctx.fillStyle = this.theme.background;
      this.ctx.fillRect(0, 0, width, height);
      this.target.appendChild(this.canvas);

      if (this.type === "big") {
        this.$overlay = $("#overlay");

        this.$overlay.css({
          position: "absolute",
          left: "4px",
          top: "4px",
          width: width,
          height: height,
          "z-index": "2"
        });

        this.octx = {};

        if (options.danmu) {
          var url = 'http://assets.imedao.com/js/widget/mobile/dm.js?v=' + Date.now();
          var s = document.createElement('script');
          s.onload = function(){
            setTimeout(function(){
              _dm(that.symbol);
            }, 500);
            s.onload = null;
          };
          s.src = url;
          document.head.appendChild(s);
        }
      }

      if ( this.type === "big" ) {
        this.labelWith = 0;
        this.renderQuote();
      }
      this.chart.currentWidth = width - this.labelWith;
      this.chart.timeHeight = 25;
      if (this.type === "small") {
        this.chart.timeHeight = 20;
      }
      if (this.type === "big") {
        this.chart.currentHeight = (height - this.chart.timeHeight) * ( 3 / 4);
      } else {
        this.chart.currentHeight = (height - this.chart.timeHeight) * 0.6;
      }
      if (this.type === "small") {
        this.chart.currentHeight += 20;
      }
      this.chart.volumeHeight = height - this.chart.timeHeight - this.chart.currentHeight;

      if (this.type === "big") {
        this.bindEvent();
      }

      this.data = {};

      // k线分段去取  有个状态
      this.kdata = {
        "1day": {},
        "1week": {}
      };

      this.getData(function(data){
        that.drawChart(true);

        setTimeout(function () {
          that.drawChart();
        }, 50);
      });
    },

    initCanvas: function () {
      $(this.canvas).remove();
      var width = this.chart.width,
        height = this.chart.height;

      this.canvas = _getCanvas(this.window.width, height);
      //this.canvas.transferControlToProxy();

      this.ctx = this.canvas.getContext("2d");
      this.ctx.fillStyle = this.theme.background;
      this.ctx.fillRect(0, 0, width, height);
      this.target.appendChild(this.canvas);
    },

    /**
     * 浮层上面划线
     */
    drawLineToOverlay: function (x, pageY) {
      var $splitLineV = this.$overlay.find(".splitLineV");
      var $splitLineH = this.$overlay.find(".splitLineH");
      var $current = this.$overlay.find(".current");
      var $percent = this.$overlay.find(".percent");
      var y = pageY - 96;

      var current = ((this.cymax - this.cymin) / this.chart.currentHeight * (this.chart.currentHeight - y) + this.cymin).toFixed(2);

      $splitLineV.css({
        left: x,
        top: 0,
        display: "block"
      });

      $splitLineH.css({
        left: 0,
        top: y,
        display: "block"
      });

      $current.css({
        display: "block",
        top: y - 8
      }).html(current);

      if (this.ctype === "day") {
        var percent = ((current - this.base) / this.base * 100).toFixed(2) + "%";
        $percent.css({
          display: "block",
          top: y - 8
        }).html(percent);
      }
    },

    /**
     * 浮层上面写文字
     */
    drawTextToOverlay: function (x, y, html) {
      $(".text").remove();
      var $text = $('<div class="text"></div>');

      $text.html(html);

      $text.css({
        left: x,
        top: y
      });

      this.$overlay.append($text);
    },

    /**
     * 清理浮层
     */
    clearOverlay: function () {
      this.$overlay.find(".text").remove();
      this.$overlay.find(".splitLineH, .splitLineV, .current, .percent").hide();

      this.$overlay.show();
    },
    // render quote
    renderQuote: function() {
      var that = this;
      function render(quote) {
        if (!quote) {
          return;
        }
        that.lastClose = parseFloat(quote.last_close);

        var current = that.formatNum(parseFloat(quote.current)),
          percent = that.formatNum(parseFloat(quote.percentage)),
          volume = decimal_2(quote.volume),
          name = quote.name,
          time = formatTime(quote.time),
          cclass,
          currentHtml;

        if (percent > 0) {
          cclass = "up";
          percent = "+" + percent + "%";

        } else {
          percent += "%";
          cclass = "down";
        }

        currentHtml = current + " (" + percent + ")";

        $("#stock")
          .find(".name")
          .html(name);

        // 货币基金和 f开头的基金 就只显示个名字  后面涨跌幅成交量信息就不显示了
        if (!/^M?F\d+/.test(that.symbol)) {
          $("#stock")
            .find(".current")
            .html(currentHtml)
            .addClass(cclass);

          $("#stock")
            .find(".volume")
            .html(volume + "股");

          $("#stock")
            .find(".time")
            .html("时间：" + time);
        }

        // render stock
        var marginRight = (that.window.width - 10 - $("#stock").width()) / 4;
        $("#stock span").css({"margin-right": marginRight, "visibility": "visible"});
      }

      render(this.quote);

    },

    /**
     * 基金数据获取
     */
    getDataForFund: function (cb) {
      var url = "/fund/forchart/day.json",
        that = this;

      if (this.isMF) {
        url = "/money_fund/forchart/day.json";
      }

      url = "http://api.xueqiu.com" + url;

      $.ajax({
        url: url,
        data: {
          symbol: this.symbol,
          period: that.period === "unit5" ? "5d" : "6m",
          access_token: this.access_token
        },
        dataType: "jsonp",
        success: function (ret) {
          that.wrapDataForFund(ret.fund);
          cb();
        }
      });
    },

    /**
     * 股票数据获取
     */
    getDataForStock: function (cb, isRefresh) {
      var that = this;

      if (!isRefresh) {
        this.showLoading();
        this.cp.begin = undefined;
        this.cp.end = undefined;
        this.cp.candles = undefined;
      }


      function _getDataFromRemote(){
        if (that.isLoadingData) {
          return false;
        }
        var params = {
            symbol: that.symbol,
            period: that.period
          },
          url = "/stock/forchart/stocklist.json";

        if (that.period === "1d") {
          params.one_min = 1;
        }

        if (that.access_token) {
          params.access_token = that.access_token;
        }


        if (that.ctype === "kline") {
          params.type = that.ktype;
          url = "/stock/forchartk/stocklist.json";

          var now = Date.now(),
            begin;

          if (that.period === "1day") {
            begin = now - 365 * dayTs;
          } else if(that.period === "1week") {
            begin = now - 365 * dayTs * 4
          }

          if (that.period === "1day" || that.period === "1week") {
            var obj = that.kdata[that.period][that.ktype];

            if (!obj) {
              obj = {};
              that.kdata[that.period][that.ktype] = obj;
            }

            if (!obj.status) {
              params.begin = begin;
              obj.status = "begin";
            } else if (obj.status === "begin") {
              params.end = begin;
              obj.status = "end";
            }
          }
        }

        url = "http://stock.xueqiu.com" + url;

        that.isLoadingData = true;
        $.ajax({
          url: url,
          data: params,
          dataType: "jsonp",
          success: function(data) {
            that.isLoadingData = false;
            that.wrapData(data);
            cb && cb();
          },
          error: function(data) {
            //that.showState("无法加载数据", 100, 3000);
          },
          complete: function(){}
        });
      }

      if (this.data[this.period] && !isRefresh) {
        this.current = this.data[this.period];
        if (this.ctype === "kline") {
          this.current = this.current[this.ktype];
          if (this.current) {
            cb();
          } else {
            _getDataFromRemote();
          }
        } else {
          cb();
        }
      } else {
        _getDataFromRemote();
      }

    },

    fixType: function () {
      // 新增基金类型
      var fundTypes = [
        "13", // 封闭式基金
        "18", // 货币基金
        "23", // 开放基金
        "24" // 分级基金
      ];

      if (_.indexOf(fundTypes, this.quote.type) > -1) {
        this.isFund = true;

        if (/^M?F/.test(this.symbol)) {
          this.ctype = "fund";
        } else {
          if (this.type === "big") {
            $("#period div")
              .eq(1)
              .text("净值")
              .data("period", "unit");
          }
        }

        if (/^MF/.test(this.symbol)) {
          this.isMF = true;
        }
      }
    },

    /**
     * 统一数据获取接口
     * @param cb
     * @param isRefresh
     */
    getData: function(cb, isRefresh){
      var that = this;

      var fn = function (cb, isRefresh) {
        that.fixType();
        if (that.ctype === "day" || that.ctype === "kline") {
          that.getDataForStock(cb, isRefresh);

          if (that.ctype === "day" && that.hasPankou) {
            that.getPankou();
          }
        } else {
          that.getDataForFund(cb);
        }
      };

      // 现在要先去获取quote了  因为要拿到quote里面的type
      if (this.quote) {
        fn(cb, isRefresh);
      } else {
        this.showLoading();
        var params = {
            code: this.symbol
          },
          url = "/v4/stock/quote.json",
          that = this;

        if (this.access_token) {
          params.access_token = this.access_token;
        }

        url = "http://stock.xueqiu.com" + url;

        $.ajax({
          url: url,
          data: params,
          dataType: "jsonp",
          success: function(data) {
            var quote = data[that.symbol];
            that.quote = quote;
            that.lastClose = quote.last_close;
            that.renderQuote();
            fn(cb, isRefresh);
          },
          complete: function(){}
        });
      }
    },

    // 封装数据
    wrapData: function(data){
      if (this.ctype === "kline") {
        this.wrapDataForKline(data);
      } else if (this.ctype === "day") {
        this.wrapDataForSline(data);
      } else {
        this.wrapDataForFund(data);
      }
    },

    /**
     * 为基金封装数据
     * @param data
     */
    wrapDataForFund: function (data) {
      var ts = [];

      if (this.isMF) {
        var seven = [],
          million = [];

        _.each(data, function (fund) {
          seven.push(fund.sevendayYield);
          million.push(fund.milliongain);
          ts.push(fund.valueDate);
        });

        this.data["seven"] = {
          value: seven,
          ts: ts
        };

        this.data["million"] = {
          value: million,
          ts: ts
        };

        this.current = this.data[this.period];

      } else {
        var unit = [];

        _.each(data, function (fund) {
          unit.push(fund.unit_nav);
          ts.push(Date.parse(new Date(fund.nav_date)));
        });

        this.data[this.period] = this.current = {
          value: unit,
          ts: ts
        };

        this.current = this.data[this.period];
      }
    },
    // k线图
    wrapDataForKline: function(data){
      var ts = [],
        ohlcma = [],
        vol = [],
        p = [],
        ma = [],
        macd = [],
        obj = this.kdata[this.period] && this.kdata[this.period][this.ktype];

      // 分段取 k线
      if (obj) {

        if (obj.status === "begin") {
          obj.data = data;
        } else if (obj.status === "end") {

          var length = data.chartlist.length;
          this.cp.end += length;
          this.cp.begin += length;

          data.chartlist = data.chartlist.concat(obj.data.chartlist);
        }
      }

      _.each(data.chartlist, function(item, i){
        ts[i] = item.time;
        ohlcma[i] = [
          item.high,
          item.open,
          item.low,
          item.close,
          item.ma5,
          item.ma10,
          item.ma20,
          item.ma30
        ];
        macd[i] = [
          item.dif,
          item.dea,
          item.macd
        ];

        vol[i] = item.volume;
        p[i] = item.percent;
      });

      if (!this.data[this.period]) {
        this.data[this.period] = {};
      }

      this.data[this.period][this.ktype] = this.current = {
        ts: ts,
        ohlcma: ohlcma,
        vol: vol,
        p: p,
        macd: macd
      };

    },

    // 股价图
    wrapDataForSline: function(data){
      var ts = [],
        vol = [],
        avg = [],
        current = [];

      _.each(data.chartlist, function(d, i){
        ts[i] = Date.parse(new Date(d.time));
        vol[i] = d.volume;
        current[i] = d.current;
        avg[i] = d.avg_price;
      });

      this.data[this.period] = this.current = {
        ts: ts,
        vol: vol,
        current: current,
        avg: avg
      }
    },

    // draw chart base
    drawChart: function(isInit){
      if (this.ctype === "kline") {
        this.drawKlineChart();
        if (this.type === "big") {
          this.drawCurrentMa();
        }
      } else if (this.ctype === "day") {
        this.drawStockChart(isInit);
      } else {
        this.drawFundChart();
      }
    },

    _clear: function(ctx) {
      this.initCanvas();
//      ctx.fillStyle = this.theme.background;
//      ctx.fillRect(0, 0, this.chart.width, this.chart.height);
    },

    // 修复股票类型
    fixBigType: function(bigType) {
      if (bigType === "指数") {
        if (this.symbol.indexOf("HK") > -1) {
          bigType = "港股";
        } else {
          bigType = "美股";
        }
      } else if (bigType === "基金") {
        bigType = "沪深";
      }
      return bigType;
    },

    getStockLineHowmany: function(bigType){
      var dayPointsCount = {
          "沪深": 60 * 4 + 2,
          "基金": 60 * 4 + 2,
          "港股": 60 * 5.5 + 2,
          "美股": 60 * 6.5 + 1,
          "比特币": 60 * 24
        },
        pointsCount5d = {
          "美股": 200,
          "沪深": 130,
          "港股": 175,
          "比特币": 720
        },
        howmany;

      bigType = this.fixBigType(bigType);

      if (this.period === "1d") {
        howmany = dayPointsCount[bigType];
      } else if(this.period === "5d") {
        howmany = pointsCount5d[bigType];
      } else {
        howmany = this.current.current.length;
      }

      return howmany;
    },
    drawChartBorder: function(x, y, width, height, isVolume) {
      var ctx = this.ctx,
        x1 = x + width,
        y1 = y + height,
        color = this.theme.borderColor;

      //if (this.type === "big") {
      //  if (!isVolume) {
      //    _drawline(ctx, fixCoord(x), fixCoord(y), fixCoord(x1), fixCoord(y), color, 0.5);
      //  }
      //  _drawline(ctx, fixCoord(x1), fixCoord(y), fixCoord(x1), fixCoord(y1), color, 0.5);
      //}

      if (isVolume) {
        y1 -= 1;
      }
      _drawline(ctx, fixCoord(x), fixCoord(y), fixCoord(x1), fixCoord(y), color, 0.5);
      _drawline(ctx, fixCoord(x), fixCoord(y), fixCoord(x), fixCoord(y1), color, 0.5);

      _drawline(ctx, fixCoord(x1), fixCoord(y1), fixCoord(x1), fixCoord(y), color, 0.5);
      _drawline(ctx, fixCoord(x1), fixCoord(y1), fixCoord(x), fixCoord(y1), color, 0.5);
    },

    /**
     * 基金净值 七日年化 万份收益 线
     * @param isInit
     */
    drawFundChart: function (isInit) {
      this.hideLoading();
      var ctx = this.ctx;

      if (!isInit) {
        this._clear(ctx);
        ctx = this.ctx;
      }

      // 画时间线上方的边  这种情况下 时间放在最下边 没有volume 只有current
      var cwidth = this.chart.width,
        cheight = this.chart.currentHeight + this.chart.volumeHeight,
        cx = this.labelWith,
        cy = 0,
        that = this;

      this.drawChartBorder(cx, cy + 1, cwidth, cheight - 1);


      var current = this.current.value.slice();


      var howmany = current.length,
        perWidth = cwidth / (howmany - 1);

      this.cp.cwidth = perWidth;

      var cs = this.theme,
        cp = this.cp,
        ts = this.current.ts,
        // current
        d_ = _minmax1d(current),
        cmin = d_[0],
        cmax = d_[1],
        crange = cmax -cmin,
        cymin = parseFloat(cmin - (0.3 * crange)),
        cymax = parseFloat(cmax + (0.3 * crange)),
        cscale = cheight / (cymax - cymin),
        h = cheight;


      var _log = (cp.logscale ? Math.log : function(x) {return x});
      this.cymin = cymin;
      this.cymax = cymax;

      var ystops = this._ygrid(cymin, cymax, 3);

      _.each(ystops, function (ystop) {
        var logystp = _log(ystop);
        var y1 = Math.round((logystp - cymin) * cscale);
        if (y1 > 20 && h - y1 > 20) { // don't draw anything in first 20 pixels
          _drawline(ctx, fixCoord(cx), h - y1, fixCoord(cx + cwidth), h - y1, cs.stepLine, 0.5);
          label = parseFloat(logystp).toFixed(2);
          that._drawText(label, cwidth - 10, h - y1 - 3, {align: 'right', padding: 5});
        }
      });

      var lineArray = [],
        y0,
        lastX,
        lastY,
        pre,
        showTs,
        allTs;

      var firstTs = _.first(ts),
        lastTs = _.last(ts);

      allTs = lastTs - firstTs;

      for ( var i = 0; i < howmany; i++ ) {
        if ( i < current.length ) {
          if (current[i] == 0) {
            continue;
          }
          var ycurrent = Math.round((current[i] - cymin) * cscale),
            t = ts[i],
            xline = this.labelWith + i * perWidth;

          lastX = xline,
          lastY = h - ycurrent;

          if (!i) {
            lineArray.push("M");
            y0 = ycurrent;
            showTs = t;
          } else {
            lineArray.push("L");
          }

          lineArray.push(xline, h - ycurrent);

          // draw ts for 5d 1m 6m 3y
          var timeInterval = Math.round(allTs / 5);

          if (t >= showTs + timeInterval) {
            showTs = t;
            _drawline(ctx, xline, h , xline, 0, cs.stroke, 0.5);

            var format = that.period === "unit5" ? "mm-dd" : undefined,
              label = that._idxToDate(i, format);

            if (i < howmany - 1) {
              that._drawText(label, xline, h + (that.chart.timeHeight / 1.6), { align: "center" });
            }
          }

          if ( pre ) {
            _drawline(ctx, pre.xline, h - pre.ycurrent, xline, lastY, "#0055a2", 1);
          }

          pre = {
            ycurrent: ycurrent,
            xline: xline
          }
        }
      }

      // 保存下 用于行情点动画
      that.currentInfo = {
        lastX: lastX,
        lastY: lastY,
        perWidth: perWidth,
        cymin: cymin,
        cscale: cscale
      };

      var firstX = lineArray[1];
      lineArray.push("L", lastX, h, "L", firstX, h);
      _drawlines(ctx, lineArray, "#0055A2", 1);

    },

    // draw stock chart
    drawStockChart: function(isInit){
      this.hideLoading();
      var ctx = this.ctx;

      if (!isInit) {
        this._clear(ctx);
        ctx = this.ctx;
      }

      var cwidth = this.chart.currentWidth,
        cheight = this.chart.currentHeight,
        cx = this.labelWith,
        that = this,
        cy = 0;

      /*
       *ctx.strokeStyle = this.theme.stroke;
       *ctx.strokeRect(fixCoord(cx), fixCoord(cy), fixCoord(cwidth), fixCoord(cheight));
       */
      this.drawChartBorder(cx, cy + 1, cwidth, cheight - 1);

      var vwidth = this.chart.currentWidth,
        vheight = this.chart.volumeHeight,
        vx = this.labelWith,
        vy = cheight + this.chart.timeHeight;

      /*
       *ctx.strokeStyle = this.theme.stroke;
       *ctx.strokeRect(fixCoord(vx), fixCoord(vy) , fixCoord(vwidth), fixCoord(vheight));
       */

      this.drawChartBorder(vx, vy + 1, vwidth, vheight, true);

      var cp = this.cp,
        w = this.chart.width,
        stockType = SNB.data.stockInfo.bigType,
        howmany = this.getStockLineHowmany(stockType),
        perWidth = cwidth / (howmany - 1);

      this.cp.cwidth = perWidth;

      var current = this.current.current.slice();
      current = _.filter(current, function(a) {
        return a > 0;
      });

      // 把lastClose 添加进去对比
      if (this.lastClose && this.period === "1d") {
        current.push(this.lastClose);
      }

      var cs = this.theme,
        cp = this.cp,
        vol = this.current.vol,
        avg = this.current.avg,
        ts = this.current.ts;


      // current
      var d_ = _minmax1d(current),
        cmin = d_[0],
        cmax = d_[1];

      if (this.period == "1d" && SNB.data.stockInfo.bigType == "沪深" && this.lastClose) {
        var max = Math.max(Math.abs(cmin - this.lastClose), Math.abs(cmax - this.lastClose));
        cmax = this.lastClose + max;
        cmin = this.lastClose - max;
      }

      var crange = cmax -cmin,
        cymin = parseFloat(cmin - (0.1 * crange)),
        cymax = parseFloat(cmax + (0.1 * crange)),
        cscale = cheight / (cymax - cymin),
        h = cheight,

        // volume
        vd_ = _minmax1d(vol),
        vmin = vd_[0],
        vmax = vd_[1],
        vrange = vmax - vmin,
        vymin = 0,
        vymax = vmax * 1.1,
        vscale = vheight / (vymax),
        vh = vy + vheight;

      var _log = (cp.logscale ? Math.log : function(x) {return x});
      var _exp = (cp.logscale ? Math.exp : function(x) {return x});

      var base = (this.period == "1d" && this.lastClose) ? this.lastClose : _.first(this.current.current);
      this.base = base;
      this.cymin = cymin;
      this.cymax = cymax;

      var stepFixedCount = 2;

      if (this.symbol.indexOf("SH900") > -1) {
        stepFixedCount = 3;
      }

      var type = parseInt(this.quote.type);
      if(type === 25){
        //期权截取小数后4位
        stepFixedCount = 4;
      }else if(type === 32 || type === 13 || type === 18 || type === 23 || type ===24){
        //港股涡轮、基金截取小数后3位
        stepFixedCount = 3;
      }

      if (this.type === "big" && this.period != "1d") {
        var ySteps = this._ygrid(cymin, cymax, 3);
        var that = this;

        _.each(ySteps, function (step) {
          var y1 = Math.round((step - cymin) * cscale);
          // 上线边缘的20px 不划线
          if (y1 > 0 && h - y1 > 0) {

            var stepText = parseFloat(step).toFixed(stepFixedCount);
            var stepPercentText = ((step- base) / base * 100).toFixed(2) + "%";
            var maxColor = that.getColor(step - base);

            var y = h - y1 - 3;
            that._drawText(stepText, 0, y, {align: 'left', color: maxColor});
            that._drawText(stepPercentText, cwidth - 5, y, {align: 'right', color: maxColor});
            _drawline(ctx, fixCoord(cx), y, fixCoord(cx + cwidth), y, cs.stepLine, 0.5);

            //label = parseFloat(step).toFixed(2);
            //that._drawText(label, cwidth - 10, h - y1 - 3, {align: 'right', padding: 5});
          }
        });
      } else {
        if (base > 0) {
          var maxText = parseFloat(cmax).toFixed(stepFixedCount);
          var minText = parseFloat(cmin).toFixed(stepFixedCount);
          var maxPercentText = ((cmax- base) / base * 100).toFixed(2) + "%";
          var minPercentText = ((cmin- base) / base * 100).toFixed(2) + "%";

          var maxy = h - Math.round((cmax- cymin) * cscale);
          var miny = h - Math.round((cmin - cymin) * cscale);


          _drawline(ctx, fixCoord(cx), maxy, fixCoord(cx + cwidth), maxy, cs.stepLine, 0.5);
          _drawline(ctx, fixCoord(cx), miny, fixCoord(cx + cwidth), miny, cs.stepLine, 0.5);

          var maxColor = this.getColor(1);
          if (cmax == this.lastClose) {
            maxColor = cs.label;
          }
          this._drawText(maxText, 0, maxy + 12, {align: 'left', color: maxColor});
          this._drawText(maxPercentText, cwidth - 5, maxy + 12, {align: 'right', color: maxColor});

          var minColor = this.getColor(-1);
          if (cmin == this.lastClose) {
            minColor= cs.label;
          }

          this._drawText(minText, 0, miny - 3, {align: 'left', color: minColor});
          this._drawText(minPercentText, cwidth - 5, miny - 3, {align: 'right', color: minColor});
        }
      }

      var volstops = this._volgrid(_exp(vymax), 2);
      _.each(volstops, function(v, index){
        var y1 = Math.round(v[1] * vscale),
          color = "#ddd";

        if ( index < volstops.length -1 ) {
          label = v[0];
          that._drawText(label, cwidth - 10, vh - y1 - 3, {align:'right', padding:5});
          color = "#f5f5f5";
        }
        //_drawline(ctx, fixCoord(cx), vh - y1, fixCoord(cx + cwidth), vh - y1, color, 0.5);
      });

      // draw lastClose line
      if (this.period === "1d" && this.lastClose) {
        current = this.current.current;
        current = current.slice();
        if (that.flashCurrent) {
          current.push(that.flashCurrent);
        }

        var lc = parseFloat(that.lastClose).toFixed(2);
        ly = Math.round((lc- cymin) * cscale);

        ctx.lineWidth = 0.5;
        ctx.strokeStyle = "#d20";
        ctx.beginPath();

        ctx.dashedLine(fixCoord(cx), h - ly, fixCoord(cx + cwidth), h - ly, [3]);

        ctx.closePath();
        ctx.stroke();
        ctx.lineWidth = 1;
      }

      if ( that.period === "1d") {
        var timeIndex = {
          "美股": [0, 180, 390],
          "沪深": [0, 120, 241],
          "港股": [0, 150, 331],
          "比特币": [0]
        };

        var exchange = that.fixBigType(SNB.data.stockInfo.bigType),
          indexArray = timeIndex[exchange];

        _.each(indexArray, function(i){
          var label = "12:00",
            xline = that.labelWith + i * perWidth,
            align = "center";

          if ( exchange === "美股" ) {
            label = "12:30";
          }
          if ( exchange === "沪深" ) {
            label = "13:00";
          }
          if ( i === 0 ) {
            align = "left";
            label = "09:30";
          } else if ( i === howmany - 1 ) {
            align = "right";
            label = "16:00";
            if ( exchange === "沪深" ) {
              label = "15:00";
            }
          }
          _drawline(ctx, xline, h , xline, 0, cs.stepLine, 0.5);
          that._drawText(label, xline, h + (that.chart.timeHeight / 1.6), { align: align });
        });
      }

      var lineArray = [],
        y0,
        lastX,
        lastY,
        pre,
        lastAvgY,
        showTs,
        allTs;

      var firstTs = _.first(ts),
        lastTs = _.last(ts);

      allTs = lastTs - firstTs;

      for ( var i = 0; i < howmany; i++ ) {
        if ( i < current.length ) {
          if (current[i] == 0) {
            continue;
          }
          var ycurrent = Math.round((current[i] - cymin) * cscale),
            yavg = Math.round((avg[i] - cymin) * cscale),
            yvolume = Math.round((vol[i] - vymin) * vscale),
            t = ts[i],
            xline = this.labelWith + i * perWidth;

          if (!i) {
            lineArray.push("M");
            y0 = ycurrent;
            showTs = t;
          } else {
            lineArray.push("L");
          }

          if (i) {
            _drawline(ctx, lastX, h - lastAvgY, xline, h - yavg, "#f70", 1);
          }

          lineArray.push(xline, h - ycurrent);
          lastX = xline;
          lastY = h - ycurrent;
          lastAvgY = yavg;

          // drawts for 5d 1m 6m 3y
          if (that.period !== "1d") {
            var dayTs = 24 * 60 * 60 * 1000,
              timeInterval = dayTs,
              periods = ["1m", "6m", "3y", "all"];

            if (_.indexOf(periods, that.period) > -1) {
              timeInterval = Math.round(allTs / 5);
            }

            if (t >= showTs + timeInterval) {
              showTs = t;
              _drawline(ctx, xline, h , xline, 0, cs.stroke, 0.5);

              var format = that.period === "5d" ? "mm-dd" : undefined,
                label = that._idxToDate(i, format);

              that._drawText(label, xline, h + (that.chart.timeHeight / 1.6), { align: "center" });
            }
          }

          if ( pre ) {
            _drawline(ctx, pre.xline, h - pre.ycurrent, xline, lastY, "#0055a2", 1);

            /* try plotting the volume */
            if ( vol[i] ) {
              var yvol = vol[i] * vscale;
              _drawline(ctx, xline, vh - yvol, xline, vh - 0.5, "#B3B3B3", 1);
            }
          }

          pre = {
            ycurrent: ycurrent,
            xline: xline
          }
        }
      }

      // 保存下 用于行情点动画
      that.currentInfo = {
        lastX: lastX,
        lastY: lastY,
        perWidth: perWidth,
        cymin: cymin,
        cscale: cscale
      };

      var firstX = lineArray[1];
      lineArray.push("L", lastX, h, "L", firstX, h);
      _drawlines(ctx, lineArray, "#0055A2", 1);
    },

    // kline chart
    drawKlineChart: function(ctx, dataset, shift, minwidth){
      var time1 = Date.now();
      this.hideLoading();
      var ctx = this.ctx;
      this._clear(ctx);
      ctx = this.ctx;

      var cwidth = this.chart.width,
        cheight = this.chart.currentHeight,
        cx = this.labelWith,
        cy = 0;

      ctx.strokeStyle = this.theme.stroke;
      this.drawChartBorder(cx, cy + 1, cwidth - 1, cheight);

      var vwidth = this.chart.width - 1,
        vheight = this.chart.volumeHeight,
        vx = this.labelWith,
        vy = cheight + this.chart.timeHeight,
        vh = vy + vheight;

      ctx.strokeStyle = this.theme.stroke;
      this.drawChartBorder(vx, vy, cwidth - 1, vheight - 1, true);

      var cs = this.theme,
        cp = this.cp,
        data = this.current.ohlcma,
        ma = this.current.ma,
        vol = this.current.vol,
        ob = this._window(data, undefined, shift, minwidth),
        xmin = ob.xmin,
        ymin = ob.ymin,
        xmax = ob.xmax,
        ymax = ob.ymax,
        h = cheight;

      this.ob = ob;

      // We get top, bottom, right, left of the plot and draw a bounding
      // rectangle 

      // If the scale is log scale, we use, Math.log and Math.exp or else
      // we use identity function.
      // Max candles we'd be plotting would not be more than hundred or
      // so, hence calculating log and exp is not as expensive as it may
      // appear than doing it for entire data. 
      var _log = (cp.logscale ? Math.log : function(x) {return x});
      var _exp = (cp.logscale ? Math.exp : function(x) {return x});
      var c = cp.cwidth / 2; // center of first (and every) cdl 
      var csize = cp.cwidth / 1.2;
      var range = ymax - ymin;
      var scale = cheight / range; // scale: how much a unit takes on the plot

      /* and X and Y axis  FIXME:  get it right */
      var ystops = this._ygrid(_exp(ymin), _exp(ymax), 4);
      for(var i in ystops) {
        var logystp = _log(ystops[i]);
        var y1 = h - Math.round((logystp - cp.ymin) * scale);
        if(y1 > 20 && y1 < h) { // don't draw anything in first 20 pixels  
          _drawline(ctx, fixCoord(cx), y1, fixCoord(cx + cwidth), y1, cs.stepLine, 0.5);
          label = ystops[i];
          //this._drawText(label, cwidth - 10, y1, { align: 'right' });
        }
      };

      var obj = this.drawChartBottom(xmin, xmax, vheight, vh, ctx, _log, _exp);

      // draw time
      var howmany = xmax - xmin;
      var xstop = Math.floor(howmany / 3);

      // 有些股票月k太少 时间太多 容易重合
      if (howmany < 10) {
        xstop = xmax - 1;
      }

      for(var i = xmin; i < xmax; i++) {
        if ( i % xstop == 0 ) {
          var label = this._idxToDate(i);
          if (!label) {
            continue;
          }
          var xlo = (c + (i - xmin) * cp.cwidth) - (csize / 2) + this.labelWith;
          var xline = xlo + Math.round(csize/2);
          if(xlo > this.labelWith + 30 && xlo < cwidth - 60) {
            _drawline(ctx, xline, h , xline, 0, cs.stepLine, 0.5);
            ctx.fillStyle = cs.label;
            ctx.textAlign = "center";
            ctx.fillText(label, xlo, h + (this.chart.timeHeight / 1.6));
          }
        }
      }

      var preMa = undefined,
        preMacd = undefined,
        preClose = undefined;

      this.cymin = cp.ymin;
      this.cymax = cp.ymax;

      for(var i = xmin; i < xmax; i++) {
        var yhi= Math.round((_log(data[i][0]) - cp.ymin) * scale);
        var yop = Math.round((_log(data[i][1]) - cp.ymin) * scale);
        var ylo = Math.round((_log(data[i][2]) - cp.ymin) * scale);
        var ycl = Math.round((_log(data[i][3]) - cp.ymin) * scale);

        var yma5 = Math.round((_log(data[i][4]) - cp.ymin) * scale);
        var yma10 = Math.round((_log(data[i][5]) - cp.ymin) * scale);
        var yma20 = Math.round((_log(data[i][6]) - cp.ymin) * scale);
        var yma30 = Math.round((_log(data[i][7]) - cp.ymin) * scale);

        var xlo = (c + (i - xmin) * cp.cwidth) - (csize / 2) + this.labelWith;
        var xline = xlo + csize / 2;

        var isOppositeColor = !_.isUndefined(this.st_color) && this.st_color == 0;

        var cl = data[i][3];
        var op = data[i][1];
        var number = cl - op;

        ctx.fillStyle = this.getColor(number);

        if (number < 0) {
          var t = ycl;
          ycl = yop;
          yop = t;
        }

        if (number == 0 && preClose) {
          ctx.fillStyle = this.getColor(cl - preClose);
        }

        //ctx.fillRect( xlo, h - yop, csize, yop - ycl);
        //if (!(yop - ycl)) {
        //  ctx.fillRect(xlo, h - yop, csize, 1);
        //}

        _drawline(ctx, xline, h - yop, xline, h - ycl, ctx.fillStyle, csize);
        if (yop == ycl) {
            ctx.fillRect(xlo, h - yop, csize, 1);
        }

        _drawline(ctx, xline, h - yhi, xline, h - ylo, ctx.fillStyle, 1);

        // ma line
        if ( preMa ) {
          _drawline(ctx, preMa.xline, h-preMa.yma5, xline, h-yma5, cs.ma5, 1);
          _drawline(ctx, preMa.xline, h-preMa.yma10, xline, h-yma10, cs.ma10, 1);
          _drawline(ctx, preMa.xline, h-preMa.yma20, xline, h-yma20, cs.ma20, 1);
          _drawline(ctx, preMa.xline, h-preMa.yma30, xline, h-yma30, cs.ma30, 1);
        }

        preMa = {
          yma5: yma5,
          yma10: yma10,
          yma20: yma20,
          yma30: yma30,
          xline: xline
        };

        preClose = data[i][3];

        if (this.indicator === "volume") {
          if (vol[i]) {
            var vscale = obj,
              yvol = vol[i] * vscale;

            _drawline(ctx, xline, vh - yvol, xline, vh, ctx.fillStyle, csize);
          }
        } else if (this.indicator === "macd") {
          var macd = this.current.macd,
            macdScale = obj.macdScale,
            min = obj.minMacd,
            max = obj.maxMacd,
            ydif = Math.round((_log(macd[i][0]) - min) * macdScale),
            ydea = Math.round((_log(macd[i][1]) - min) * macdScale),
            ymacd = Math.round((_log(macd[i][2]) - min) * macdScale);

          if (preMacd) {

            // macd line
            var zeroY = vh - (vheight / 2),
              a = macd[i][2],
              color = a > 0 ? cs.gcandle : cs.rcandle;

            if (isOppositeColor) {
              color = a < 0 ? cs.gcandle : cs.rcandle;
            }

            _drawline(ctx, xline, vh - ymacd, xline, zeroY, color, 5);
            //ctx.fillRect( xlo, vh - yvol - 1, csize, yvol);
            _drawline(ctx, preMacd.xline, vh - preMacd.ydif, xline, vh - ydif, cs.dif, 1);
            _drawline(ctx, preMacd.xline, vh - preMacd.ydea, xline, vh - ydea, cs.dea, 1);
          }

          preMacd = {
            xline: xline,
            ydif: ydif,
            ydea: ydea,
            ymacd: ymacd
          };
        }
      }

      // 字要压住图
      var ystops = this._ygrid(_exp(ymin), _exp(ymax), 4);
      for(var i in ystops) {
        var logystp = _log(ystops[i]);
        var y1 = h - Math.round((logystp - cp.ymin) * scale);
        if(y1 > 20 && y1 < h) { // don't draw anything in first 20 pixels  
          //_drawline(ctx, fixCoord(cx), y1, fixCoord(cx + cwidth), y1, cs.stroke, 0.5);
          var label = ystops[i],
            labelStr = label + "";

          if (/\.\d{5}/.test(labelStr)) {
            labelStr = label.toFixed(2);
          }
          this._drawText(labelStr, cwidth - 10, y1 - 3, { align: 'right' });
        }
      };
      console.log("draw time", Date.now() - time1);

    },

    // 画成交量及各种指标的标线
    drawChartBottom: function(xmin, xmax, vheight, vh, ctx, _log, _exp) {
      var that = this,
        cx = this.labelWith,
        cs = this.theme,
        cwidth = this.chart.width - 1,
        vy = vh - vheight;

      if (this.indicator === "volume") {
        var vol = this.current.vol,
          vymax = _minmax1d(vol.slice(xmin, xmax))[1] * 1.1,
          vymin = 0,
          vrange = vymax - vymin,
          vscale = this.chart.volumeHeight / vrange,
          volstops = this._volgrid(_exp(vymax), 2);

        _.each(volstops, function(volstop, i) {
          var y1 = Math.round(volstop[1] * vscale),
            color = cs.stroke;

          if (i === volstops.length - 1) {
            color = "#ddd";
          }

          //_drawline(ctx, fixCoord(cx), fixCoord(vh - y1), fixCoord(cx + cwidth), fixCoord(vh - y1), color, 0.5);
          label = volstop[0];
          //that._drawText(label, cwidth - 10, vh - y1, { align: 'right' });
          that._drawText(label, cwidth - 10, vh - y1 - 2, { align: 'right', padding: 5 });
        });

        return vscale;
      } else if (this.indicator === "macd") {
        var macd = this.current.macd,
          d_ = _minmax2d(macd.slice(xmin, xmax)),
          min = d_[0],
          max = d_[1];
        maxABS = _.max([Math.abs(min), Math.abs(max)]),
          maxMacd = maxABS,
          minMacd = maxABS * -1,
          macdScale = vheight / (maxABS * 2);

        ctx.strokeRect(fixCoord(cx), fixCoord(vy) , fixCoord(cwidth), fixCoord(vheight));
        // 底部最小
        that._drawText(minMacd, cwidth - 10, vh - 2,  { align: 'right' });

        // 最大
        that._drawText(maxMacd, cwidth - 10, vy - 2,  { align: 'right' });

        var y = vy + vheight / 2;
        // 0
        that._drawText("0", cwidth - 10, y - 2,  { align: 'right' });
        _drawline(ctx, fixCoord(cx), y, fixCoord(cx + cwidth), y, cs.stroke);

        return { macdScale: macdScale, minMacd: minMacd, maxMacd: maxMacd };
      }
    },

    _idxToDate: function(i, format) {
      var ts = this.current.ts[i];
      if (_.isUndefined(ts)) {
        return "";
      }
      if( !_.isNumber(ts)) {
        ts = Date.parse(ts);
      }
      return _tsToDate(ts, format);
    },
    _drawText: function(txt, x, y, style, ctx) {
      var cs = this.theme;
      var ctx = ctx || this.ctx;
      var color = style.color || cs.label;
      var font = style.font || '8pt Helvetica Neue';
      var padding = style.padding || 2;
      var align = style.align || 'start';

      ctx.font = font;
      ctx.textAlign = align;
      var l = x + 2*padding + ctx.measureText(txt).width;
      ctx.fillStyle = "transparent";
      //FIXME : Fix for opera, check if it works 
      ctx.fillRect(x + padding - 1,y-15, l, 15);
      ctx.fillStyle = color;
      ctx.fillText(txt, x+padding, y);

      return l;
    },


    _volgrid: function(maxvol, howmany) {
      var labels = [];
      var lookup = [1000000000.0, 1000000.0, 1000.0, 1.0];
      var avols = [];
      for(var i = 0; i < howmany; i++) {
        avols.push(maxvol/(howmany-i));
      }
      var suffix = ['B', 'M', 'K', ''];
      for (var i in lookup) {
        if (avols[0]/lookup[i] > 1) {
          if ( labels.length < howmany ) {
            for(var j in avols ) {
              var n =  (avols[j] - avols[j]%lookup[i])/lookup[i] + suffix[i];
              labels.push([n, avols[j]]);
            }
          }
        }
      }
      return labels;
    },
    _ygrid: function(ymin, ymax, howmany) {
      var approx = (ymax - ymin)/howmany,
        lookup = [0.001, 0.0025, 0.005, 0.01, 0.025, 0.05, 0.1, 0.25, 0.5, 1.0, 2.5, 5.0, 10.0, 25.0, 50.0,
          100.0, 250.0, 500.0, 1000.0, 2500.0, 5000.0,
          10000.0, 25000.0, 50000.0, 100000.0, 500000.0, 1000000.0];

      var na = []
      for (i in lookup) {
        var b = lookup[i]/approx;
        if (b < 1.0) { b = 1/b; }
        na.push(b);
      }
      var closest = lookup[na.indexOf(Math.min.apply(this, na))];
      var minindex = Math.ceil(ymin / closest) - 1;
      var maxindex = Math.floor(ymax / closest) + 1;
      vals = [];
      for(var j = minindex; j <= maxindex; j++) {
        vals.push(j * closest);
      }

      return vals;
    },


    // 获取最大最小值
    _window : function(data, overlays, shift, minwidth) {

      if ( this.ctype === "day" ) {
        var dayPointsCount = {
            "沪深": 30 * 4 + 2,
            "港股": 30 * 5.5 + 2,
            "美股": 30 * 6.5 + 1,
            "比特币": 30 * 24
          },
          cp = this.cp,
          w = this.chart.width,
          stockType = this.fixBigType(SNB.data.stockInfo.bigType),
          howmany = dayPointsCount[stockType];

        cp.cwidth = w / howmany;

        var max, min;
        var d_ = _minmax2d(data);
        min = d_[0], max = d_[1];

        var range = max - min;
        cp.ymin = min - (0.05 * range); // little margin below
        cp.ymax = max + (0.05 * range); // little margin above 

        return { ymin: cp.ymin, ymax: cp.ymax};
      } else {
        /* right now only intializes chart params using data.*/
        var cp = this.cp,
          w = this.chart.width,
          begin,
          end,
          howmany = 0,
          minwidth = minwidth || cp.minwidth

        shift = shift || 0;
        cp.candles = cp.candles || data.length;
        howmany = data.length;
        if ( howmany > ( w / minwidth ) ) {
          howmany = Math.round(w / minwidth);
        }

        if (this.isPinch) {
          var pinchAllIndex = cp.begin + this.pinchIndex,
            newPinchIndex = Math.round((howmany * this.pinchIndex) / cp.candles);

          cp.begin = pinchAllIndex - newPinchIndex;
          cp.end = pinchAllIndex + (howmany - newPinchIndex);

          if (cp.end > data.length - 1) {
            cp.end = data.length - 1;
            cp.begin = cp.end - howmany;
          } else {
            this.pinchIndex = newPinchIndex;
          }

          if (cp.begin < 0) {
            cp.begin = 0;
          }
        } else {
          cp.end = cp.end || data.length; // first time we set it to end
          cp.end -= shift;
          if ( cp.end > data.length ) {
            cp.end = data.length;
          }
          cp.begin = cp.end - howmany;


          if ( cp.begin < 0 ) {
            cp.begin = 0;
            cp.end = cp.begin + howmany;
          }
        }
        begin = cp.begin;
        end = cp.end;
        cp.candles = howmany;
        cp.cwidth = w / cp.candles;

        if (!this.isPinch) {
          if ( cp.cwidth > 80 ) {
            cp.cwidth = 80;
          }
        }

        /* Y range is going to be dynamic get, min and max from data */
        var max, min;
        var d_ = _minmax2d(data.slice(begin, end));
        min = d_[0], max = d_[1];

        cp.begin = begin;
        cp.end = end;

        if ((max/min > 2.0) && cp.autoscale) {
          cp.logscale = true;
        }
        var range = max - min;
        cp.ymin = min - (0.05 * range); // little margin below
        cp.ymax = max + (0.05 * range); // little margin above 

        if(cp.logscale) {
          min = 0.9 * min;
          max = 1.1 * max;
          cp.ymin = Math.log(min);
          if(isNaN(cp.ymin)) {
            /* bollingers might go negative n stock split, for which Math.log is not defined.
             we set the min to 0 for now */
            cp.ymin = 0;
          }
          cp.ymax = Math.log(max);
        }
        return { xmin: begin, xmax: end, ymin: cp.ymin, ymax: cp.ymax};
      }
    },

    // 获取窗口大小
    getWH: function(options){
      var width = window.innerWidth > 0 ? window.innerWidth : screen.width,
        height = window.innerHeight > 0 ? window.innerHeight : screen.innerHeight,
        bodyWidth = $("body").width();

      this.window.width = bodyWidth < width ? bodyWidth : width;
      this.window.height = height;

      if (options && options.width && options.height) {
        this.window.width = options.width;
        this.window.height = options.height;
      }
    },

    // 渲染股票信息
    renderStockInfo: function(){

    },

    // 渲染时间选择
    renderHtml: function(options){
      // render period
      // 减20就好了  不知道为啥减30 正好
      var width = this.window.width,
        height= this.window.height,
        liWidth = width / 9,
        chartHeight = (height - 28);

      $(".currentPoint").hide();

      $("#container").addClass(this.type).css({
        height: this.window.height
      });

      if ( this.type === "big" ) {
        chartHeight = height - 101;
        $("#close").show();
      } else {
        chartHeight = height;
        liWidth = width / 5;
      }

      // 基金的话 要切换大图的日期
      if (/^M?F\d+/.test(this.symbol)) {
        if (/^MF/.test(this.symbol)) {
          $("#period").html('<div data-period="seven">七日年化</div> <div data-period="million">万份收益</div>');
        } else {
          $("#period").html('<div data-period="unit5">5日净值</div><div data-period="unit">累计净值</div>');
        }
      }

      $("#period div").css({"width": liWidth});
      $("#period").find("div[data-period='" + this.period + "']").addClass("select");
      if (this.type === "big") {
        $("#period").show();
      }

      this.chart.width = width - 8;
      this.chart.height = chartHeight - 8;


      if (SNB.data.stockInfo.bigType !== "沪深" || _.indexOf(noFuQuanSymbols, this.symbol) > -1) {
        $(".fuquan").remove();
      }

      if (this.type === "big" && this.ctype === "kline") {
        this.chart.width -= 67;
        $("#close").show();
        $("#chart-type").css({ height: chartHeight - 9 });
        if (this.ctype === "kline") {
          $("#chart-type").show();
          $("#zoom").show();
        }
      }

      if (this.ctype === "day") {
        if (this.period === "1d" && this.hasPankou) {
          this.chart.width -= 115;
          this.chart.currentWidth = this.chart.width;
        } else {
          this.chart.currentWidth = width - 8;
        }
      }

      if (this.ctype === "fund") {
        this.chart.currentWidth = width - 8;
      }

      if (this.period === "1d" && this.hasPankou) {
        $("#pankou").show();
      } else {
        $("#pankou").hide();
      }

      $("#chart").css({ height: chartHeight, width: this.chart.width }).show();
      if (this.type === "big" && this.$overlay) {
        var w = this.chart.width;
        if (this.ctype === "kline") {
          w += 8;
        }
        this.$overlay.css({
          width: w
        });
      }


      // loading top left
      var loadingTop = height / 2 - 12,
        loadingLeft =  width / 2 - 12;

      $("#loading").css({
        top: loadingTop,
        left: loadingLeft
      }).show();
    },
    adaptChartType: function() {
      this.chart.width = this.window.width;
      if (this.type === "big" && this.ctype === "kline") {
        this.chart.width -= 67;
      }

      $("#chart").css({ height: this.chart.height, width: this.chart.width }).show();
      if ( this.type === "big" && this.ctype === "kline" ) {
        $("#chart-type").show().css({ "visibility": "visible" });
        $("#chart-type").css({ height: this.chart.height });
        $("#zoom").show();
      } else {
        $("#zoom").hide();
        $("#chart-type").hide();
      }
    },

    // show loading
    showLoading: function(){
      $("#loading").show();
    },

    // hide loading
    hideLoading: function(){
      $("#loading").hide();
    },

    // 显示各种状态
    showState: function(str, width, timeout) {
      $("#loading")
        .css({ width: width })
        .html(str)
        .show();

      setTimeout(function() {
        $("#loading")
          .hide()
          .css({ width: 80 })
          .html("加载中...");

      }, timeout || 1000);
    },

    // 渲染图
    renderChart: function(){

    },

    // bind event
    bindEvent: function(){
      var that = this,
        eventName = "click";

      // 关闭
      $("#close").on(eventName, function(e) {
        callBrick("close");
      });

      // 打开
      if (this.type === "small") {
        $(".chartContainer").on(eventName, function(e){
          var params = {
            ctype: that.ctype,
            type: "big",
            symbol: that.symbol,
            period: that.period
          };

          callBrick("big");
          return false;
        });
      }

      function selectPeriod($target) {
        if ( $target.hasClass("select") || that.isLoadingData ) {
          return false;
        } else {
          $target.siblings().removeClass("select");
          $target.addClass("select");
        }

        that.period = $target.data("period");

        var dayPeriods = ["1d", "5d", "1m", "6m", "3y", "all"],
          smallPeriods = ["1d", "5d", "1month", "1week", "1day"];

        if (_.indexOf(smallPeriods, that.period) > -1) {
          callBrick(that.period);
        }

        if (_.indexOf(dayPeriods, that.period) > -1) {
          that.ctype = "day";
        } else if (_.indexOf(fundPeriods, that.period) > -1) {
          that.ctype = "fund";
        } else {
          that.ctype = "kline";
        }

        that.clearOverlay();
        //$("#chart-type").show();
        that.renderHtml();
        that.getData(function(data){
          that.adaptChartType();
          that.drawChart(data);
        });

      }

      if (that.type === "big") {
        var hammerContainer = new Hammer(document.getElementById("container"));

        hammerContainer.on("tap", function(event) {
          var pageX = event.pointers ? event.pointers[0].pageX : event.pageX,
            pageY = event.pointers ? event.pointers[0].pageY: event.pageY,
            beginY = that.window.height - 54,
            periodWidth = that.window.width / 9,
            index = Math.floor(pageX / periodWidth);

          if (pageY > beginY) {
            selectPeriod($("#period div").eq(index));
            return false;
          }
        });
      }

      var hammerPeriod = new Hammer(document.getElementById("period"));

      hammerPeriod.on("tap", function(e) {
        var target = e.target;
        if (target.tagName === "SPAN") {
          target = target.parentNode;
        }

        selectPeriod($(target));
      });

      $("#container .indicator, #container .fuquan").on(eventName, "li", function(e){
        if ($(this).hasClass("select")) {
          return false;
        }

        $(this).addClass("select");
        $(this).siblings().removeClass("select");

        that.clearOverlay();
        if ($(this).parent().hasClass("fuquan")) {
          that.ktype = $(this).data("type");
          that.getData(function(data){
            that.drawChart(data);
          });
        } else {
          that.indicator = $(this).data("type");
          that.getData(function(data){
            that.drawChart(data);
          });
        }
      });

      var hammerZoom = new Hammer(document.getElementById("zoom"));

      hammerZoom.on("tap", function(e) {
        var target = e.target;
        if ($(target).hasClass("zoom-out")) {
          that.cp.minwidth += 1;
          that.drawKlineChart();
        } else {
          if (that.cp.minwidth > 1) {
            that.cp.minwidth -= 1;
            that.drawKlineChart();
          }
        }
      })

      var eventStart = "mousedown",
        eventMove = "mousemove",
        eventEnd = "mouseup";

      var sto = undefined;
      /*
       *var hammerEl = Hammer(document.getElementById("chart"), {
       *  preventDefault: true,
       *  gesture: true
       *});
       */

      if (that.type === "small") {
        return false;
      }

      var hammerEl = new Hammer( document.getElementById("overlay"));

      hammerEl.get("press").set({
        time: 300
      });

      hammerEl.get("pinch").set({
        enable: true
      });

      hammerEl.on("panstart", function(e){
        if ( that.ctype === "day" || that.ctype === "fund" ) {
          return false;
        }

        var event = e,
          target = event.target;
        //this.style.cursor = 'move';
        var ctx = that.octx,
          pageX = event.pointers ? event.pointers[0].pageX : event.pageX;

        ctx.begin_x = that._canvasOffsetX(pageX, target);
        ctx.origin_x = that._canvasOffsetX(pageX, target);
        ctx.start = true;
        ctx.startts = Date.now();

        var obj = that.kdata[that.period] && that.kdata[that.period][that.ktype];
        if (obj) {
          if (obj.status === "begin") {
            setTimeout(function(e){
              that.getData(undefined, true);
            }, 500);
          }
        }
      });

      hammerEl.on("panmove", function(e){
        if ( sto ) {
          clearTimeout(sto);
        }

        var event = e,
          target = event.target;
        var ctx = that.octx,
          pageX = event.pointers ? event.pointers[0].pageX : event.pageX,
          pageY = event.pointers ? event.pointers[0].pageY : event.pageY;

        console.log(pageY);

        var speed = (pageX - ctx.origin_x) / (Date.now() - ctx.startts);

        var myx = that._canvasOffsetX(pageX, target);
        var xo = myx - ctx.begin_x;


        if ( ctx.isHold ) {
          that.showCurrentStockInfo(pageX, pageY);
        } else {
          if(ctx.start) {
            if (Math.abs(xo) > that.cp.minwidth) {
              size = Math.floor(xo / that.cp.minwidth);
              /*
               *              var ts = speed > 1 ? (5 - speed) * 20 : 0;
               *
               *              setTimeout(function(){
               *                that.drawKlineChart(that.ctx, that.current, size);
               *              }, ts);
               */

              that.drawKlineChart(that.ctx, that.current, size);
              ctx.begin_x = myx;
            }
          }
        }
      });
      hammerEl.on("panend", function(e){
        drawCounts = 0;
        that.octx.start = false;
        that.octx.isHold = false;
        //this.style.cursor = 'default';

        var ctx = that.octx,
          event = e.srcEvent,
          pageX = event.pointers ? event.pointers[0].pageX : event.pageX;

        $("#quote").hide();
        $("#stock").show();
        that.clearOverlay();
        if (that.ctype === "kline") {
          that.drawCurrentMa();
        }
      });

      hammerEl.on("press", function(e){
        if (that.type === "small") {
          return false;
        }
        sto = setTimeout(function(){
          that.octx.isHold = true;
          var event = e,
            pageX = event.pointers ? event.pointers[0].pageX : event.pageX,
            pageY = event.pointers ? event.pointers[0].pageY : event.pageY;
          // 出来一根线
          that.showCurrentStockInfo(pageX, pageY);
        }, 5);
      });

      hammerEl.on("pressup", function(e) {
        that.octx.isHold = false;
        $("#quote").hide();
        $("#stock").show();
        that.clearOverlay();
        if (that.ctype === "kline") {
          that.drawCurrentMa();
        }
      })

      var pinchTimes = 0,
        drawChartCounts = 0;

      // zoom + -
      hammerEl.on("pinchstart", function(e) {
        if (that.ctype === "kline") {
          that.isPinch = true;

          var pageX = e.pointers ? e.pointers[0].pageX : e.pageX;
          that.pinchIndex = Math.round(pageX / that.cp.cwidth);
        }
      });

      hammerEl.on("pinchmove", function(e) {
        if (that.ctype === "kline") {
          var scale = e.scale,
            minwidth = that.cp.minwidth * scale;

          pinchTimes ++;

          if (pinchTimes % 6 === 0) {
            drawChartCounts ++;
            that.clearOverlay();
            that.drawKlineChart(undefined, undefined, undefined, minwidth);
          }
        }
      });

      hammerEl.on("pinchend", function(e) {
        if (that.ctype === "kline") {
          drawChartCounts = 0;
          pinchTimes = 0;
          var scale = e.scale;
          that.cp.minwidth *= e.scale;
          that.drawKlineChart();

          that.isPinch = false;
        }
      });
    },

    showCurrentStockInfo: function(pageX, pageY){
      this.clearOverlay();
      var currentX = pageX,
        perWidth = this.cp.cwidth,
        quoteHtml;

      if (this.ctype === "day") {
        var i = Math.round(currentX / perWidth),
          length = this.current.current.length,
          xline = this.labelWith + i * perWidth,
          current = this.formatNum(this.current.current[i]),
          avg = this.formatNum(this.current.avg[i]),
          volume = this.current.vol[i],
          format = this.period === "1d" || this.period === "5d",
          time = formatTime(this.current.ts[i], undefined, format);


        if (i < length) {
          quoteHtml = ' '
          + '<span>当前价：' + current + '</span>'
          + '<span>成交量：' + volume + '</span>'
          + '<span>均价：' + avg + '</span>'
          + '<span>时间：' + time + '</span>';

          this.drawLineToOverlay(fixCoord(xline), fixCoord(pageY));
        }
      } else if (this.ctype === "kline") {
        var cp = this.cp,
          i = Math.ceil(currentX / this.cp.cwidth),
          c = Math.round(cp.cwidth / 2), // center of first (and every) cdl
          csize = Math.round(cp.cwidth/1.6),
          xmin = this.cp.begin,
          xlo = (c + (i - 1) * cp.cwidth) - (csize / 2) + this.labelWith,
          xline = xlo + Math.round(csize / 2),

          // data
          index = xmin + i - 1;

        this.clearOverlay();
        this.drawCurrentMa(index);
        if (this.indicator === "macd") {
          this.drawCurrentMACD(index);
        } else if (this.indicator === "volume") {
          this.drawCurrentVolume(index);
        }
        this.drawLineToOverlay(fixCoord(xline), fixCoord(pageY));
        //_drawline(this.octx, xline, this.chart.height, xline, 0, "#aaa", 0.5);
        quoteHtml = this.getCurrentQuoteHtml(index);
      } else {
        var periodName = {
          "seven": "七日年化收益率",
          "million": "万份收益",
          "unit": "净值",
          "unit5": "净值"
        };

        var i = Math.round(currentX / perWidth),
          length = this.current.value.length,
          xline = this.labelWith + i * perWidth,
          current = this.formatNum(this.current.value[i]),
          time = formatTime(this.current.ts[i]);


        if (i < length) {
          quoteHtml = ' '
          + '<span>' + periodName[this.period] + '：' + current + '</span>'
          + '<span>时间：' + time + '</span>';

          this.drawLineToOverlay(fixCoord(xline), fixCoord(pageY));
        }
      }

      $("#quote").html(quoteHtml);
      $("#stock").hide();
      $("#quote").show();
    },
    drawCurrentMa: function(index){
      if (_.isUndefined(index)) {
        index = this.current.ohlcma.length - 1;
      }

      var ohlcma = this.current.ohlcma[index],
        formatNum = this.formatNum,
        ma5 = "MA5：" + formatNum(ohlcma[4]),
        ma10 = "MA10：" + formatNum(ohlcma[5]),
        ma20 = "MA20：" + formatNum(ohlcma[6]),
        ma30 = "MA30：" + formatNum(ohlcma[7]),
        startX = 10;

      var maHtml = ' '
        + '<span style="color:' + this.theme.ma5 + '">' + ma5 + '</span>'
        + '<span style="color:' + this.theme.ma10 + '">' + ma10 + '</span>'
        + '<span style="color:' + this.theme.ma20 + '">' + ma20 + '</span>'
        + '<span style="color:' + this.theme.ma30 + '">' + ma30 + '</span>';

      this.drawTextToOverlay(10, 8, maHtml);
    },
    drawCurrentMACD: function(index) {
      var macd = this.current.macd[index];
        formatNum = this.formatNum,
        dif = "DIF：" + formatNum(macd[0]),
        dea = "DEA：" + formatNum(macd[1]),
        m = "MACD：" + formatNum(macd[2]),
        //maStr = "MA5: " + ohlcma[4] + " MA10: " + ohlcma[5] + " MA20: " + ohlcma[6],
        startX = 10,
        vy = this.chart.currentHeight + this.chart.timeHeight,
        y = vy + 15;

      var macdHtml = ' '
        + '<span style="color:' + this.theme.dif + '">' + dif + '</span>'
        + '<span style="color:' + this.theme.dea + '">' + dea + '</span>'
        + '<span style="color:' + this.theme.macd + '">' + m + '</span>';

      this.drawTextToOverlay(10, y - 10, macdHtml);
    },
    drawCurrentVolume: function(index) {
      var volume = this.current.vol[index];
      v = "成交量：" + this.formatNum(volume, undefined, 0),
        startX = 10,
        vy = this.chart.currentHeight + this.chart.timeHeight,
        y = vy + 15;

      var volumeHtml = '<span style="color: #aaa">' + v + '</span>';
      this.drawTextToOverlay(10, y - 10, volumeHtml);
    },
    formatNum: function(num, isPercent, count){
      var numStr = "";
      if (_.isNumber(num)) {
        if (count === 0) {
          numStr = parseInt(num);
        } else {
          numStr = num.toFixed(count || 2);
        }

        if (isPercent) {
          numStr += "%";
        }
      }
      return numStr;
    },
    getColor: function (number, defaultColor) {
      var cs = this.theme;
      var isOppositeColor = !_.isUndefined(this.st_color) && this.st_color == 0;
      var ltColor = isOppositeColor ? cs.gcandle : cs.rcandle;
      var gtColor = isOppositeColor ? cs.rcandle : cs.gcandle;

      var color = defaultColor || "#888";

      if (number > 0) {
        color = gtColor;
      } else if (number < 0) {
        color = ltColor;
      }
      return color;
    },
    getPankou: function () {
      var that = this;
      $.ajax({
        url: "http://stock.xueqiu.com/stock/pankou.json",
        data: {
          symbol: this.symbol,
          access_token: this.access_token
        },
        dataType: "jsonp",
        success: function (ret) {
          var sellLiArray = [];
          var buyLiArray = [];
          var i = 1;
          for (; i < 6; i++) {
            var lastClose = that.lastClose;
            var buyPrice = ret["bp" + i] || 0;
            var sellPrice = ret["sp" + i] || 0;
            var sellColor = that.getColor(sellPrice - lastClose);
            var buyColor = that.getColor(buyPrice - lastClose);

            var fixedCount = SNB.data.stockInfo.bigType === "基金" ? 3 : 2;
            if (that.quote && that.quote.type == 25) {
              fixedCount = 4;
            }

            if (SNB.data.stockInfo.market == "沪B") {
              fixedCount = 3;
            }

            var sellLi = '<li><span class="name">卖' + i +'</span>';
            sellLi += '<span class="price" style="color:' + sellColor + '">' + parseFloat(sellPrice).toFixed(fixedCount) + '</span>';
            sellLi += '<span class="volume">' + parseFloat(ret["sc" + i] || 0).toFixed(0) + '</span></li>';
            sellLiArray.push(sellLi);

            var buyLi = '<li><span class="name">买' + i +'</span>';
            buyLi += '<span class="price" style="color:' + buyColor + '">' + parseFloat(buyPrice).toFixed(fixedCount) + '</span>';
            buyLi += '<span class="volume">' + parseFloat(ret["bc" + i] || 0).toFixed(0) + '</span></li>';
            buyLiArray.push(buyLi);
          }

          sellLiArray.reverse();

          $("#sell").html(sellLiArray.join(""));
          $("#buy").html(buyLiArray.join(""));

          var lineHeight = that.chart.height / 10;
          $("#pankou li").css({
            "line-height": lineHeight + "px"
          });
        }
      });
    },
    getCurrentQuoteHtml: function(index){
      var preohlcma = this.current.ohlcma[index - 1],
        time = this._idxToDate(index, "yyyy-mm-dd"),
        formatNum = this.formatNum,
        openClass = "",
        highClass = "",
        lowClass = "",
        closeClass = "",
        chgClass = "",
        percentClass = "";

      if ( preohlcma ) {
        var ohlcma = this.current.ohlcma[index],
          lastClose = preohlcma[3];

        openClass = ohlcma[1] > lastClose ? "up" : "down",
          highClass = ohlcma[0] > lastClose ? "up" : "down",
          lowClass = ohlcma[2] > lastClose ? "up" : "down",
          closeClass = ohlcma[3] > lastClose ? "up" : "down",
          percentClass = this.current.p[index] > 0 ? "up" : "down";
      }

      var html = ' '
        + '<span>时间:'
        + '<span class="time">' + time + '</span>'
        + '</span>'
        + '<span>高:'
        + '<span class="' + highClass + '">' + formatNum(ohlcma[0]) + '</span>'
        + '</span>'
        + '<span>开:'
        + '<span class="' + openClass + '">' + formatNum(ohlcma[1]) + '</span>'
        + '</span>'
        + '<span>低:'
        + '<span class="' + lowClass + '">' + formatNum(ohlcma[2]) + '</span>'
        + '</span>'
        + '<span>收:'
        + '<span class="' + closeClass + '">' + formatNum(ohlcma[3]) + '</span>'
        + '</span>'
        + '<span>涨跌:'
        + '<span class="' + percentClass + '">' + formatNum(this.current.p[index], true) + '</span>'
        + '</span>';

      return html;
    },

    // 变更主题
    changeTheme: function(){

    },

    _canvasOffsetX: function(x, c) {
      var ox = 0;
      do {
        ox += c.offsetLeft;
      }while (c = c.offsetParent) ; // from quirksmode 
      return  x-ox;
    },
    _canvasOffsetY: function(y,c) {
      var oy = 0;
      do {
        oy += c.offsetTop;
      } while (c = c.offsetParent) ; // from quirksmode
      return y-oy;
    },

    // 刷新图
    refreshChart: function() {
      var that = this;

      // 2分钟取一次list
      setInterval(function() {
        var time = that.getTradeTime();
        if (that.ctype === "day" && that.period === "1d" && time.phase === "trade") {
          if (SNB.data.stockInfo.bigType == "港股") {
            return false;
          }
          that.getData(function(data){
            that.drawChart(data);
          }, true);
        }
      }, 60000);

      // 五档盘口
      setInterval(function() {
        var time = that.getTradeTime();
        if (that.ctype === "day" && that.hasPankou && time.phase === "trade") {
          that.getPankou();
        }
      }, 1000);
    },

    // 动点
    refreshCurrent: function (current, percentage) {
      var that = this;

      if (_.isUndefined(percentage) && this.lastClose) {
        percentage = (current - this.lastClose) / this.lastClose * 100;
      }
      this.flashCurrent = current;
      if (this.ctype === "day" && this.period === "1d") {
        this.drawStockChart();

        $(".currentPoint").css({
          left: this.currentInfo.lastX - 2.5,
          top: this.currentInfo.lastY - 2.5
        }).show();

        if (this.type === "big") {
          var text = parseFloat(current).toFixed(2) + " (" + parseFloat(percentage).toFixed(2) + "%)";
          var cclass = "";
          if (percentage > 0) {
            cclass = "up";
          } else if (percentage < 0) {
            cclass = "down";
          }

          $(".current")
            .removeClass("down up")
            .addClass(cclass)
            .text(text);
        }

        return false;
      }
    },
    drawLastClose: function(lastClose) {
      var that = this;

      this.lastClose = lastClose;
      return false;
      if (that.ctype === "day" && that.period === "1d") {
        that.getData(function(data){
          that.drawChart(data);
        });
      }
    },
    changePeriod: function(period) {
      var that = this,
        dayPeriods = ["1d", "5d", "1m", "6m", "3y", "all"],
        klinePeriods = ["1day", "1week", "1month"];

      this.period = period;

      if (_.indexOf(dayPeriods, that.period) > -1) {
        this.ctype = "day";
      } else if (_.indexOf(klinePeriods, that.period) > -1) {
        this.ctype = "kline";
      } else {
        this.ctype = "fund";
      }

      that.renderHtml();
      this.getData(function(data){
        that.drawChart(data);
      });
    },
    calculateTs: function (ts, specialTime) {
      var specialTimeStr = new Date(ts).toString();
      specialTimeStr = specialTimeStr.replace(/\d+:\d+:\d+/g, specialTime);
      return Date.parse(specialTimeStr);
    },
    getTradeTime: function () {
      var ts = Date.now(),
        date = new Date(),
        weekday = date.getDay(),
        marketTradeTime = {
          start: this.calculateTs(ts, "09:30:00"),
          end: this.calculateTs(ts, "16:05:00")
        };

      // 周六日直接除去
      if (weekday == 0 || weekday == 6) {
        return {phase: "end"};
      }

      if (ts > marketTradeTime.start &&  ts < marketTradeTime.end) {
        return {phase: "trade"};
      } else {
        if (ts < marketTradeTime.start) {
          // 获取距开盘的时间  setTimeout用
          return {phase: "before", ts: marketTradeTime.start - ts};
        } else {
          return {phase: "end"};
        }
      }
    }
  };

  window.stockChart = stockChart;
  var sc = new stockChart();
  /*
   *sc.init("chart", {
   *  theme: SNB.data.theme 
   *});
   */
})(window);

